package ubisec.discovery;


import java.util.*;

import javax.xml.namespace.QName;
import javax.wsdl.Definition;

import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.Logger;
import org.apache.log4j.Level;
import org.apache.log4j.SimpleLayout;


import ubisec.configuration.Setting;
import ubisec.discovery.message.*;
import ubisec.discovery.message.util.*;
import ubisec.profile.*;
import ubisec.discovery.tool.*;
import ubisec.util.*;
import ubisec.util.bloomfilter.*;
import ubisec.net.IPAddress;

/**
*
* Directory.java
*
* @author Rafik CHIBOUT 
* @author Francoise SAILHAN
* @version 0.1 sept. 2004
*  
*/

public class Directory
{
    private boolean deploy                    = false;
    private NodeProfile localProfile          = null;
    private Message messageProfileToSD        = null;
    private Message messageProfileToDirectory = null;
    private Message messageServicesResume     = null;
    private Hashtable WSRegistry;
    private Hashtable WSRemoteRepository;
    private Hashtable directories; 
    private BloomFilter2 servicesResume       = null;
    private boolean localResumeChanged        = false;
 
    private static int LISTENING_PERIOD=0;
    private static int ADVERTISEMENT_PERIOD=1; 
    private static int RESUME_EXCHANGE_PERIOD=2; 
    private static int COMMUNICATION_PORT=3;
    private static int DIRECTORY_ADVERTISEMENT_MSG_NUMBER=4;
    private static int DIRECTORY_GET_ALL_RESUMES_MSG_NUMBER=5;
    private static int GET_SERVICE_REQUEST_TO_DIRECTORY_MSG_NUMBER=6;
    private static int GET_SERVICE_RESPONSE_TO_SD_MSG_NUMBER=7;
    private static int DIRECTORY_TO_DIRECTORY_ADVERTISEMENT_MSG_NUMBER=8;
    private static int DIRECTORY_TO_DIRECTORY_ADVERTISEMENT_RESPONSE_MSG_NUMBER=9;
    private static int DIRECTORY_GET_ALL_RESUMES_RESPONSE_MSG_NUMBER=10;
    private static int DIRECTORY_RESUME_PUBLICATION_MSG_NUMBER=11;
    private static int WAITING_GET_SERVICE_RESPONSE_MIN=12;
    private static int WAITING_GET_SERVICE_RESPONSE_MAX=13;
    private static int INITIAL_TTL=14;    
    private static int TTL_UPDATING=15;
    private static int HOPS_NUMBER=16;
    private static int BLOOM_FILTER_ACTIVATION=17;
    private static int BLOOM_FILTER_SIZE=18;
    private static int BLOOM_FILTER_HASH_FUNCTION_NUMBER=19;
    private static int CACHE_ACTIVATION=20;
    private static int CACHE_SIZE=21;
    private static int QOS_ACTIVATION=22;
    private static int MAX_DIRECTORY_REQUESTED=23;
    private static int ADVERTISEMENT_HOPS = 24;

    private static int LISTENING_PERIOD_VALUE=5;
    private static int ADVERTISEMENT_PERIOD_VALUE=4000; 
    private static int RESUME_EXCHANGE_PERIOD_VALUE=20000; 
    private static int COMMUNICATION_PORT_VALUE=9999;
    private static int DIRECTORY_ADVERTISEMENT_MSG_NUMBER_VALUE=0;
    private static int DIRECTORY_GET_ALL_RESUMES_MSG_NUMBER_VALUE=0;
    private static int GET_SERVICE_REQUEST_TO_DIRECTORY_MSG_NUMBER_VALUE=0;
    private static int GET_SERVICE_RESPONSE_TO_SD_MSG_NUMBER_VALUE=0;
    private static int DIRECTORY_TO_DIRECTORY_ADVERTISEMENT_MSG_NUMBER_VALUE=0;
    private static int DIRECTORY_TO_DIRECTORY_ADVERTISEMENT_RESPONSE_MSG_NUMBER_VALUE=0;
    private static int DIRECTORY_GET_ALL_RESUMES_RESPONSE_MSG_NUMBER_VALUE=0;
    private static int DIRECTORY_RESUME_PUBLICATION_MSG_NUMBER_VALUE=0;
    private static int WAITING_GET_SERVICE_RESPONSE_MIN_VALUE=50;
    private static int WAITING_GET_SERVICE_RESPONSE_MAX_VALUE=10000;
    private static int INITIAL_TTL_VALUE=30000; 
    private static int TTL_UPDATING_VALUE=3000; 
    private static int INT_HOPS_NUMBER_VALUE=1;
    private static int BLOOM_FILTER_ACTIVATION_VALUE=1;
    private static int BLOOM_FILTER_SIZE_VALUE=2800;
    private static int BLOOM_FILTER_HASH_FUNCTION_NUMBER_VALUE=4;
    private static int CACHE_ACTIVATION_VALUE=1;
    private static int CACHE_SIZE_VALUE=100;
    private static int QOS_ACTIVATION_VALUE=0;
    private static int MAX_DIRECTORY_REQUESTED_VALUE=5;
    private static BitSet2 ADVERTISEMENT_HOPS_VALUE = ubisec.discovery.message.util.HOPS.N1;
    
    //static MessagesManager messagesManager = new MessagesManager(COMMUNICATION_PORT_VALUE);
    static MessagesManager messagesManager = null;
    private BitSet2 HOPS_NUMBER_VALUE=ubisec.discovery.message.util.HOPS.N1;
    
    
    private int[] requestsNumber = new int[100];
    private long daltaT = 30000; //  0.5 min
    private int requestsCounter =0; // requests number in daltaT/100
    private boolean requestCounterUpdateAutorized = true;
    
    private static Logger logger = Logger.getLogger(Directory.class); 
    private Level loggerLevel = Level.INFO;
        
    public Directory()
    {
    	SimpleLayout layout = new SimpleLayout();
    	ConsoleAppender appender = new ConsoleAppender(layout,ConsoleAppender.SYSTEM_OUT);
    	appender.setName("screen");
    	appender.activateOptions();
		logger.addAppender(appender);
    	logger.setLevel(loggerLevel);
    	
	   	if (Setting.HopsNumber>1 && Setting.HopsNumber<128)
	   	{
    		INT_HOPS_NUMBER_VALUE=Setting.HopsNumber;
    		BigBitSet tmp = new BigBitSet(INT_HOPS_NUMBER_VALUE);
    		HOPS_NUMBER_VALUE=tmp.getBitSet()[0];
	   	}
	   	if (Setting.MaxDirectoryRequested>0)
	   		MAX_DIRECTORY_REQUESTED_VALUE=Setting.MaxDirectoryRequested;
    	if (Setting.PortNumber>1024)
    		COMMUNICATION_PORT_VALUE=Setting.PortNumber;
    	if (Setting.BloomFilterActivation==false)
    		BLOOM_FILTER_ACTIVATION_VALUE=0;
    	if (Setting.BloomFilterSize>10)
    		BLOOM_FILTER_SIZE_VALUE=Setting.BloomFilterSize;
    	if (Setting.BloomFilterHashFunctionNumber>1)
    		BLOOM_FILTER_HASH_FUNCTION_NUMBER_VALUE=Setting.BloomFilterHashFunctionNumber;
    	if (Setting.CacheActivation==false)
    		CACHE_ACTIVATION_VALUE=0;
    	if (Setting.CacheSize>0 && Setting.CacheSize<=500)
    		CACHE_SIZE_VALUE=Setting.CacheSize;
    	if (Setting.QoSActivation==false)
    		QOS_ACTIVATION_VALUE=0;
    	    	
    	LISTENING_PERIOD_VALUE=Setting.DIR_LISTENING_PERIOD_VALUE;
    	ADVERTISEMENT_PERIOD_VALUE=Setting.DIR_ADVERTISEMENT_PERIOD_VALUE;
    	if (Setting.DIR_ADVERTISEMENT_HOPS_VALUE>0 && Setting.DIR_ADVERTISEMENT_HOPS_VALUE<=Setting.HopsNumber)
	   	{
    		INT_HOPS_NUMBER_VALUE=Setting.DIR_ADVERTISEMENT_HOPS_VALUE;
    		BigBitSet tmp = new BigBitSet(INT_HOPS_NUMBER_VALUE);
    		ADVERTISEMENT_HOPS_VALUE=tmp.getBitSet()[0];
	   	}
    	
    	RESUME_EXCHANGE_PERIOD_VALUE=Setting.DIR_RESUME_EXCHANGE_PERIOD_VALUE; 
    	WAITING_GET_SERVICE_RESPONSE_MIN_VALUE=Setting.DIR_WAITING_GET_SERVICE_RESPONSE_MIN_VALUE;
    	WAITING_GET_SERVICE_RESPONSE_MAX_VALUE=Setting.DIR_WAITING_GET_SERVICE_RESPONSE_MAX_VALUE;
    	INITIAL_TTL_VALUE=Setting.DIR_INITIAL_TTL_VALUE; 
    	TTL_UPDATING_VALUE=Setting.DIR_TTL_UPDATING_VALUE; 
    	WSRegistry = new Hashtable();
    	WSRemoteRepository = new Hashtable();
    	directories = new Hashtable();
    	for (int i=0; i<100; i++) requestsNumber[i]=0;
    }

    public void deploy(int ADVERTISEMENT_HOPS)
    {
    	if (ADVERTISEMENT_HOPS>0 && ADVERTISEMENT_HOPS <= 128)
	   	{
    		BigBitSet tmp = new BigBitSet(ADVERTISEMENT_HOPS);
    		ADVERTISEMENT_HOPS_VALUE=tmp.getBitSet()[0];
	   	}
    	deploy();
    }
    public void deploy()
    {
	//localProfile = new NodeProfile("neighbors",5555,5556);
	localProfile = SD.localProfile;
	localProfile.setMessageType("2");
	messagesManager = SD.manager;
	//messagesManager.clean();
	BitSet2[] toRemove = new BitSet2[3];
	toRemove[0]=MessageType.DIRECTORY_TO_DIRECTORY_ADVERTISEMENT;
	toRemove[1]=MessageType.DIRECTORY_GET_ALL_RESUMES;
	toRemove[2]=MessageType.DIRECTORY_RESUME_PUBLICATION;
	messagesManager.clean(toRemove);
	//servicesResume = new BloomFilter2(28 * 100000, 10);
	//servicesResume = new BloomFilter2(28 * 1000, 10);
	servicesResume = new BloomFilter2(BLOOM_FILTER_SIZE_VALUE, BLOOM_FILTER_HASH_FUNCTION_NUMBER_VALUE);
	createMessageLocalProfile();
	createServicesResumeMessage();
	deploy = true;
	Thread advertisement = directorySendAdvertisementProcess();
	Thread listening = directoryListenProcess();
	Thread resumeSend = directorySendResumeProcess();
	Thread ttlUpdate = repositoryUpdateProcess();
	Thread directroriesChecking = DirectroriesChecking();
	Thread servicePopularity =servicePopularityUpdate();
	// send resume directory request message to all known directories
	
	advertisement.start();
	findAllDirectoriesResumes();
	listening.start();
	resumeSend.start();
	ttlUpdate.start();
	directroriesChecking.start();
	servicePopularity.start();
	logger.info("Directory has been deployed, it cover neighbors over "+ADVERTISEMENT_HOPS_VALUE.getInt()+" hops");
	//resumeSend.start();
    }

    public void undeploy()
    {
	deploy = false;
	messageProfileToSD = null;
	messageProfileToDirectory = null;
	messageServicesResume = null;
	servicesResume = null;
	logger.info("Local Directory is undeployed");
    }

    public boolean isDeployed()
    {
    	return deploy;
    }
    
    public void setProperty(int propertyName, int propertyValue)
    {
    	switch (propertyName)
	    {
	    case 0: LISTENING_PERIOD_VALUE=propertyValue;break;
	    case 1: ADVERTISEMENT_PERIOD_VALUE=propertyValue;break;
	    case 2: RESUME_EXCHANGE_PERIOD_VALUE=propertyValue;break;
	    case 3: COMMUNICATION_PORT_VALUE=propertyValue;break;
	    case 4: DIRECTORY_ADVERTISEMENT_MSG_NUMBER_VALUE=propertyValue;break;
	    case 5: DIRECTORY_GET_ALL_RESUMES_MSG_NUMBER_VALUE=propertyValue;break;
	    case 6: GET_SERVICE_REQUEST_TO_DIRECTORY_MSG_NUMBER_VALUE=propertyValue;break;
	    case 7: GET_SERVICE_RESPONSE_TO_SD_MSG_NUMBER_VALUE=propertyValue;break;
	    case 8: DIRECTORY_TO_DIRECTORY_ADVERTISEMENT_MSG_NUMBER_VALUE=propertyValue;break;
	    case 9: DIRECTORY_TO_DIRECTORY_ADVERTISEMENT_RESPONSE_MSG_NUMBER_VALUE=propertyValue;break;
	    case 10: DIRECTORY_GET_ALL_RESUMES_RESPONSE_MSG_NUMBER_VALUE=propertyValue;break;
	    case 11: DIRECTORY_RESUME_PUBLICATION_MSG_NUMBER_VALUE=propertyValue;break;
	    case 12: WAITING_GET_SERVICE_RESPONSE_MIN_VALUE=propertyValue;break;
	    case 13: WAITING_GET_SERVICE_RESPONSE_MAX_VALUE=propertyValue;break;
	    case 14: INITIAL_TTL_VALUE=propertyValue;break;
	    case 15: TTL_UPDATING_VALUE=propertyValue;break;
	    }
    }

    public int getPropertyValue(int propertyName)
    {
	switch (propertyName)
	    {
	    case 0: return LISTENING_PERIOD_VALUE;
	    case 1: return ADVERTISEMENT_PERIOD_VALUE;
	    case 2: return RESUME_EXCHANGE_PERIOD_VALUE;
	    case 3: return COMMUNICATION_PORT_VALUE;
	    case 4: return DIRECTORY_ADVERTISEMENT_MSG_NUMBER_VALUE;
	    case 5: return DIRECTORY_GET_ALL_RESUMES_MSG_NUMBER_VALUE;
	    case 6: return GET_SERVICE_REQUEST_TO_DIRECTORY_MSG_NUMBER_VALUE;
	    case 7: return GET_SERVICE_RESPONSE_TO_SD_MSG_NUMBER_VALUE;
	    case 8: return DIRECTORY_TO_DIRECTORY_ADVERTISEMENT_MSG_NUMBER_VALUE;
	    case 9: return DIRECTORY_TO_DIRECTORY_ADVERTISEMENT_RESPONSE_MSG_NUMBER_VALUE;
	    case 10: return DIRECTORY_GET_ALL_RESUMES_RESPONSE_MSG_NUMBER_VALUE;
	    case 11: return DIRECTORY_RESUME_PUBLICATION_MSG_NUMBER_VALUE;
	    case 12: return WAITING_GET_SERVICE_RESPONSE_MIN_VALUE;
	    case 13: return WAITING_GET_SERVICE_RESPONSE_MAX_VALUE;
	    case 14: return INITIAL_TTL_VALUE;
	    case 15: return TTL_UPDATING_VALUE;
	    }
	return -1;
    }

    private void createMessageLocalProfile()
    {
	// message Profile to send To SD
	messageProfileToSD = MessageFactory.createMessage();
	messageProfileToSD.setMessageNumber(DIRECTORY_ADVERTISEMENT_MSG_NUMBER_VALUE);
	messageProfileToSD.setMessageType(MessageType.DIRECTORY_ADVERTISEMENT); 
	messageProfileToSD.setMessageSource(new IPAddress(localProfile.getIPAddress()));
	messageProfileToSD.setMessagePort(COMMUNICATION_PORT_VALUE);
	messageProfileToSD.setMessageHop(ADVERTISEMENT_HOPS_VALUE);
	messageProfileToSD.setMessageHopCounter(new BitSet2());
	messageProfileToSD.setMessageContent(localProfile.getBytes());
	// message Profile to send To Directory
	messageProfileToDirectory = MessageFactory.createMessage();
	messageProfileToDirectory.setMessageNumber(DIRECTORY_TO_DIRECTORY_ADVERTISEMENT_MSG_NUMBER_VALUE);
	messageProfileToDirectory.setMessageType(MessageType.DIRECTORY_TO_DIRECTORY_ADVERTISEMENT); 
	messageProfileToDirectory.setMessageSource(new IPAddress(localProfile.getIPAddress()));
	messageProfileToDirectory.setMessagePort(COMMUNICATION_PORT_VALUE);
	messageProfileToDirectory.setMessageHop(HOPS.N6);
	messageProfileToDirectory.setMessageHopCounter(new BitSet2());
	messageProfileToDirectory.setMessageContent(localProfile.getBytes());
    }

    private void createServicesResumeMessage()
    {
	messageServicesResume = MessageFactory.createMessage();
	messageServicesResume.setMessageNumber(DIRECTORY_RESUME_PUBLICATION_MSG_NUMBER_VALUE);
	messageServicesResume.setMessageType(MessageType.DIRECTORY_RESUME_PUBLICATION); 
	messageServicesResume.setMessageSource(new IPAddress(localProfile.getIPAddress()));
	messageServicesResume.setMessagePort(COMMUNICATION_PORT_VALUE);
	messageServicesResume.setMessageHop(HOPS_NUMBER_VALUE);
	messageServicesResume.setMessageHopCounter(new BitSet2());
	messageServicesResume.setMessageContent(servicesResume.getBloomFilterBytes());
    }
    
    
    private Message createAllResumesMessage()
    {
    	Message forReturn = MessageFactory.createMessage();
    	forReturn.setMessageNumber(DIRECTORY_GET_ALL_RESUMES_RESPONSE_MSG_NUMBER_VALUE);
    	forReturn.setMessageType(MessageType.DIRECTORY_GET_ALL_RESUMES_RESPONSE); 
    	forReturn.setMessageSource(new IPAddress(localProfile.getIPAddress()));
    	forReturn.setMessagePort(COMMUNICATION_PORT_VALUE);
    	forReturn.setMessageHop(HOPS_NUMBER_VALUE);
    	forReturn.setMessageHopCounter(new BitSet2());
    	// create the content
    	int totalLength = 0;
    	// calculate the totallength of content (all resumes)
    	totalLength = localProfile.getBytes().length+servicesResume.getBloomFilterBytes().length+8;
    	Hashtable clonedDirectories = (Hashtable)directories.clone();
    	Enumeration directoriesEnumeration = clonedDirectories.elements();
    	while (directoriesEnumeration.hasMoreElements())
    	    {
    		DirectoryResume directory = (DirectoryResume)directoriesEnumeration.nextElement();
    		
    		if (directory.status() == 1)
    			totalLength = totalLength+directory.getDirectoryResumeBytes().length+directory.getDirectoryProfile().getBytes().length+8;
    		// bloom + bloom size + profile + profile size
    	    }
    	
    	byte[] content = new byte[totalLength];
    	int pos=0;
    	//set local profile and resume of local directory
    	byte[] tmp = localProfile.getBytes();
		BigBitSet bigBitSet = new BigBitSet(tmp.length);
		BitSet2[] bitSets = bigBitSet.getBitSet();
	    content[pos]=bitSets[0].getByte();pos++;
	    content[pos]=bitSets[1].getByte();pos++;
	    content[pos]=bitSets[2].getByte();pos++;
	    content[pos]=bitSets[3].getByte();pos++;
		for (int i=0;i<tmp.length;i++)
	    {
	    	content[pos]=tmp[i]; pos++;
	    }
		
		tmp = servicesResume.getBloomFilterBytes();
		bigBitSet = new BigBitSet(tmp.length);
	    bitSets = bigBitSet.getBitSet();
	    content[pos]=bitSets[0].getByte();pos++;
	    content[pos]=bitSets[1].getByte();pos++;
	    content[pos]=bitSets[2].getByte();pos++;
	    content[pos]=bitSets[3].getByte();pos++;
		for (int i=0;i<tmp.length;i++)
	    {
	    	content[pos]=tmp[i]; pos++;
	    }
    	// add the others directories resumes    	
    	directoriesEnumeration = clonedDirectories.elements();
    	while (directoriesEnumeration.hasMoreElements())
    	    {
    		DirectoryResume directory = (DirectoryResume)directoriesEnumeration.nextElement();
    		if (directory.status()==1)
    		{
    		tmp = directory.getDirectoryProfile().getBytes();
    		bigBitSet = new BigBitSet(tmp.length);
		    bitSets = bigBitSet.getBitSet();
		    content[pos]=bitSets[0].getByte();pos++;
		    content[pos]=bitSets[1].getByte();pos++;
		    content[pos]=bitSets[2].getByte();pos++;
		    content[pos]=bitSets[3].getByte();pos++;
    		for (int i=0;i<tmp.length;i++)
		    {
		    	content[pos]=tmp[i]; pos++;
		    }
    		tmp = directory.getDirectoryResumeBytes();
    		bigBitSet = new BigBitSet(tmp.length);
    		bitSets = bigBitSet.getBitSet();
		    content[pos]=bitSets[0].getByte();pos++;
		    content[pos]=bitSets[1].getByte();pos++;
		    content[pos]=bitSets[2].getByte();pos++;
		    content[pos]=bitSets[3].getByte();pos++;
    		for (int i=0;i<tmp.length;i++)
		    {
		    	content[pos]=tmp[i]; pos++;
		    }
    		}
    		}
    	forReturn.setMessageContent(content);
    	return forReturn;
    }

    private void findAllDirectoriesResumes()
    {
    messagesManager.clean(new BitSet2[]{MessageType.DIRECTORY_GET_ALL_RESUMES_RESPONSE});
	// send to all Directories an advertisement 
	//messageProfileToSD.setMessageNumber(DIRECTORY_TO_DIRECTORY_ADVERTISEMENT_MSG_NUMBER_VALUE);
	//messagesManager.broadcastMessage(messageProfileToDirectory, COMMUNICATION_PORT_VALUE);
	//DIRECTORY_TO_DIRECTORY_ADVERTISEMENT_MSG_NUMBER_VALUE++;

    	// listen to Directories advertisements
    	Message[] messages = messagesManager.getMessagesType(MessageType.DIRECTORY_TO_DIRECTORY_ADVERTISEMENT);
    	if (messages!=null && messages.length>0) directoryAdvertisement(messages);
    	else 
    	{
    		// wait advertisement response from Directories
    		try 
			{	int counter =0;
    			while (counter<3*ADVERTISEMENT_PERIOD_VALUE && directories.size()==0)
    			{   
    				Thread.sleep(50);
    				counter = counter +50;
    				messages = messagesManager.getMessagesType(MessageType.DIRECTORY_TO_DIRECTORY_ADVERTISEMENT);
    				if (messages!=null && messages.length>0) directoryAdvertisement(messages);
    			}
    			//Thread.sleep(3*ADVERTISEMENT_PERIOD_VALUE);
			}
    		catch(Exception ex)
			{
    			logger.error("at waiting for directories advertisements");
			}
    	
    		//ask resume for all directories which sent back avdertisement response
    		messages = messagesManager.getMessagesType(MessageType.DIRECTORY_TO_DIRECTORY_ADVERTISEMENT);
    		directoryAdvertisement(messages);
    	}
	
    	Enumeration iPAddresses = directories.keys();
    	messageServicesResume.setMessageType(MessageType.DIRECTORY_GET_ALL_RESUMES);
    	messageServicesResume.setMessageNumber(DIRECTORY_GET_ALL_RESUMES_MSG_NUMBER_VALUE);
    	if (iPAddresses.hasMoreElements())
	    {
			String iPAddress =(String)iPAddresses.nextElement();
			messagesManager.sendInstantaneousMessageTo(messageServicesResume, iPAddress,COMMUNICATION_PORT_VALUE);
			logger.debug("DIRECTORY_GET_ALL_RESUME is sent to "+iPAddress);
	    }
    	else 
    	{
    		messageServicesResume.setMessageHop(HOPS.N6);
    	    messagesManager.instantaneousBroadcastMessage(messageServicesResume,COMMUNICATION_PORT_VALUE);
    	    logger.debug("DIRECTORY_GET_ALL_RESUME is sent to ALL");
    	}
	DIRECTORY_GET_ALL_RESUMES_MSG_NUMBER_VALUE++;
    }

    public void register(String wsdlDocument, String endPoint, String wsdlCustLocal, String wsdlCustRemote)
    {
	if (!deploy)
	    {
		logger.warn("Direcroty not yet deployed");
		//return ;
	    }
	try
	    {
		String QoS="";
		if (wsdlDocument.indexOf("<ServiceDiscovery>") !=-1 && wsdlDocument.indexOf("</ServiceDiscovery>")!=-1)
			QoS = wsdlDocument.substring(wsdlDocument.indexOf("<ServiceDiscovery>"), wsdlDocument.indexOf("</ServiceDiscovery>")+19);
		logger.debug("Register:QoS="+QoS);
		QoSParameters QoSParam = new QoSParameters(QoS);
			
		wsdlDocument = wsdlDocument.substring(QoS.length());
		while (wsdlDocument.charAt(0)=='\n' || wsdlDocument.charAt(0)=='\r')
			wsdlDocument = wsdlDocument.substring(1);
		
		Definition def = WSDLTools.readWSDL(wsdlDocument);
		QName defQName = def.getQName();

		WSRegistryEntry entry = new WSRegistryEntry(defQName,def,wsdlDocument, endPoint);
		entry.setQoSParameters(QoSParam);
		entry.setWSDLCustLocal(wsdlCustLocal);
		entry.setWSDLCustRemote(wsdlCustRemote);
		WSRegistry.put((new Integer(WSRegistry.size())),entry);
		logger.info("service "+defQName+" is successfully registred");
		addServiceToResume(wsdlDocument);
		messageServicesResume.setMessageContent(servicesResume.getBloomFilterBytes());
		localResumeChanged=true;
	    }
	catch (Exception ex)
	    {
			logger.error("at Directory.register()");
	    }
    }
	

    private void registerIntoRemoteRepository(String wsdlDocument, String endPoint, String wsdlCustLocal, String wsdlCustRemote, String custLocalEP, String CustRemoteEP)
    {
	if (!deploy)
	    {
			logger.warn("Directory not yet deployed");
			//return ;
	    }
	
	if (WSRemoteRepository.size()==CACHE_SIZE_VALUE)
	{
		// 	must to remove an entry before add this new service into the cache or leave
		Integer entryKey = getSmallPopularityEntry();
		WSCacheEntry entry = (WSCacheEntry)WSRemoteRepository.get(entryKey);
		//if (entry.getPopularity()>=1) return;
		if (System.currentTimeMillis()-entry.getLastAccessTime()<=daltaT) return;
		logger.debug("The service "+entry.getEndPoint()+" is removed from remote repository it will be replaced by "+endPoint);
		WSRemoteRepository.remove(entryKey);
	}
		
	try
	    {
		String QoS="";
		if (wsdlDocument.indexOf("<ServiceDiscovery>") !=-1 && wsdlDocument.indexOf("</ServiceDiscovery>")!=-1)
			QoS = wsdlDocument.substring(wsdlDocument.indexOf("<ServiceDiscovery>"), wsdlDocument.indexOf("</ServiceDiscovery>")+19);
		logger.debug("RegisterIntoRemoteRepository:QoS="+QoS);
		
		QoSParameters QoSParam = new QoSParameters(QoS);
		
		wsdlDocument = wsdlDocument.substring(QoS.length());
		while (wsdlDocument.charAt(0)=='\n' || wsdlDocument.charAt(0)=='\r')
			wsdlDocument = wsdlDocument.substring(1);
		
		
		Definition def = WSDLTools.readWSDL(wsdlDocument);
		QName defQName = def.getQName();
		WSCacheEntry entry = new WSCacheEntry(defQName,def,wsdlDocument, endPoint);
		entry.addAccessTimes(System.currentTimeMillis());
		entry.setQoSParameters(QoSParam);
		entry.setWSDLCustLocal(wsdlCustLocal);
		entry.setCustLocalEP(custLocalEP);
		entry.setWSDLCustRemote(wsdlCustRemote);
		entry.setCustRemoteEP(CustRemoteEP);
		entry.incrementPopularity();
		WSRemoteRepository.put((new Integer(WSRemoteRepository.size())),entry);
		//System.out.println("servicee "+defQName+" is registred");
	    }
	catch (Exception ex)
	    {
		logger.error("at Directory.registerIntoRemoteRepository()");
	    }
    }

    private Integer getSmallPopularityEntry()
    {
    	WSRegistryEntry smallPopularityEntry = null;
    	Integer smallPopularityEntryKey = null;
    	Enumeration registryEnumerationKeys = WSRemoteRepository.keys();
    	if (registryEnumerationKeys.hasMoreElements())
    	{
    		smallPopularityEntryKey = (Integer)registryEnumerationKeys.nextElement();
    		smallPopularityEntry = (WSRegistryEntry)WSRemoteRepository.get(smallPopularityEntryKey);
    	}
    	
    	while (registryEnumerationKeys.hasMoreElements())
    	    {
    		Integer key = (Integer)registryEnumerationKeys.nextElement();
    		WSRegistryEntry entry = (WSRegistryEntry)WSRemoteRepository.get(key);
    		if (entry.getPopularity()<smallPopularityEntry.getPopularity())
    		{
    			smallPopularityEntryKey = key;
    			smallPopularityEntry = entry;
    		}
    		  
    	    }
    	return smallPopularityEntryKey;
    }
    
    private void registerIntoRemoteRepository(GetServiceData[] services)
    {
	if (services !=null)
	    for (int i=0;i<services.length;i++)
		{
		    registerIntoRemoteRepository(services[i].getWSDLDocument(), services[i].getServiceEP(), services[i].getWSDLCustLocal(), services[i].getWSDLCustRemote(), services[i].getCustLocalEP(), services[i].getCustRemoteEP());
		}
    }

    private void addServiceToResume(String wsdlDocument)
    {
    String[] resumes = WSDLTools.getResume(wsdlDocument);
	for (int i=0; i<resumes.length;i++) 
	    servicesResume.addToFilter(resumes[i]);
    }

    private void register(byte[] data)
    {

	try
	    {
		BitSet2[] length = new BitSet2[4];
		length[0]= new BitSet2(data[0]);
		length[1]= new BitSet2(data[1]);
		length[2]= new BitSet2(data[2]);
		length[3]= new BitSet2(data[3]);
		BigBitSet length2 = new BigBitSet(length);
		int EPLength= length2.getInt();
		byte[] endPoint = new byte[EPLength];
		for (int i=0;i<EPLength;i++) 
		    endPoint[i] = data[4+i];
		String endPointStr = new String(endPoint);
		int index = 4+EPLength;
		
		
		length = new BitSet2[4];
		length[0]= new BitSet2(data[index]);index++;
		length[1]= new BitSet2(data[index]);index++;
		length[2]= new BitSet2(data[index]);index++;
		length[3]= new BitSet2(data[index]);index++;
		length2 = new BigBitSet(length);
		int WSDLLength= length2.getInt();
		byte[] wsdl = new byte[WSDLLength];
		for (int i=0;i<WSDLLength;i++)
		{
		    wsdl[i] = data[index];
		    index++;
		}
		String wsdlStr = new String(wsdl);
				
		length = new BitSet2[4];
		length[0]= new BitSet2(data[index]);index++;
		length[1]= new BitSet2(data[index]);index++;
		length[2]= new BitSet2(data[index]);index++;
		length[3]= new BitSet2(data[index]);index++;
		length2 = new BigBitSet(length);
		int WSDLCustLocalLength= length2.getInt();
		byte[] wsdlCustLocal = new byte[WSDLCustLocalLength];
		for (int i=0;i<WSDLCustLocalLength;i++)
		{
		    wsdlCustLocal[i] = data[index];
		    index++;
		}
		String wsdlCustLocalStr = new String(wsdlCustLocal);
		
		length = new BitSet2[4];
		length[0]= new BitSet2(data[index]);index++;
		length[1]= new BitSet2(data[index]);index++;
		length[2]= new BitSet2(data[index]);index++;
		length[3]= new BitSet2(data[index]);index++;
		length2 = new BigBitSet(length);
		int WSDLCustRemoteLength= length2.getInt();
		byte[] wsdlCustRemote = new byte[WSDLCustRemoteLength];
		for (int i=0;i<WSDLCustRemoteLength;i++)
		{
		    wsdlCustRemote[i] = data[index];
		    index++;
		}
		String wsdlCustRemoteStr = new String(wsdlCustRemote);
		register(wsdlStr, endPointStr, wsdlCustLocalStr, wsdlCustRemoteStr);
	    }
	catch(Exception ex)
	    {
		logger.error("Error at Directory.register");
	    }
    }

    private void register(Message[] messages)
    {
	if (messages==null) return;
	if (messages.length>0) logger.debug("register messages is received");
	for (int i=0;i<messages.length;i++)
	    register(messages[i].getMessageContent());
	    
    }

    public void unregister(String wsdlDocument, String endPoint)
    {
	if (!deploy)
	    {
		logger.warn("Directory not yet deployed");
		//return ;
	    }
	String QoS="";
	if (wsdlDocument.indexOf("<ServiceDiscovery>") !=-1 && wsdlDocument.indexOf("</ServiceDiscovery>")!=-1)
		QoS = wsdlDocument.substring(wsdlDocument.indexOf("<ServiceDiscovery>"), wsdlDocument.indexOf("</ServiceDiscovery>")+19);
	logger.debug("Unregister:QoS="+QoS);
	wsdlDocument = wsdlDocument.substring(QoS.length());
	while (wsdlDocument.charAt(0)=='\n' || wsdlDocument.charAt(0)=='\r')
		wsdlDocument = wsdlDocument.substring(1);
	Definition def = WSDLTools.readWSDL(wsdlDocument);
	Enumeration registryKeysEnumeration = WSRegistry.keys(); 
	while (registryKeysEnumeration.hasMoreElements())
	    {
		Integer key = (Integer)registryKeysEnumeration.nextElement();
		WSRegistryEntry entry = (WSRegistryEntry)WSRegistry.get(key);
		if (entry.getEndPoint().equals(endPoint))
		    {
			Definition defRegistred = entry.getWSDLDefinition();
			if (compare(defRegistred, def)) 
				
			    {
				WSRegistry.remove(key);
				removeServiceFromResume(wsdlDocument);
				messageServicesResume.setMessageContent(servicesResume.getBloomFilterBytes());
				localResumeChanged=true;
			    }
		    }
	    }
    }

    private void removeServiceFromResume(String wsdlDocument)
    {
	String[] resumes = WSDLTools.getResume(wsdlDocument);
	for (int i=0; i<resumes.length;i++) 
	    servicesResume.removeFromFilter(resumes[i]);
    }

    private void unregister(byte[] data)
    {
	try
	    {
		BitSet2[] EPLength = new BitSet2[4];
		EPLength[0]= new BitSet2(data[0]);
		EPLength[1]= new BitSet2(data[1]);
		EPLength[2]= new BitSet2(data[2]);
		EPLength[3]= new BitSet2(data[3]);
		BigBitSet EPLength2 = new BigBitSet(EPLength);
		int EPLength3= EPLength2.getInt();
		byte[] endPoint = new byte[EPLength3];
		for (int i=0;i<EPLength3;i++) 
		    endPoint[i] = data[4+i];
		String endPointStr = new String(endPoint);
		int jump = 4+EPLength3;
		byte[] wsdlDocument = new byte[data.length-jump];
		for (int i=0;i<wsdlDocument.length;i++) 
		    wsdlDocument[i] = data[i+jump];
		String wsdlDocumentStr = new String(wsdlDocument);
		unregister(wsdlDocumentStr, endPointStr);
	    }
	catch(Exception ex)
	    {
		System.out.println("Error at Directory.unregister");
	    }
    }
    
    private void unregister(Message[] messages)
    {
	if (messages==null) return;
	for (int i=0;i<messages.length;i++)
	    unregister(messages[i].getMessageContent());
    }

    public GetServiceData[] getServices(String wsdlDocument, BitSet2 hops, BitSet2 counter)
    {
	if (!deploy)
	    {
		logger.warn("Directory not yet deployed");
		//return (new GetServiceData[0]);
	    }
	//long t1 = System.currentTimeMillis();
	String QoS="";
	if (wsdlDocument.indexOf("<ServiceDiscovery>") !=-1 && wsdlDocument.indexOf("</ServiceDiscovery>")!=-1)
		QoS = wsdlDocument.substring(wsdlDocument.indexOf("<ServiceDiscovery>"), wsdlDocument.indexOf("</ServiceDiscovery>")+19);
	logger.debug("GetService:QoS="+QoS);
	
	QoSParameters QoSParam = new QoSParameters(QoS);
	
	wsdlDocument = wsdlDocument.substring(QoS.length());
	while (wsdlDocument.charAt(0)=='\n' || wsdlDocument.charAt(0)=='\r')
		wsdlDocument = wsdlDocument.substring(1);
	
	Definition def = WSDLTools.readWSDL(wsdlDocument);
	//long t2 = System.currentTimeMillis();
	//System.out.println("******Service definition created  after "+(t2-t1));
	logger.debug("search service in local registry");
	GetServiceData[] servicesFound = getServiceFromRegistry(def, QoSParam);
	if (servicesFound!=null && servicesFound.length>0) 
	    {
		logger.info("service found in local registry");
		 // search CustLocal and CustRemote EndPoints
		for (int m=0; m< servicesFound.length; m++)
		{
		if (servicesFound[m].getWSDLCustLocal()!= null && !servicesFound[m].getWSDLCustLocal().equals("")) 
		{
			logger.debug("search cast local");
			GetServiceData[] custFound = getServices(servicesFound[m].getWSDLCustLocal(), hops, counter);
			if (custFound != null && custFound.length>0)
			servicesFound[m].setCustLocalEP(custFound[0].getServiceEP());
		}
		if (servicesFound[m].getWSDLCustRemote()!= null && !servicesFound[m].getWSDLCustRemote().equals(""))
		{
			logger.debug("search cast remote");
			GetServiceData[] remoteFound = getServices(servicesFound[m].getWSDLCustRemote(), hops, counter);
			if (remoteFound != null && remoteFound.length>0)
			servicesFound[m].setCustRemoteEP(remoteFound[0].getServiceEP());
		}
		}
		return servicesFound;
	    }
	logger.debug("search service in remote repository");
	if (CACHE_ACTIVATION_VALUE==1) servicesFound = getServiceFromRemoteRepository(def, QoSParam);
	if (servicesFound!=null && servicesFound.length>0) 
	    {
		// search CustLocal and CustRemote EndPoints
		for (int m=0; m< servicesFound.length; m++)
		{
		 if ((servicesFound[m].getCustLocalEP()== null || servicesFound[m].getCustLocalEP().equals("")) && servicesFound[m].getWSDLCustLocal()!= null && !servicesFound[m].getWSDLCustLocal().equals("")) 
		{
			logger.debug("search cast local");
			GetServiceData[] custFound = getServices(servicesFound[m].getWSDLCustLocal(), hops, counter);
			if (custFound != null && custFound.length>0)
			servicesFound[m].setCustLocalEP(custFound[0].getServiceEP());
		}
		 
		if ((servicesFound[m].getCustRemoteEP()== null || servicesFound[m].getCustRemoteEP().equals("")) && servicesFound[m].getWSDLCustRemote()!= null && !servicesFound[m].getWSDLCustRemote().equals(""))
		{
			logger.debug("search cast remote");
			GetServiceData[] remoteFound = getServices(servicesFound[m].getWSDLCustRemote(), hops, counter);
			if (remoteFound != null && remoteFound.length>0)
			servicesFound[m].setCustRemoteEP(remoteFound[0].getServiceEP());
		}
		}
		while (!requestCounterUpdateAutorized) {
			try
			{
			Thread.sleep(1);
			}
			catch(Exception ex)
			{
				logger.error("wait error");
			}
			}
		requestsCounter++;
		logger.debug("requestsCounter="+requestsCounter);
		logger.info("service found in the remote repository");
		return servicesFound;
	    }
	logger.debug("search service at remote directories");
	logger.debug("counter.getInt()="+counter.getInt() +" hops.getInt()=" +hops.getInt());
	if ((counter.getInt()+1)<hops.getInt()) 
		{
		    counter.increment();
		    logger.debug("counter.increment().getInt()="+counter.getInt());
			servicesFound = getServicesFromRemoteDirectories(wsdlDocument, QoS, hops, counter);
		}
	if (servicesFound!=null && servicesFound.length>0)
	{
		logger.info("service found at remote directories");
	// add service to remote repository
		if (CACHE_ACTIVATION_VALUE==1) 
			{
			registerIntoRemoteRepository(servicesFound);
			while (!requestCounterUpdateAutorized) {
				try
				{
				Thread.sleep(1);
				}
				catch(Exception ex)
				{
					logger.error("wait error");
				}
				}
			requestsCounter++;
			logger.debug("requestsCounter="+requestsCounter);
			}
	}
	return servicesFound;
	
	/*
	//long t1 = System.currentTimeMillis();
	if (!isServiceInResume(wsdlDocument , servicesResume)) 
	    {
		// search at the remote directories
		//System.out.println("search at the remote directories");
		return getServicesFromRemoteDirectories(wsdlDocument);
	    }
	//long t2 = System.currentTimeMillis();
	//System.out.println("service is in Resume");
	Definition def = WSDLTools.readWSDL(wsdlDocument);
	//long t3 = System.currentTimeMillis();
	GetServiceData[] servicesFound = getServiceFromRegistry(def);
	//long t4 = System.currentTimeMillis();
	//if (servicesFound!=null && servicesFound.length>0) System.out.println("EP Found="+servicesFound[0].getServiceEP());
	//System.out.println("******Service in Resume detection after "+(t2-t1));
	//System.out.println("******Service definition created  after "+(t3-t2));
	//System.out.println("******Service find in Registry after "+(t4-t3));
	//System.out.println("******Service find in Directory after "+(t4-t1));
	return servicesFound;
	*/
    }

    private boolean isServiceInResume(String wsdlDocument , BloomFilter2 bloomFilter)
    {
	boolean bool= true;
	String[] resumes = WSDLTools.getResume(wsdlDocument);
	for (int i=0; i<resumes.length;i++) 
	    bool = bool && bloomFilter.isInFilter(resumes[i]);
	return bool;
    }
    
    private byte[] getServices(byte[] data, BitSet2 hops, BitSet2 counter)
    {
	String wsdlDocument = new String(data);
	GetServiceData[] services = getServices(wsdlDocument, hops, counter);
	int totalLength = 0;
	byte[] forReturn = null;
	int pos =0; 
	if (services!=null)
	    {
	    for (int i=0; i<services.length; i++)
		{
		    byte[] service = services[i].getServiceEP().getBytes();
		    totalLength = totalLength+service.length+4;
		    service = services[i].getWSDLDocument().getBytes();
		    totalLength = totalLength+service.length+4;
		    service = services[i].getCustLocalEP().getBytes();
		    totalLength = totalLength+service.length+4;
		    service = services[i].getWSDLCustLocal().getBytes();
		    totalLength = totalLength+service.length+4;
		    service = services[i].getCustRemoteEP().getBytes();
		    totalLength = totalLength+service.length+4;
		    service = services[i].getWSDLCustRemote().getBytes();
		    totalLength = totalLength+service.length+4;
		}

	    forReturn = new byte[totalLength];
	    for (int i=0; i<services.length;i++)
		{
		    // ep address
		    byte[] service = services[i].getServiceEP().getBytes();
		    BigBitSet bigBitSet = new BigBitSet(service.length);
		    BitSet2[] bitSets = bigBitSet.getBitSet();
		    forReturn[pos]=bitSets[0].getByte();pos++;
		    forReturn[pos]=bitSets[1].getByte();pos++;
		    forReturn[pos]=bitSets[2].getByte();pos++;
		    forReturn[pos]=bitSets[3].getByte();pos++;
		    for (int j=0;j<service.length;j++)
			{
			    forReturn[pos]=service[j];
			    pos++;
			}
		    // wsdl document
		    service = services[i].getWSDLDocument().getBytes();
		    bigBitSet = new BigBitSet(service.length);
		    bitSets = bigBitSet.getBitSet();
		    forReturn[pos]=bitSets[0].getByte();pos++;
		    forReturn[pos]=bitSets[1].getByte();pos++;
		    forReturn[pos]=bitSets[2].getByte();pos++;
		    forReturn[pos]=bitSets[3].getByte();pos++;
		    for (int j=0;j<service.length;j++)
			{
			    forReturn[pos]=service[j];
			    pos++;
			}
		    
		    service = services[i].getCustLocalEP().getBytes();
		    bigBitSet = new BigBitSet(service.length);
		    bitSets = bigBitSet.getBitSet();
		    forReturn[pos]=bitSets[0].getByte();pos++;
		    forReturn[pos]=bitSets[1].getByte();pos++;
		    forReturn[pos]=bitSets[2].getByte();pos++;
		    forReturn[pos]=bitSets[3].getByte();pos++;
		    for (int j=0;j<service.length;j++)
			{
			    forReturn[pos]=service[j];
			    pos++;
			}
		    
		    service = services[i].getWSDLCustLocal().getBytes();
		    bigBitSet = new BigBitSet(service.length);
		    bitSets = bigBitSet.getBitSet();
		    forReturn[pos]=bitSets[0].getByte();pos++;
		    forReturn[pos]=bitSets[1].getByte();pos++;
		    forReturn[pos]=bitSets[2].getByte();pos++;
		    forReturn[pos]=bitSets[3].getByte();pos++;
		    for (int j=0;j<service.length;j++)
			{
			    forReturn[pos]=service[j];
			    pos++;
			}
		    
		    service = services[i].getCustRemoteEP().getBytes();
		    bigBitSet = new BigBitSet(service.length);
		    bitSets = bigBitSet.getBitSet();
		    forReturn[pos]=bitSets[0].getByte();pos++;
		    forReturn[pos]=bitSets[1].getByte();pos++;
		    forReturn[pos]=bitSets[2].getByte();pos++;
		    forReturn[pos]=bitSets[3].getByte();pos++;
		    for (int j=0;j<service.length;j++)
			{
			    forReturn[pos]=service[j];
			    pos++;
			}
		    
		    service = services[i].getWSDLCustRemote().getBytes();
		    bigBitSet = new BigBitSet(service.length);
		    bitSets = bigBitSet.getBitSet();
		    forReturn[pos]=bitSets[0].getByte();pos++;
		    forReturn[pos]=bitSets[1].getByte();pos++;
		    forReturn[pos]=bitSets[2].getByte();pos++;
		    forReturn[pos]=bitSets[3].getByte();pos++;
		    for (int j=0;j<service.length;j++)
			{
			    forReturn[pos]=service[j];
			    pos++;
			}
		}
	    return forReturn;
	    }
	return new byte[0];
    }

    private void getServices(Message[] messages)
    {
	if (messages != null)
	    {
		for (int i=0;i<messages.length;i++)
		    {
			byte[] services = getServices(messages[i].getMessageContent(), messages[i].getMessageHop(),messages[i].getMessageHopCounter()); 
			Message message = MessageFactory.createMessage();
			if (messages[i].getMessageType().equals(MessageType.GET_SERVICE_FROM_DIRECTORY))
			    {
				message.setMessageNumber(messages[i].getMessageNumber());
				//GET_SERVICE_RESPONSE_TO_DIRECTORY_MSG_NUMBER_VALUE++;
				message.setMessageType(MessageType.GET_SERVICE_RESPONSE_TO_DIRECTORY);
			    }
			else 
			    {
				message.setMessageNumber(messages[i].getMessageNumber());
				//GET_SERVICE_RESPONSE_TO_SD_MSG_NUMBER_VALUE++;

				message.setMessageType(MessageType.GET_SERVICE_RESPONSE_TO_SD);
			    }
			message.setMessageSource(new IPAddress(localProfile.getIPAddress()));
			message.setMessageDestination(messages[i].getMessageSource());
			message.setMessagePort(messages[i].getMessagePort());
			message.setMessageHop(HOPS_NUMBER_VALUE);
			message.setMessageHopCounter(new BitSet2());
			message.setMessageContent(services);
			messagesManager.sendInstantaneousMessageTo(message, message.getMessageDestination().getHostIPAddress(), message.getMessagePort());
		    }
	    }
    }
    
    private int directoryAdvertisement(NodeProfile directoryProfile)
    {
    	// return 0 if the Directory is already exist add ttl is updated
    	// return 1 if the Directory is add
    	if (directories.containsKey(directoryProfile.getIPAddress())) 
		{
		DirectoryResume directory =(DirectoryResume)directories.get(directoryProfile.getIPAddress()); 
		directory.setTTL(ADVERTISEMENT_PERIOD_VALUE*10);
		//directory.setTTL(INITIAL_TTL_VALUE);
		return 0;
		}
	DirectoryResume directory = new DirectoryResume(directoryProfile);
	directory.setTTL(ADVERTISEMENT_PERIOD_VALUE*10);
	//directory.setTTL(INITIAL_TTL_VALUE);
	directories.put(directoryProfile.getIPAddress(),directory);
	logger.debug("Directory Advertisement from "+directoryProfile.getIPAddress()+" is received");
	return 1;
    }   

    private void directoryAdvertisement(Message[] messages)
    {
	
	if (messages != null)
	    {
		for (int i=0;i<messages.length;i++)
		    {
			//System.out.println("Advertisement is received");
			NodeProfile directoryProfile = new NodeProfile(messages[i].getMessageContent());
			if (directoryAdvertisement(directoryProfile)==1)
			{
				//send GET ALL RESUME REQUEST to this new directory
				messagesManager.sendInstantaneousMessageTo(messageServicesResume, directoryProfile.getIPAddress() ,COMMUNICATION_PORT_VALUE);
				logger.debug("DIRECTORY_GET_ALL_RESUME is sent to "+directoryProfile.getIPAddress());
				DIRECTORY_GET_ALL_RESUMES_MSG_NUMBER_VALUE++;
			}
			// no response
			/*
			messageProfileToDirectory.setMessageDestination(messages[i].getMessageSource());
			messageProfileToDirectory.setMessageType(MessageType.DIRECTORY_TO_DIRECTORY_ADVERTISEMENT_RESPONSE);
			messagesManager.sendMessageTo(messageProfileToDirectory, messageProfileToDirectory.getMessageDestination().getHostIPAddress(), messageProfileToDirectory.getMessagePort()); 
			DIRECTORY_TO_DIRECTORY_ADVERTISEMENT_RESPONSE_MSG_NUMBER_VALUE++;
			*/
		    }
	    }
    }

    /*
    private void directoryAdvertisementResponse(Message[] messages)
    {
	if (messages != null)
	    {
		for (int i=0;i<messages.length;i++)
		    {
			NodeProfile directoryProfile = new NodeProfile(messages[i].getMessageContent());
			directoryAdvertisement(directoryProfile);
		    }
	    }
    }
    */

    private void setDirectoryResume(byte[] resume, String directoryIPAddress)
    {
    	
	DirectoryResume directory = (DirectoryResume)directories.get(directoryIPAddress);
	if (directory != null) 
		{
			directory.setTTL(ADVERTISEMENT_PERIOD_VALUE*10);
		    //directory.setTTL(INITIAL_TTL_VALUE);
			directory.setDirectoryResume(resume);
		}
	logger.debug("DirectoryResume from "+directoryIPAddress+" is received");
    }

    private void setDirectoryProfileAndResume(byte[] profile, byte[] resume)
    {
    	NodeProfile directoryProfile = new NodeProfile(profile);
    	if (directoryProfile.getIPAddress().equals(localProfile.getIPAddress())) return;
    	if (directories.containsKey(directoryProfile.getIPAddress())) 
		{
		DirectoryResume directory = (DirectoryResume)directories.get(directoryProfile.getIPAddress());
		directory.setDirectoryResume(resume);
		directory.setTTL(ADVERTISEMENT_PERIOD_VALUE*10);
		//directory.setTTL(INITIAL_TTL_VALUE);
		logger.debug("Directory Profile and Resume are received from "+directoryProfile.getIPAddress());
		logger.debug("directories.size()="+directories.size());
		return;
		}
	DirectoryResume directory = new DirectoryResume(directoryProfile);
	directory.setDirectoryResume(resume);
	directory.setTTL(ADVERTISEMENT_PERIOD_VALUE*10);
	//directory.setTTL(INITIAL_TTL_VALUE);
	directories.put(directoryProfile.getIPAddress(),directory);
	logger.debug("****Directory Profile and Resume are received from "+directoryProfile.getIPAddress());
	logger.debug("directories.size()="+directories.size());
    }

    /*
    private void directoryResume(Message[] messages)
    {
    	// must be can receive one or + resumes into one message
	if (messages != null)
	    {
		for (int i=0;i<messages.length;i++)
		    {
			//String resume = new String(messages[i].getMessageContent()); 
			byte[] resume = messages[i].getMessageContent();
			String directoryIPAddress = messages[i].getMessageSource().getHostIPAddress();
			setDirectoryResume(resume, directoryIPAddress);
			//messageServicesResume.setMessageType(MessageType.DIRECTORY_RESUME_EXCHANGE_RESPONSE);
			//messagesManager.sendMessageTo(messageServicesResume, directoryIPAddress, COMMUNICATION_PORT_VALUE);
			//DIRECTORY_RESUME_EXCHANGE_RESPONSE_MSG_NUMBER_VALUE++;
		    }
	    }
    }
*/
    private void directoryGetAllResumesRequest(Message[] messages)
    {
       	// send all resumes
    	if (messages != null)
	    {
    		Message allResumesMessage = null;
    		if (messages.length>0) allResumesMessage = createAllResumesMessage();
    		for (int i=0;i<messages.length;i++)
		    {
    			String directoryIPAddress = messages[i].getMessageSource().getHostIPAddress();
				//messagesManager.sendMessageTo(allResumesMessage, directoryIPAddress, COMMUNICATION_PORT_VALUE);
				messagesManager.sendInstantaneousMessageTo(allResumesMessage, directoryIPAddress, COMMUNICATION_PORT_VALUE);
			    DIRECTORY_GET_ALL_RESUMES_RESPONSE_MSG_NUMBER_VALUE++;
			    logger.debug("All the resumes are sent to "+directoryIPAddress);
		    }
	    }
    }
    
    private void directoryGetAllResumesResponse(Message[] messages)
    {
    	
    	if (messages == null) return;
    	if (messages.length>0) logger.debug("directoryGetAllResumesResponse is received");
    	for (int i=0;i<messages.length;i++)
    	{
    		byte[] content =messages[i].getMessageContent();
    		int pos=0;
    		if (content.length>=8)
    		{
    			while (pos<content.length)
    			{
    			BitSet2[] bitsets = new BitSet2[4];
    			bitsets[0]= new BitSet2(content[pos]);pos++;
    			bitsets[1]= new BitSet2(content[pos]);pos++;
    			bitsets[2]= new BitSet2(content[pos]);pos++;
    			bitsets[3]= new BitSet2(content[pos]);pos++;
    			BigBitSet bitSetLength = new BigBitSet(bitsets);
    			int length = bitSetLength.getInt();
    			byte[] profile = new byte[length];
    			for (int j=0;j<length;j++)
    			{
    				profile[j]=content[pos]; pos++;
    			}
    			
    			bitsets = new BitSet2[4];
    			bitsets[0]= new BitSet2(content[pos]);pos++;
    			bitsets[1]= new BitSet2(content[pos]);pos++;
    			bitsets[2]= new BitSet2(content[pos]);pos++;
    			bitsets[3]= new BitSet2(content[pos]);pos++;
    			bitSetLength = new BigBitSet(bitsets);
    			length = bitSetLength.getInt();
    			byte[] resume = new byte[length];
    			for (int j=0;j<length;j++)
    			{
    				resume[j]=content[pos]; pos++;
    			}
    			setDirectoryProfileAndResume(profile,resume);
    			}
    		}
    	}
    		
   }

    
    private void directoryResumePublication(Message[] messages)
    {
	if (messages != null)
	    {
		for (int i=0;i<messages.length;i++)
		    {
			byte[] resume = messages[i].getMessageContent();
			String directoryIPAddress = messages[i].getMessageSource().getHostIPAddress();
			logger.debug("directoryResumePub from address "+directoryIPAddress);
			setDirectoryResume(resume, directoryIPAddress);
		    }
	    }
    }

    private GetServiceData[] getServiceFromRegistry(Definition def, QoSParameters QoSParam)
    {
	Vector servicesFound = new Vector();
	QName serviceName = def.getQName();
       	Enumeration registryEnumeration = WSRegistry.elements();
	while (registryEnumeration.hasMoreElements())
	    {
		WSRegistryEntry entry = (WSRegistryEntry)registryEnumeration.nextElement();
		if (entry.getServiceName().equals(serviceName))
		    {
			Definition defRegistred = entry.getWSDLDefinition();
			if (compare(def,defRegistred)) 
			    {
				QoSParameters entryQoS = entry.getQoSParameters();
				//System.out.println("I will compare the QoS");
				if (entryQoS.greaterThan(QoSParam))
				{
				GetServiceData service = new GetServiceData();
				service.setServiceEP(entry.getEndPoint());
				String QoS ="<ServiceDiscovery>"+entryQoS.getXMLDocument()+"</ServiceDiscovery>";
				service.setWSDLDocument(QoS+entry.getWSDLDocument());
				service.setWSDLCustLocal(entry.getWSDLCustLocal());
				service.setWSDLCustRemote(entry.getWSDLCustRemote());
				servicesFound.add(service);
				}
			    }
		    }
	    }

	GetServiceData[] forReturn = new GetServiceData[servicesFound.size()];
	for (int i=0; i<forReturn.length;i++)
	    forReturn[i]=(GetServiceData)servicesFound.elementAt(i);
	return forReturn     ;
    }

    private GetServiceData[] getServiceFromRemoteRepository(Definition def, QoSParameters QoSParam)
    {
	Vector servicesFound = new Vector();
	QName serviceName = def.getQName();
       	Enumeration registryEnumeration = WSRemoteRepository.elements();
	while (registryEnumeration.hasMoreElements())
	    {
		WSCacheEntry entry = (WSCacheEntry)registryEnumeration.nextElement();
		if (entry.getServiceName().equals(serviceName))
		    {
			Definition defRegistred = entry.getWSDLDefinition();
			if (compare(def,defRegistred)) 
			    {
				QoSParameters entryQoS = entry.getQoSParameters();
				if (entryQoS.greaterThan(QoSParam))
				{
				entry.addAccessTimes(System.currentTimeMillis());
				entry.incrementPopularity();
				GetServiceData service = new GetServiceData();
				service.setServiceEP(entry.getEndPoint());
				String QoS ="<ServiceDiscovery>"+entryQoS.getXMLDocument()+"</ServiceDiscovery>";
				service.setWSDLDocument(QoS+entry.getWSDLDocument());
				service.setWSDLCustLocal(entry.getWSDLCustLocal());
				service.setCustLocalEP(entry.getCustLocalEP());
				service.setWSDLCustRemote(entry.getWSDLCustRemote());
				service.setCustRemoteEP(entry.getCustRemoteEP());
				servicesFound.add(service);
				}
			    }
		    }
	    }

	GetServiceData[] forReturn = new GetServiceData[servicesFound.size()];
	for (int i=0; i<forReturn.length;i++)
	    forReturn[i]=(GetServiceData)servicesFound.elementAt(i);
	return forReturn     ;
    }

    private GetServiceData[] getServicesFromRemoteDirectories(String wsdlDocument, String QoS, BitSet2 hops, BitSet2 counter)
    {
    logger.debug("directories.size()="+directories.size());
    
    messagesManager.clean(new BitSet2[]{MessageType.GET_SERVICE_RESPONSE_TO_DIRECTORY});
    Enumeration directoriesEnumeration = directories.elements();
	
	Message messageGetService = MessageFactory.createMessage();
	messageGetService.setMessageNumber(GET_SERVICE_REQUEST_TO_DIRECTORY_MSG_NUMBER_VALUE);
	GET_SERVICE_REQUEST_TO_DIRECTORY_MSG_NUMBER_VALUE++;
	messageGetService.setMessageType(MessageType.GET_SERVICE_FROM_DIRECTORY); 
	messageGetService.setMessageSource(new IPAddress(localProfile.getIPAddress()));
	messageGetService.setMessagePort(COMMUNICATION_PORT_VALUE);
	messageGetService.setMessageHop(hops);
	messageGetService.setMessageHopCounter(counter);
	messageGetService.setMessageContent((QoS+wsdlDocument).getBytes());
	int numberOfRequestedDir=0;
	// search without bloom filter
	if (BLOOM_FILTER_ACTIVATION_VALUE==0)
	{
		
		while (directoriesEnumeration.hasMoreElements() && numberOfRequestedDir<MAX_DIRECTORY_REQUESTED)
	    {
		DirectoryResume directory = (DirectoryResume)directoriesEnumeration.nextElement();
		messagesManager.sendInstantaneousMessageTo(messageGetService, directory.getDirectoryProfile().getIPAddress(), COMMUNICATION_PORT_VALUE);
		logger.debug("Get Service From dir is sent to "+directory.getDirectoryProfile().getIPAddress());
		numberOfRequestedDir++;
	    }
		Message[] messages = null;
		int time = 0;
		while ((messages == null || messages.length==0) && time<WAITING_GET_SERVICE_RESPONSE_MAX_VALUE)
	    {
			try
			{
				Thread.sleep(WAITING_GET_SERVICE_RESPONSE_MIN_VALUE);		
				time = time + WAITING_GET_SERVICE_RESPONSE_MIN_VALUE;	
				messages = messagesManager.getMessagesType(MessageType.GET_SERVICE_RESPONSE_TO_DIRECTORY , messageGetService.getMessageNumber());
			}
			catch (Exception ex)
			{
				logger.error("Error at Directory.getServicesFromRemoteDirectories:Thread.sleep");
			}
	    }
		
		if (messages ==null || messages.length ==0) 
			return (new GetServiceData[0]);
		//********** some reponses are received get content and construct string[]
		Vector servicesEPTmp = new Vector();
		for (int i=0;i<messages.length;i++)
	    	{
			Vector vectTmp = SD.getServicesFrom(messages[i].getMessageContent());
			for (int z=0;z<vectTmp.size();z++) servicesEPTmp.add((GetServiceData)vectTmp.elementAt(z));
	    	}
		GetServiceData[] forReturn = new GetServiceData[servicesEPTmp.size()];
		for (int i=0;i<forReturn.length;i++)
			forReturn[i]=(GetServiceData)servicesEPTmp.elementAt(i);
		return forReturn;
	}
	
	else while (directoriesEnumeration.hasMoreElements()  && numberOfRequestedDir<MAX_DIRECTORY_REQUESTED)
	    {
		DirectoryResume directory = (DirectoryResume)directoriesEnumeration.nextElement();
		BitSet2 bitsettmp = directory.getDirectoryBitSet();
		//BloomFilter2 tmp = new BloomFilter2(28 * 1000, 10);
		BloomFilter2 tmp = new BloomFilter2(BLOOM_FILTER_SIZE_VALUE, BLOOM_FILTER_HASH_FUNCTION_NUMBER_VALUE);
		tmp.setBloomFilter(bitsettmp);

		if (isServiceInResume(wsdlDocument, tmp)) 
	      {
			logger.info("Service found at remote directory address ="+directory.getDirectoryProfile().getIPAddress());
			try
			    {
				messagesManager.sendInstantaneousMessageTo(messageGetService, directory.getDirectoryProfile().getIPAddress(), COMMUNICATION_PORT_VALUE);
				numberOfRequestedDir++;
				Message[] messages = null;
				int time = 0;
				while ((messages == null || messages.length==0) && time<WAITING_GET_SERVICE_RESPONSE_MAX_VALUE)
				    {
					Thread.sleep(WAITING_GET_SERVICE_RESPONSE_MIN_VALUE);		
					time = time + WAITING_GET_SERVICE_RESPONSE_MIN_VALUE;	
					messages = messagesManager.getMessagesType(MessageType.GET_SERVICE_RESPONSE_TO_DIRECTORY , messageGetService.getMessageNumber());
				    }
				//System.out.println("time wait is ="+time);
				if (messages ==null || messages.length ==0) 
				    return (new GetServiceData[0]);
				//********** some reponses are received get content and construct string[]
				Vector servicesEPTmp = new Vector();
				for (int i=0;i<messages.length;i++)
				    {
					Vector vectTmp = SD.getServicesFrom(messages[i].getMessageContent());
					for (int z=0;z<vectTmp.size();z++) servicesEPTmp.add((GetServiceData)vectTmp.elementAt(z));
				    }
				GetServiceData[] forReturn = new GetServiceData[servicesEPTmp.size()];
				for (int i=0;i<forReturn.length;i++)
				    forReturn[i]=(GetServiceData)servicesEPTmp.elementAt(i);
				return forReturn;
			    }
			catch (Exception ex)
			    {
				logger.error("getServicesFromRemoteDirectories:Error at getService from remote directories");
			    }
	      }
		
	    }
	return new GetServiceData[0];
    }

    // election agregation
    
    public void electionMessagesRequest(Message[] messages)
    {
    	if (messages == null || messages.length==0) return;
    		
    	    	
    	Message messageElectionResponse = MessageFactory.createMessage();
    	messageElectionResponse.setMessageType(MessageType.DIRECTORY_ELECTION_PROCESS_RESPONSE); 
    	messageElectionResponse.setMessageSource(new IPAddress(localProfile.getIPAddress()));
    	messageElectionResponse.setMessagePort(COMMUNICATION_PORT_VALUE);
    	messageElectionResponse.setMessageHop(HOPS_NUMBER_VALUE);
    	messageElectionResponse.setMessageHopCounter(new BitSet2());
    	    	
    	long totalTime =0;	    	
    	for (int i=0;i<messages.length;i++)
	    	{
    		
    		
    		logger.debug("Dir:Election Request from "+messages[i].getMessageSource().getHostIPAddress() +"is received");
    		messageElectionResponse.setMessageInitiator(messages[i].getMessageInitiator());
    		messageElectionResponse.setMessageNumber(messages[i].getMessageNumber());
    		//mi]essageElectionResponse.setMessageNumber(DIRECTORY_ELECTION_PROCESS_RESPONSE_MSG_NUMBER_VALUE);
    		//response only when my profile is better than the received profile 
    		//is commented
    		//NodeProfile Profile = new NodeProfile(messages[i].getBytes());
    		//if (Election.betterProfileComparedTo(localProfile, Profile))
		    	//{
    		// wait for response from others nodes
    		try
			{
    			long timeToWait = Setting.ELECTION_WAITING_RESPONSE_TIME_VALUE*(messages[i].getMessageHop().getInt()-(messages[i].getMessageHopCounter().getInt()+1));
    			logger.debug("Dir:ELECTION_WAITING_RESPONSE_TIME_VALUE="+Setting.ELECTION_WAITING_RESPONSE_TIME_VALUE);
    			logger.debug("Dir:messages[i].getMessageHop().getInt()="+messages[i].getMessageHop().getInt());
    			logger.debug("Dir:messages[i].getMessageHopCounter().getInt()"+messages[i].getMessageHopCounter().getInt());
    			
    			logger.debug("Dir:timeToWait="+timeToWait);
    			logger.debug("Dir:totalTime="+totalTime);
    			if (timeToWait > totalTime) timeToWait = timeToWait - totalTime;
    			else timeToWait = 0;
    			logger.debug("Dir:waiting for election response from the others nodes, time="+timeToWait);
    			Thread.sleep(timeToWait);
    			totalTime = totalTime+timeToWait;
			}
    		catch(Exception ex)
			{
    			logger.error("Error at waiting for election response from the others nodes");
			}
    		
    	  	
        	
    		byte[] toSend = new byte[0];
    		int totalLength=0;
        	    		
    		Message[] responseMessages = messagesManager.getMessagesType(MessageType.DIRECTORY_ELECTION_PROCESS_RESPONSE, messages[i].getMessageNumber(), messages[i].getMessageInitiator());
    		if (responseMessages != null)
    		{
    			for (int r=0; r<responseMessages.length; r++)
    				if (responseMessages[r].getMessageContent()!=null ) totalLength = totalLength+responseMessages[r].getMessageContent().length;
    		}
    		
    		toSend = new byte[totalLength];
    		int position=0;
    			
    		if (responseMessages != null)
    		{
    			for (int r=0; r<responseMessages.length; r++)
    				if (responseMessages[r].getMessageContent()!=null ) 
    					{
    					for (int z=0; z< responseMessages[r].getMessageContent().length ; z++)
    	    			{
    	    				toSend[position]= responseMessages[r].getMessageContent()[z];
    	    				position++;
    	    			}
    					}
    		}
    				
    		logger.debug("Dir:Total length of sent profile="+toSend.length);
    		messageElectionResponse.setMessageContent(toSend);
    		messagesManager.sendInstantaneousMessageTo(messageElectionResponse, messages[i].getMessageSource().getHostIPAddress(), COMMUNICATION_PORT_VALUE);
    		logger.debug("from Dir:Election candidature (type="+messageElectionResponse.getMessageType().getInt()+")is sent to "+messages[i].getMessageSource().getHostIPAddress() +" with final destination is "+messages[i].getMessageInitiator().getHostIPAddress());
		    	//}		
	    	}
    }
    
    
    
    
    
    private void registryUpdate()
    {
	Enumeration registryEnumeration = WSRegistry.keys();
	while (registryEnumeration.hasMoreElements())
	    {
		Integer entryKey = (Integer)registryEnumeration.nextElement();
		WSRegistryEntry entry = (WSRegistryEntry)WSRegistry.get(entryKey);
		entry.updateTTL(TTL_UPDATING_VALUE);
		entry.updateQoSTTL(TTL_UPDATING_VALUE);
		if (entry.getTTL()==0) 
		    {
			entry = (WSRegistryEntry)WSRegistry.remove(entryKey);
			logger.info("The service "+entry.getEndPoint()+" is removed from Registry");
		    }
	    }
    }
    
    private void remoteRepositoryUpdate()
    {
	Enumeration registryEnumeration = WSRemoteRepository.keys();
	while (registryEnumeration.hasMoreElements())
	    {
		Integer entryKey = (Integer)registryEnumeration.nextElement();
		WSRegistryEntry entry = (WSRegistryEntry)WSRemoteRepository.get(entryKey);
		entry.setTTL(entry.getTTL()-TTL_UPDATING_VALUE);
		entry.updateQoSTTL(TTL_UPDATING_VALUE);
		if (entry.getTTL()<=0) 
		    {
			entry = (WSRegistryEntry)WSRemoteRepository.remove(entryKey);
			logger.info("The service "+entry.getEndPoint()+" is removed from Remote Repository");
		    }
	    }
    }

    private boolean compare(Definition def1, Definition def2)
    {
	// return true if def1 is equals def2 else false
	return WSDLTools.compare(def1, def2);
    }

    public Thread directoryListenProcess() {
        try
	    {
		Thread directoryListenProcessThread = 
		    new Thread( new Runnable() 
			{
			    public void run() 
			    {
				while (deploy)
				    {
					try
					    {
						
						Message[] messages = messagesManager.getMessagesType(MessageType.REGISTER_SERVICE);
						register(messages);
						messages = messagesManager.getMessagesType(MessageType.UNREGISTER_SERVICE);
						unregister(messages);
						//messages = messagesManager.getMessagesType(MessageType.GET_SERVICE_RESPONSE_TO_DIRECTORY);
						//getServices(messages);
						messages = messagesManager.getMessagesType(MessageType.GET_SERVICE_FROM_DIRECTORY);
						getServices(messages);
						messages = messagesManager.getMessagesType(MessageType.GET_SERVICE_FROM_SD);
						//if (messages!=null && messages.length>0) System.out.println("some get service from sd are received");
						getServices(messages);
						messages = messagesManager.getMessagesType(MessageType.DIRECTORY_RESUME_PUBLICATION);
						directoryResumePublication(messages);
						messages = messagesManager.getMessagesType(MessageType.DIRECTORY_GET_ALL_RESUMES_RESPONSE);
						directoryGetAllResumesResponse(messages);
						messages = messagesManager.getMessagesType(MessageType.DIRECTORY_GET_ALL_RESUMES);
						directoryGetAllResumesRequest(messages);
						//messages = messagesManager.getMessagesType(MessageType.DIRECTORY_TO_DIRECTORY_ADVERTISEMENT_RESPONSE);
						//directoryAdvertisementResponse(messages);
						messages = messagesManager.getMessagesType(MessageType.DIRECTORY_TO_DIRECTORY_ADVERTISEMENT);
						directoryAdvertisement(messages);
						messages = messagesManager.getMessagesType(MessageType.DIRECTORY_ELECTION_PROCESS_REQUEST);
		    			electionMessagesRequest(messages);
						//directoryResume(messages);
						messagesManager.clean(new BitSet2[]{MessageType.DIRECTORY_ADVERTISEMENT, MessageType.DIRECTORY_ELECTION_PROCESS_REQUEST});
						Thread.sleep(LISTENING_PERIOD_VALUE);
					    }
					catch (Exception ex)
					    {
						logger.error("at Directory.directoryListenProcess.run()");
						ex.printStackTrace();
					    }
				    }
			    }
			});
		return directoryListenProcessThread;
	    }
	catch (Exception ex)
	    {
		logger.error("at create Directory.directoryListenProcess Thread");
	    }
	return null;
    }

   public Thread directorySendAdvertisementProcess() {
        try
	    {
		Thread directorySendAdvertisementProcessThread = 
		    new Thread( new Runnable() 
			{
			    public void run() 
			    {
				while (deploy)
				    {
					try
					    { 
						// Message to SD
					       messageProfileToSD.setMessageNumber(DIRECTORY_ADVERTISEMENT_MSG_NUMBER_VALUE);
					       messageProfileToDirectory.setMessageNumber(DIRECTORY_ADVERTISEMENT_MSG_NUMBER_VALUE);
					       messagesManager.instantaneousBroadcastMessage(messageProfileToSD, COMMUNICATION_PORT_VALUE);
					       messagesManager.instantaneousBroadcastMessage(messageProfileToDirectory, COMMUNICATION_PORT_VALUE);
					       DIRECTORY_ADVERTISEMENT_MSG_NUMBER_VALUE++;
					       //Message to Directory
					       //messageProfileToSD.setMessageNumber(DIRECTORY_TO_DIRECTORY_ADVERTISEMENT_MSG_NUMBER_VALUE);
					       //messagesManager.broadcastMessage(messageProfileToDirectory, COMMUNICATION_PORT_VALUE);
					       //DIRECTORY_TO_DIRECTORY_ADVERTISEMENT_MSG_NUMBER_VALUE++;
					       Thread.sleep(ADVERTISEMENT_PERIOD_VALUE);
					    }
					catch (Exception ex)
					    {
						logger.error("at Directory.directorySendAdvertisementProcess.run()");
						ex.printStackTrace();
					    }
				    }
			    }
			});
		return directorySendAdvertisementProcessThread;
	    }
	catch (Exception ex)
	    {
		logger.error("at create Directory.directorySendAdvertisementProcess Thread");
	    }
	return null;
    }

   public Thread directorySendResumeProcess() {
        try
	    {
		Thread directorySendResumeProcessThread = 
		    new Thread( new Runnable() 
			{
			    public void run() 
			    {
			    	try
					   {
			    		Thread.sleep(RESUME_EXCHANGE_PERIOD_VALUE);
			    		while (deploy && localResumeChanged)
			    		{
			    		
			    			localResumeChanged=false;
					       // test here if need to send
					       // if change Level is greater than max 
					       Enumeration iPAddresses = directories.keys();
					       messageServicesResume.setMessageType(MessageType.DIRECTORY_RESUME_PUBLICATION);
					       while (iPAddresses.hasMoreElements()) 
						   {
						       String iPAddress =(String)iPAddresses.nextElement();
						       messagesManager.sendInstantaneousMessageTo(messageServicesResume, iPAddress,COMMUNICATION_PORT_VALUE);
						       logger.debug("DIRECTORY_RESUME_PUBLICATION is sent");
						       DIRECTORY_RESUME_PUBLICATION_MSG_NUMBER_VALUE++;
						   }
					
					    }
					   }
					catch (Exception ex)
					 {
						logger.error("Error at Directory.directorySendResumeProcess.run()");
						//ex.printStackTrace();
					 }
				    
			    }
			});
		return directorySendResumeProcessThread;
	    }
	catch (Exception ex)
	    {
		logger.error("at create Directory.directorySendResumeProcess Thread");
	    }
	return null;
    }

   public Thread repositoryUpdateProcess() {
        try
	    {
		Thread remoteRepositoryUpdateProcessThread = 
		    new Thread( new Runnable() 
			{
			    public void run() 
			    {
				while (deploy)
				    {
					try
					    { 
							Thread.sleep(TTL_UPDATING_VALUE);
							registryUpdate();
							remoteRepositoryUpdate();
					    }
					catch (Exception ex)
					    {
							logger.error("at Directory.remoteRepositoryUpdateProcessThread.run()");
							//ex.printStackTrace();
					    }
				    }
			    }
			});
		return remoteRepositoryUpdateProcessThread;
	    }
	catch (Exception ex)
	    {
		logger.error("at create Directory.remoteRepositoryUpdateProcessThread");
	    }
	return null;
    }
   
   public Thread DirectroriesChecking() {
	try
	{
	 Thread directoriesCheckingProcessThread = new Thread(new Runnable(){
	    public void run() 
	    {
		while (deploy)
		    {
			try
			{
				Thread.sleep(ADVERTISEMENT_PERIOD_VALUE);
				Iterator directoriesIterator = directories.values().iterator();
				while (directoriesIterator.hasNext())
				{
					DirectoryResume directory = (DirectoryResume)directoriesIterator.next();
					directory.setTTL(directory.getTTL()-ADVERTISEMENT_PERIOD_VALUE);
					logger.debug("***TTL dir ="+directory.getTTL());
					if (directory.getTTL()<=0) 
					{
						logger.info("The directory "+directory.getDirectoryProfile().getIPAddress()+" is removed");
						directoriesIterator.remove();
					}
				}
				
			}
			catch(Exception ex)
			{
				logger.error("at Dir.DirectoriesCheckingProcessThread.run()");
			}
		    }			
	    }			
	 });			
	 return directoriesCheckingProcessThread; 
	}
	catch (Exception ex)
	{
		logger.error("at create Dir.DirectoriesChecking Thread");
	}
	return null;
}

   
   public Thread servicePopularityUpdate() {
	try
	{
		
	 Thread servicePopularityUpdateThread = new Thread(new Runnable(){
	    public void run() 
	    {
	    int position=0;
		while (deploy)
		    {
			try
			{
				Thread.sleep(daltaT/100);
				requestCounterUpdateAutorized = false;
				requestsNumber[position]=requestsCounter;
				requestsCounter=0;
				requestCounterUpdateAutorized = true;
				position++;
				if (position==100) 
					{
					position = 0;
					// calcul AVG requests in T for one service
					// sum requests in T
					int sum =0;
					for (int i=0; i<100; i++)
						sum = sum+requestsNumber[i];
					logger.debug("sum="+sum);
					logger.debug("WSRemoteRepository.size()="+WSRemoteRepository.size());
					double avgRequest = 0;
					Double sumDouble = new Double(sum);
					if (WSRemoteRepository.size()>0) avgRequest = sumDouble.doubleValue()/WSRemoteRepository.size(); 
					logger.debug("avgRequest="+avgRequest);				
					// update popularity for each service in the cache
					// popularity = popularity - avgRequest
					Enumeration registryEnumeration = WSRemoteRepository.elements();
					while (registryEnumeration.hasMoreElements())
					    {
						WSRegistryEntry entry = (WSRegistryEntry)registryEnumeration.nextElement();
						entry.decrementPopularity(avgRequest);
					    }
					
					}
			}
			catch(Exception ex)
			{
				logger.error("Error at Dir.servicePopularityUpdateThread.run()");
				//ex.printStackTrace();
			}
		    }			
	    }			
	 });			
	 return servicePopularityUpdateThread; 
	}
	catch (Exception ex)
	{
		logger.error("at create Dir.servicePopularityUpdate Thread");
	}
	return null;
}

   
}
