package ubisec.discovery;

import ubisec.configuration.Setting;
import ubisec.discovery.message.*;
import ubisec.discovery.message.util.*;
import ubisec.profile.*;
import ubisec.util.*;
import ubisec.net.IPAddress;

import java.util.Vector;

import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.SimpleLayout;

/**
*
* Election.java
*
* @author Rafik CHIBOUT 
* @author Francoise SAILHAN
* @version 0.1 oct. 2004
*
*  
*/

public class Election
{
    public static int LISTENING_PERIOD=0;
    public static int ADVERTISEMENT_PERIOD=1; 
    public static int COMMUNICATION_PORT=2;
    public static int DIRECTORY_ELECTION_PROCESS_MSG_NUMBER=3;
    public static int DIRECTORY_ELECTION_PROCESS_RESPONSE_MSG_NUMBER=4;
    public static int DIRECTORY_ELECT_MSG_NUMBER=5;
    public static int ELECTION_WAITING_RESPONSE_TIME=6;
    public static int HOPS_NUMBER=7;
    
    private static int LISTENING_PERIOD_VALUE=5;
    private static int ADVERTISEMENT_PERIOD_VALUE=2000; 
    private static int COMMUNICATION_PORT_VALUE=9999;
    private static int DIRECTORY_ELECTION_PROCESS_MSG_NUMBER_VALUE=0;
    private static int DIRECTORY_ELECTION_PROCESS_RESPONSE_MSG_NUMBER_VALUE=0;
    private static int DIRECTORY_ELECT_MSG_NUMBER_VALUE=0;
    private static int ELECTION_WAITING_RESPONSE_TIME_VALUE=2000;
    private static int INT_HOPS_NUMBER_VALUE=1;
            
    public static MessagesManager manager = null;
    private NodeProfile localProfile  = null; 
    private Vector candidates = new Vector();
    private BitSet2 HOPS_NUMBER_VALUE=HOPS.N1;
    
    private static Logger logger = Logger.getLogger(Election.class); 
    private Level loggerLevel = Level.INFO;
    
    
    public Election()
    {
    	SimpleLayout layout = new SimpleLayout();
    	ConsoleAppender appender = new ConsoleAppender(layout,ConsoleAppender.SYSTEM_OUT);
    	appender.setName("screen");
    	appender.activateOptions();
		logger.addAppender(appender);
    	logger.setLevel(loggerLevel);
    	
    	if (Setting.ELECTION_HOPS_VALUE>1 && Setting.ELECTION_HOPS_VALUE<128)
    	{
    		INT_HOPS_NUMBER_VALUE=Setting.ELECTION_HOPS_VALUE;
    		BigBitSet tmp = new BigBitSet(INT_HOPS_NUMBER_VALUE);
    		HOPS_NUMBER_VALUE=tmp.getBitSet()[0];
    	}
    	if (Setting.PortNumber>1024)
    		COMMUNICATION_PORT_VALUE=Setting.PortNumber;
       	LISTENING_PERIOD_VALUE=ubisec.configuration.Setting.ELECTION_LISTENING_PERIOD_VALUE;
    	ADVERTISEMENT_PERIOD_VALUE=ubisec.configuration.Setting.ELECTION_ADVERTISEMENT_PERIOD_VALUE; 
    	ELECTION_WAITING_RESPONSE_TIME_VALUE=ubisec.configuration.Setting.ELECTION_WAITING_RESPONSE_TIME_VALUE;
    	localProfile = SD.localProfile;
    }

    public void setProperty(int propertyName, int propertyValue)
    {
    	switch (propertyName)
	    {
	    case 0: LISTENING_PERIOD_VALUE=propertyValue;break;
	    case 1: ADVERTISEMENT_PERIOD_VALUE=propertyValue;break;
	    case 2: COMMUNICATION_PORT_VALUE=propertyValue;break;
	    case 3: DIRECTORY_ELECTION_PROCESS_MSG_NUMBER_VALUE=propertyValue;break;
	    case 4: DIRECTORY_ELECTION_PROCESS_RESPONSE_MSG_NUMBER_VALUE=propertyValue;break;
	    case 5: DIRECTORY_ELECT_MSG_NUMBER_VALUE=propertyValue;break;
	    case 6: ELECTION_WAITING_RESPONSE_TIME_VALUE=propertyValue;break;
	    case 7: INT_HOPS_NUMBER_VALUE=propertyValue;
			    BigBitSet tmp = new BigBitSet(INT_HOPS_NUMBER_VALUE);
		        HOPS_NUMBER_VALUE=tmp.getBitSet()[0];
		        break; 
	    }
    }
    
    public int getPropertyValue(int propertyName)
    {
	switch (propertyName)
	    {
	    case 0: return LISTENING_PERIOD_VALUE;
	    case 1: return ADVERTISEMENT_PERIOD_VALUE;
	    case 2: return COMMUNICATION_PORT_VALUE;
	    case 3: return DIRECTORY_ELECTION_PROCESS_MSG_NUMBER_VALUE;
	    case 4: return DIRECTORY_ELECTION_PROCESS_RESPONSE_MSG_NUMBER_VALUE;
	    case 5: return DIRECTORY_ELECT_MSG_NUMBER_VALUE;
	    case 6: return ELECTION_WAITING_RESPONSE_TIME_VALUE;
	    case 7: return INT_HOPS_NUMBER_VALUE;
	    }
	return -1;
    }

    public NodeProfile[] election(float MIN_PERFORMANCE_THRESHOLD_VALUE)
    {
    	manager = SD.manager;
    	manager.clean(new BitSet2[]{MessageType.DIRECTORY_ELECTION_PROCESS_RESPONSE, MessageType.DIRECTORY_ELECT});
    	localProfile.profileUpdate();
    	Message messageElectionRequest = MessageFactory.createMessage();
    	messageElectionRequest.setMessageNumber(DIRECTORY_ELECTION_PROCESS_MSG_NUMBER_VALUE);
    	messageElectionRequest.setMessageType(MessageType.DIRECTORY_ELECTION_PROCESS_REQUEST); 
    	messageElectionRequest.setMessageSource(new IPAddress(localProfile.getIPAddress()));
    	messageElectionRequest.setMessageInitiator(new IPAddress(localProfile.getIPAddress()));
    	messageElectionRequest.setMessagePort(COMMUNICATION_PORT_VALUE);
    	int hops2 = HOPS_NUMBER_VALUE.getInt();
    	BigBitSet hopsBits = new BigBitSet(hops2*2);
    	BitSet2[] bitSetHops = hopsBits.getBitSet(); 
    	messageElectionRequest.setMessageHop(bitSetHops[0]);
    	messageElectionRequest.setMessageHopCounter(new BitSet2());
    	messageElectionRequest.setMessageContent(localProfile.getBytes());
    	manager.instantaneousBroadcastMessage(messageElectionRequest, COMMUNICATION_PORT_VALUE);
    	// read Election Response Message & Election Request to may be interromp election
    	DIRECTORY_ELECTION_PROCESS_MSG_NUMBER_VALUE++;
    	NodeProfile[] directories = null;
    	Message[] messages = null;
    	try
	    {
    		int counter =0;
    		//manager.dontRemove(true);
    		long timeToWait = ELECTION_WAITING_RESPONSE_TIME_VALUE*hops2*2;
    		logger.debug("Election wait "+timeToWait);
    		while (counter < timeToWait)
    		{
    			messages = manager.getMessagesType(MessageType.DIRECTORY_ELECTION_PROCESS_REQUEST);
    			if (mustToStop(messages)) 
				{
					logger.debug("Election.election: I STOP MY ELECTION");
					//if (localProfile.getBenefit()>=MIN_PERFORMANCE_THRESHOLD_VALUE)
					//{
						Message messageElectionResponse = MessageFactory.createMessage();
						messageElectionResponse.setMessageNumber(DIRECTORY_ELECTION_PROCESS_RESPONSE_MSG_NUMBER_VALUE);
						messageElectionResponse.setMessageType(MessageType.DIRECTORY_ELECTION_PROCESS_RESPONSE); 
						messageElectionResponse.setMessageSource(new IPAddress(localProfile.getIPAddress()));
						messageElectionResponse.setMessagePort(COMMUNICATION_PORT_VALUE);
						messageElectionResponse.setMessageHop(HOPS_NUMBER_VALUE);
						messageElectionResponse.setMessageHopCounter(new BitSet2());
						
								
						logger.debug("I will send my Profile and the others election response");
						long totalTime=0;
						for (int i=0; i<messages.length; i++) 
						{
							byte[] byteProfile = null;
							localProfile.setHopsNumberFromInitiatir(messages[i].getMessageHopCounter().getInt()+1);
							localProfile.profileUpdate();
							byteProfile = localProfile.getBytes();
														
							byte[] toSend = new byte[0];
				    		int totalLength = 0;
				        	if (byteProfile!=null) totalLength = 4+byteProfile.length;
				    		
				        	try
							{
				    			long timeToWait2 = ELECTION_WAITING_RESPONSE_TIME_VALUE*(messages[i].getMessageHop().getInt()-(messages[i].getMessageHopCounter().getInt()+1));
				    						    			
				    			if (timeToWait2 > totalTime) timeToWait2 = timeToWait2 - totalTime;
				    			else timeToWait2 = 0;
				    			Thread.sleep(timeToWait2);
				    			totalTime = totalTime+timeToWait2;
							}
				    		catch(Exception ex)
							{
				    			logger.error("at waiting for election response from the others nodes");
							}
				    		
				    		Message[] responseMessages = manager.getMessagesType(MessageType.DIRECTORY_ELECTION_PROCESS_RESPONSE, messages[i].getMessageNumber(), messages[i].getMessageInitiator());
				    		if (responseMessages != null)
				    		{
				    			for (int r=0; r<responseMessages.length; r++)
				    				if (responseMessages[r].getMessageContent()!=null ) totalLength = totalLength+responseMessages[r].getMessageContent().length;
				    		}
				    		
				    		toSend = new byte[totalLength];
				    		int position=0;
				    		if (byteProfile!=null) 
				    		{
				    			BigBitSet profileLength = new BigBitSet(byteProfile.length);
				    			BitSet2[] bitSet = profileLength.getBitSet();
				    			toSend[0]=bitSet[0].getByte();
				    			toSend[1]=bitSet[1].getByte();
				    			toSend[2]=bitSet[2].getByte();
				    			toSend[3]=bitSet[3].getByte();
				    			position = 4;
				    			for (int z=0; z< byteProfile.length ; z++)
				    			{
				    				toSend[position]= byteProfile[z];
				    			    position++;
				    			}
				    		}
				    				
				    		if (responseMessages != null)
				    		{
				    			for (int r=0; r<responseMessages.length; r++)
				    				if (responseMessages[r].getMessageContent()!=null) 
				    					{
				    					/*BigBitSet contentLength = new BigBitSet(responseMessages[r].getMessageContent().length);
				    					BitSet2[] contentBitSet = contentLength.getBitSet();
				    	    			toSend[position] = contentBitSet[0].getByte(); position++;
				    	    			toSend[position] = contentBitSet[1].getByte(); position++;
				    	    			toSend[position] = contentBitSet[2].getByte(); position++;
				    	    			toSend[position] = contentBitSet[3].getByte(); position++;
				    	    			*/
				    	    			for (int z=0; z< responseMessages[r].getMessageContent().length ; z++)
				    	    			{
				    	    				toSend[position]= responseMessages[r].getMessageContent()[z];
				    	    				position++;
				    	    			}
				    					}
				    		}
				    		if (toSend != null && toSend.length>4)
				    		{
				    			messageElectionResponse.setMessageNumber(messages[i].getMessageNumber());
				    			messageElectionResponse.setMessageInitiator(messages[i].getMessageInitiator());
				    			messageElectionResponse.setMessageContent(toSend);
				    			manager.sendInstantaneousMessageTo(messageElectionResponse, messages[i].getMessageSource().getHostIPAddress(), COMMUNICATION_PORT_VALUE);
				    		}
						}
						DIRECTORY_ELECTION_PROCESS_RESPONSE_MSG_NUMBER_VALUE++;
					//}
					Thread.sleep(timeToWait-counter);
					return null;
				}
    			messages = manager.getMessagesType(MessageType.DIRECTORY_ADVERTISEMENT);
    			if (messages !=null  && messages.length!=0) 
    		    {
    				directories = new NodeProfile[messages.length];
    				for (int i=0; i<messages.length;i++)
    					directories[i]= new NodeProfile(messages[i].getMessageContent());
    				return directories;
    		    }
    			messages = manager.getMessagesType(MessageType.DIRECTORY_ELECTION_PROCESS_RESPONSE, messageElectionRequest.getMessageNumber(), messageElectionRequest.getMessageInitiator());
    			addCandidates(messages);
    			Thread.sleep(50);
    			counter = counter + 50;
    		}
	    }
		catch (Exception ex)
		{
			//ex.printStackTrace();
			logger.error("Election wait error");
		}
		
		directories = new NodeProfile[1];
		if (candidates.size()==0) 
		{	
			//if (localProfile.getBenefit()<MIN_PERFORMANCE_THRESHOLD_VALUE) return null;
			directories[0] = localProfile;
		}
		else 
	    {
			try
			{
			logger.debug("some nodes are candidate to become Directories");
			NodeProfile[] nodeProfiles = null;
			int s=0; 
			//if (localProfile.getBenefit()> MIN_PERFORMANCE_THRESHOLD_VALUE)
			//{
				//logger.debug("my node candidate too candidates.size()="+candidates.size());
				nodeProfiles = new NodeProfile[candidates.size()+1];
				nodeProfiles[0] = localProfile;
				logger.debug("localProfile ip="+localProfile.getIPAddress() +" is add as candidate");
				s=1;
			//}	
			//else nodeProfiles = new NodeProfile[candidates.size()];
		
			for (int j=0; j<candidates.size(); j++)
		    {
				nodeProfiles[j+s] = (NodeProfile)candidates.elementAt(j);
				logger.debug("NodeProfile ip="+nodeProfiles[j+s].getIPAddress() +" is add as candidate");
		    }
			
			if (nodeProfiles == null) return null;
			
			if (nodeProfiles.length>1) 
				{
				directories = electionChoice(nodeProfiles);
				logger.debug("-------directories choosed are :");
				int m=0; 
				while (m<directories.length && !directories[m].getIPAddress().equals(localProfile.getIPAddress()))
					m++;
				if (m>0 && m<directories.length) 
				{
					NodeProfile tmp = directories[0];
					directories[0] = directories[m];
					directories[m] = tmp;
				}
					
				}
			else directories[0] = nodeProfiles[0];
			
			Message messageElect = MessageFactory.createMessage();
			messageElect.setMessageNumber(DIRECTORY_ELECT_MSG_NUMBER_VALUE);
			messageElect.setMessageType(MessageType.DIRECTORY_ELECT);
			messageElect.setMessageSource(new IPAddress(localProfile.getIPAddress()));
			messageElect.setMessagePort(COMMUNICATION_PORT_VALUE);
			messageElect.setMessageHop(HOPS_NUMBER_VALUE);
			messageElect.setMessageHopCounter(new BitSet2());
			// send electe message for all nodes excepte the local node
			for (int m=0; m<directories.length; m++)
			if (!directories[m].getIPAddress().equals(localProfile.getIPAddress()))
		    {			
				int advertisementHops = directories[m].getIdealRay();
				if (advertisementHops==0) advertisementHops=1;
				BigBitSet hops = new BigBitSet(advertisementHops);
				byte[] content = new byte[1];
				content[0]= hops.getBitSet()[0].getByte();
				messageElect.setMessageContent(content);
				manager.sendInstantaneousMessageTo(messageElect, directories[m].getIPAddress(), COMMUNICATION_PORT_VALUE);
				
		    }
			}
			catch(NullPointerException ex)
			{
				logger.error("Null pointer Exception in election choice");
				//ex.printStackTrace();
			}
	    }
	//manager.stop();
	return  directories;
    }	

    private void addCandidates(Message[] messages)
    {
    	if (messages == null) return; 
    	for (int j=0; j<messages.length; j++)
	    {
		logger.debug("candidate Messages are received from "+messages[j].getMessageSource().getHostIPAddress());
		byte[] byteProfiles = messages[j].getMessageContent();
		//logger.debug("Total length of received profiles="+byteProfiles.length);
		if (byteProfiles !=null && byteProfiles.length>=4)
		{
			int i=0;
			while (i<byteProfiles.length)
			{
				BitSet2[] bitSet = new BitSet2[4];
				bitSet[0] = new BitSet2(byteProfiles[i]); i++;
				bitSet[1] = new BitSet2(byteProfiles[i]); i++;
				bitSet[2] = new BitSet2(byteProfiles[i]); i++;
				bitSet[3] = new BitSet2(byteProfiles[i]); i++;
				BigBitSet bigBitSet = new BigBitSet(bitSet);
				int profileLength = bigBitSet.getInt();
				//logger.debug("received profile length="+profileLength);
				byte[] byteProfile = new byte[profileLength];
				for (int t=0; t<profileLength; t++)
				{
				 byteProfile[t]= byteProfiles[i];
				 i++;
				}
			
				NodeProfile nodeProfile = new NodeProfile(byteProfile); 
				if (!nodeProfile.getIPAddress().equals(localProfile.getIPAddress())) candidates.add(nodeProfile);
			}
		}
	    }
    }
            
    private String getIPOf(String iplevel)
    {
    	return iplevel.substring(0, iplevel.indexOf(":"));
    }
    
    private int getLevelOf(String iplevel)
    {
    	int index = iplevel.indexOf(":");
    	if (index ==-1 ) return -1;
        return Integer.valueOf(iplevel.substring(index+1).trim()).intValue();
    }
    
    private void addNotCoveredSDTo(String[][] list, int line , String ip, int level)
    {
    	if (ip.equals(list[line][0])) return;
    	for (int i=1;i<500;i++)
    		if (list[line][i]!=null && getIPOf(list[line][i]).equals(ip)) return;
    	int j=1;
    	while (j<201 && list[line][j]!= null && !list[line][j].equals("")) j++;
    	if (j<201) list[line][j] = ip+":"+level;
    }
   
    
    private void addSDTo(String[][] list, int line , String ip, int level)
    {
    	if (ip.equals(list[line][0])) return;
    	for (int i=1;i<500;i++)
    		if (list[line][i]!=null && getIPOf(list[line][i]).equals(ip)) return;
    	int j=1;
    	while (j<401 && list[line][j]!= null && !list[line][j].equals("")) j++;
    	if (j<401) list[line][j] = ip+":"+level;
    }
    
    private void addDirTo(String[][] list, int line, String ip, int level)
    {
    	if (ip.equals(list[line][0])) return;
    	for (int i=1;i<500;i++)
    		if (list[line][i]!=null && getIPOf(list[line][i]).equals(ip)) return;
    	int j=1;
    	while (j<500 && list[line][j]!= null && !list[line][j].equals("")) j++;
    	if (j<500) list[line][j] = ip+":"+level;
    }
    
    public NodeProfile[] electionChoice(NodeProfile[] nodeProfiles)
    {
    	//  return the better Profile
    	if (nodeProfiles == null || nodeProfiles.length==0) return null;
    	String[][] list = new String[nodeProfiles.length][500];
    	int[] level = new int[nodeProfiles.length];
    	for (int i=0;i<nodeProfiles.length;i++)
	    {
	      list[i][0] = nodeProfiles[i].getIPAddress();
	      level[i] = nodeProfiles[i].getHopsNumberFromInitiator();
	      
	      IPAddress[] notCoveredSDIPAddress = nodeProfiles[i].getNeighborsNotCoveredSDAddresses();
	      int k=0;
	      for (k=0; k<notCoveredSDIPAddress.length;k++)
	      	list[i][k+1] = notCoveredSDIPAddress[k].getHostIPAddress()+":1";
	      //logger.debug("notCoveredSDIPAddress.length="+k);
	      
	      IPAddress[] SDIPAddress = nodeProfiles[i].getNeighborsSDAddresses();
	      for (k=0; k<SDIPAddress.length;k++)
	      	list[i][k+201] = SDIPAddress[k].getHostIPAddress()+":1";
	      //logger.debug("SDIPAddress.length="+k);
	      
	      IPAddress[] DirIPAddress = nodeProfiles[i].getNeighborsDirectoriesAddresses();
	      for (k=0; k<DirIPAddress.length;k++)
	      	list[i][k+401] = DirIPAddress[k].getHostIPAddress()+":1";
	      //logger.debug("DirIPAddress.length="+k);
	    }
    	
    	try{
    	
    	for (int lev = 1; lev <INT_HOPS_NUMBER_VALUE ; lev++)
    	for (int i=0; i<list.length;i++)
    	{
    		for (int j=1; j<500; j++)
    		{
    			if (list[i][j]!= null && !list[i][j].equals("") && getLevelOf(list[i][j]) == lev)
    			{
    				String ip = getIPOf(list[i][j]);
    				int k=0;
    				while (k<list.length && (list[k][0]== null || !list[k][0].equals(ip)))
    						k++;
    				if (k<list.length)
    				{
    					int l=1;
    					while (l<201 && list[k][l]!= null && getLevelOf(list[k][l])==1)
    					{
    						addNotCoveredSDTo(list, i,getIPOf(list[k][l]) ,lev+1);
    						l++;
    					}
    					l=201;
    					while (l<401 && list[k][l]!= null && getLevelOf(list[k][l])==1)
    					{
    						addSDTo(list, i,getIPOf(list[k][l]) ,lev+1);
    						l++;
    					}
    					l=401;
    					while (l<201 && list[k][l]!= null && getLevelOf(list[k][l])==1)
    					{
    						addDirTo(list, i,getIPOf(list[k][l]) ,lev+1);
    						l++;
    					}
    						
    				}
    			}
    		}
    			
    	}
    	}
    	catch(Exception ex )
		{
    		ex.printStackTrace();
		}
    	
    	
    	
    	
	// return the better Profile
    Vector selectedProfilesVect = new Vector();
    NodeProfile[] selectedProfiles = new NodeProfile[0];
    int[] ProfileRay = new int[10]; // max 10 nodes selected 
    int profileIndex =0;
    while (isThereNotCoveredSD(list, level))
    {
    NodeProfile forReturn = nodeProfiles[0];
    int position=0;
	for (int i=1;i<nodeProfiles.length;i++)
	    {
		//if (betterProfileComparedTo(nodeProfiles[i], forReturn)) forReturn=nodeProfiles[i];
		if (nodeProfiles[i].getHopsNumberFromInitiator()<=INT_HOPS_NUMBER_VALUE && betterProfileComparedTo(list, list[i], list[position], nodeProfiles[i], forReturn)) 
			{
			forReturn = nodeProfiles[i];
			position = i;
			}
	    }
	selectedProfilesVect.add(forReturn);
	ProfileRay[profileIndex]= forReturn.getIdealRay();
	profileIndex++;
	System.out.println("Election.election: the Profile of "+forReturn.getIPAddress()+" is chosen" + "ray="+forReturn.getIdealRay());
	
	// delete all not covered SD will be covered by the selected node
	Vector tmpIP= new Vector();
	tmpIP.add(forReturn.getIPAddress());
	for (int k=1; k<201; k++)
	{
		if (list[position][k]!=null && !list[position][k].equals(""))
		{
			tmpIP.add(getIPOf(list[position][k]));
			list[position][k]=null;
		}
	}
	removeNotCoveredSD(list, tmpIP);
    
    }
    
    selectedProfiles = new NodeProfile[selectedProfilesVect.size()];
    
    for (int k=0; k<selectedProfilesVect.size();k++)
	{
    	selectedProfiles[k]=(NodeProfile)selectedProfilesVect.elementAt(k);
    	selectedProfiles[k].setIdealRay(ProfileRay[k]);
	}
    
	return selectedProfiles;
  	
    }

    private boolean isThereNotCoveredSD(String[][] list , int[] level)
    {
    	for (int i=0; i<list.length; i++)
    	{
    		if (level[i] <=INT_HOPS_NUMBER_VALUE)
    		for (int j=1; j<201 ;j++)
    			if (list[i][j]!=null && !list[i][j].equals("")) return true;
    	}
    	//logger.debug("There arn't any not covered nodes*****");
    	return false;
    }
    
    private void removeNotCoveredSD(String[][] list, Vector ips)
    {
    	for (int i=0; i<list.length; i++)
    	{
    		for (int j=1; j<201 ;j++)
    			if (list[i][j]!=null && ips.contains(getIPOf(list[i][j]))) list[i][j]=null ;
    	}
    }
    
    private boolean mustToStop(Message[] messages)
    {
	//return true if the there is a message which has small IP address
	if (messages == null || messages.length == 0) return false;
	for (int i=0;i<messages.length; i++)
	    {
		NodeProfile Profile = new NodeProfile(messages[i].getMessageContent());
		
		if (smallerThanIPAddress(Profile.getIPAddress(), localProfile.getIPAddress())) 
		    {
			logger.debug("Election stoped");
			return true;
		    }
	    }
	return false;
    }

    static boolean betterProfileComparedTo(NodeProfile Profile1 , NodeProfile Profile2)
    {
	//System.out.println("comparaison between Profiles");
	//return true if Profile1 is better than Profile2
    	logger.debug("Profile1 IP="+Profile1.getIPAddress());
    	logger.debug("Profile2 IP="+Profile2.getIPAddress());
    	int NotCoveredSDNeighborsProfile1 = Profile1.getNeighborsNotCoveredSDAddresses().length;
    	int NotCoveredSDNeighborsProfile2 = Profile2.getNeighborsNotCoveredSDAddresses().length;
    	if (Profile1.getNeighborsDirectoriesAddresses().length==0) NotCoveredSDNeighborsProfile1++;
    	if (Profile2.getNeighborsDirectoriesAddresses().length==0) NotCoveredSDNeighborsProfile2++;
    	logger.debug("NotCoveredSDNeighborsProfile1="+NotCoveredSDNeighborsProfile1);
    	logger.debug("NotCoveredSDNeighborsProfile2="+NotCoveredSDNeighborsProfile2);
    	// Max (NotCoveredSDNeighborsNumber)
    	if (NotCoveredSDNeighborsProfile1>NotCoveredSDNeighborsProfile2) return true;
    	else if (NotCoveredSDNeighborsProfile1<NotCoveredSDNeighborsProfile2) return false;
    	
    	// Min IdealRay
    	int IdealRayProfile1 = Profile1.getIdealRay();
    	int IdealRayProfile2 = Profile2.getIdealRay();
    	logger.debug("IdealRayProfile1="+IdealRayProfile1);
    	logger.debug("IdealRayProfile2="+IdealRayProfile2);
    	if (IdealRayProfile1<IdealRayProfile2) return true;
    	if (IdealRayProfile1>IdealRayProfile2) return false;
    	
    	int DirNeighborsOfNeighborsNumberProfile1 = Profile1.getDirNeighborsOfMyNeighbors();
    	int DirNeighborsOfNeighborsNumberProfile2 = Profile2.getDirNeighborsOfMyNeighbors();
    	logger.debug("DirNeighborsOfNeighborsNumberProfile1="+DirNeighborsOfNeighborsNumberProfile1);
    	logger.debug("DirNeighborsOfNeighborsNumberProfile2="+DirNeighborsOfNeighborsNumberProfile2);
    	// Min (DirNeighborsOfNeighborsNumber)
    	//if (DirNeighborsOfNeighborsNumberProfile1<DirNeighborsOfNeighborsNumberProfile2) return true;
    	//else if (DirNeighborsOfNeighborsNumberProfile1>DirNeighborsOfNeighborsNumberProfile2) return false;
    	
    	
    	int DirNeighborsProfile1 = Profile1.getNeighborsDirectoriesAddresses().length;
    	int TotalSDNeighborsProfile1 = Profile1.getNeighbors()-DirNeighborsProfile1;
    	int DirNeighborsProfile2 = Profile2.getNeighborsDirectoriesAddresses().length;
    	int TotalSDNeighborsProfile2 = Profile2.getNeighbors()-DirNeighborsProfile2;
    	logger.debug("TotalSDNeighborsProfile1="+TotalSDNeighborsProfile1);
    	logger.debug("TotalSDNeighborsProfile2="+TotalSDNeighborsProfile2);
    	//MAX (TotalNeighborsNumber - DirNeighborsNumber)
    	//if (TotalSDNeighborsProfile1>TotalSDNeighborsProfile2) return true;
    	//else if (TotalSDNeighborsProfile1<TotalSDNeighborsProfile2) return false;
    	
    	//calculate  DirNeighborsOfNeighborsNumber/TotalSDNeighborsProfile
    	
    	if (DirNeighborsOfNeighborsNumberProfile1==0) 
    		{
    		if (DirNeighborsOfNeighborsNumberProfile2!=0) return true;
    		if (TotalSDNeighborsProfile1>TotalSDNeighborsProfile2) return true;
        	else if (TotalSDNeighborsProfile1<TotalSDNeighborsProfile2) return false;
    		float benefit1 = Profile1.getBenefit();
        	float benefit2 = Profile2.getBenefit();
        	logger.debug("BenefitProfile1="+benefit1);
        	logger.debug("BenefitProfile2="+benefit2);
        	// compare the benefit
        	if (benefit1<benefit2) return false;
        	return true;
    		}
    	
    	if (DirNeighborsOfNeighborsNumberProfile2==0) return false;
    	
    	float floatTotalSDNeighborsProfile1 = TotalSDNeighborsProfile1;
    	float floatDirNeighborsOfNeighborsNumberProfile1 = DirNeighborsOfNeighborsNumberProfile1;
    	float floatTotalSDNeighborsProfile2 = TotalSDNeighborsProfile2;
    	float floatDirNeighborsOfNeighborsNumberProfile2 = DirNeighborsOfNeighborsNumberProfile2;
    	
    	float SDPerDir1 = floatTotalSDNeighborsProfile1/floatDirNeighborsOfNeighborsNumberProfile1;
    	float SDPerDir2 = floatTotalSDNeighborsProfile2/floatDirNeighborsOfNeighborsNumberProfile2;
    	logger.debug("SDPerDir1="+SDPerDir1);
    	logger.debug("SDPerDir2="+SDPerDir2);
    	
    	if (SDPerDir1>SDPerDir2) return true;
    	else if (SDPerDir1<SDPerDir2) return false;
    		   	
    	    	
    	float benefit1 = Profile1.getBenefit();
    	float benefit2 = Profile2.getBenefit();
    	logger.debug("BenefitProfile1="+benefit1);
    	logger.debug("BenefitProfile2="+benefit2);
    	// compare the benefit
    	if (benefit1<benefit2) return false;
    	return true;
    	/*// OLD
    	int NbDirNeighborsProfile1 =0;
    	if (Profile1.getNeighborsDirectoriesAddresses()!=null) NbDirNeighborsProfile1 = Profile1.getNeighborsDirectoriesAddresses().length;
    	int NbDirNeighborsProfile2 =0;
    	if (Profile2.getNeighborsDirectoriesAddresses()!=null) NbDirNeighborsProfile2 = Profile2.getNeighborsDirectoriesAddresses().length;
    	    	
    	if (NbDirNeighborsProfile1==0 && NbDirNeighborsProfile2!=0) return true;
    	if (NbDirNeighborsProfile1!=0 && NbDirNeighborsProfile2==0) return false;
    	
    	int NbDirOfNeighborsProfile1 = Profile1.getDirNeighborsOfMyNeighbors();
    	int NbDirOfNeighborsProfile2 = Profile2.getDirNeighborsOfMyNeighbors();
    	
    	if (NbDirOfNeighborsProfile1==0 && NbDirOfNeighborsProfile2!=0) return true;
    	if (NbDirOfNeighborsProfile1!=0 && NbDirOfNeighborsProfile2==0) return false;
    	
    	int NbSDNeighborsProfile1= Profile1.getSDNeighborsOfMyNeighbors();
    	int NbSDNeighborsProfile2= Profile2.getSDNeighborsOfMyNeighbors();
    	
    	if (NbDirOfNeighborsProfile1==0 && NbDirOfNeighborsProfile2==0)
    		if (NbSDNeighborsProfile1>NbSDNeighborsProfile2) return true;
    		else if (NbSDNeighborsProfile1<NbSDNeighborsProfile2) return false;
    	    else if (Profile1.getBenefit()<Profile2.getBenefit()) return false; 
    	    	
        	else return true;
    	
    	double CoverageBenefit1 = NbSDNeighborsProfile1/NbDirOfNeighborsProfile1; 
    	double CoverageBenefit2 = NbSDNeighborsProfile2/NbDirOfNeighborsProfile2;
    	if (CoverageBenefit1>CoverageBenefit2) return true;
    	else if (CoverageBenefit1<CoverageBenefit2) return false;
    	
    	if (Profile1.getBenefit()<Profile2.getBenefit()) return false;
    	return true;
    	*/
    }

    private boolean betterProfileComparedTo(String[][] list, String [] node1,String[] node2, NodeProfile Profile1, NodeProfile Profile2)
    {
	//System.out.println("comparaison between nodes");
	//return true if node1 is better than node2
    	logger.debug("node1 IP="+node1[0]);
    	logger.debug("node2 IP="+node2[0]);
    	
    	int NotCoveredSDNeighborsNode1 = 0;
    	int NotCoveredSDNeighborsNode2 = 0;
    	int IdealRayNode1 = 0;
    	int IdealRayNode2 = 0;
    	
    	int i=1;
    	while (i<201)
    	{
    		if (node1[i]!=null && !node1[i].equals("")) 
    		{
    		int level = getLevelOf(node1[i]);
    		if (level > IdealRayNode1) IdealRayNode1 = level;
    		NotCoveredSDNeighborsNode1++;
    		}
    		i++;
    	}
    	
    	i=1;
    	while (i<201)
    	{
    		if(node2[i]!=null && !node2[i].equals(""))
    		{
    		int level = getLevelOf(node2[i]);
    		if (level > IdealRayNode2) IdealRayNode2 = level;
    		NotCoveredSDNeighborsNode2++;
    		}
    		i++;
    	}
    	
    	Profile1.setIdealRay(IdealRayNode1);
    	Profile2.setIdealRay(IdealRayNode2);
    	
    	int SDNeighborsNode1 = 0;
    	int SDNeighborsNode2 = 0;
    	
    	i=201;
    	while (i<401)
    	{
    		if (node1[i]!=null && !node1[i].equals("")) 
    			SDNeighborsNode1++;
    		i++;
    	}
    	i=201;
    	while (i<401)
    	{
    		if (node2[i]!=null && !node2[i].equals("")) 
    			SDNeighborsNode2++;
    		i++;
    	}
    	
    	
    	int DirNeighborsNode1 = 0;
    	int DirNeighborsNode2 = 0;
    	
    	i=401;
    	while (i<500)
    	{
    		if (node1[i]!=null && !node1[i].equals("")) 
    			DirNeighborsNode1++;
    		i++;
    	}
    	i=401;
    	while (i<500)
    	{
    		if (node2[i]!=null && !node2[i].equals("")) 
    		   	DirNeighborsNode2++;
    		i++;
    	}
    	
    	int itselfNode1=0;
    	int itselfNode2=0;
    	
    	if (DirNeighborsNode1==0) 
    		{
    			NotCoveredSDNeighborsNode1++;
    			itselfNode1=1;
    			if (IdealRayNode1==0) IdealRayNode1=1;
    		}
    	else if (IdealRayNode1==0) IdealRayNode1=2; 
    		
    	if (DirNeighborsNode2==0) 
    		{
    		NotCoveredSDNeighborsNode2++;
    		itselfNode2=1;
    		if (IdealRayNode2==0) IdealRayNode2=1;
    		}
    	else if (IdealRayNode2==0) IdealRayNode2=2;
    	
    	logger.debug("NotCoveredSDNeighborsNode1="+NotCoveredSDNeighborsNode1);
    	logger.debug("NotCoveredSDNeighborsNode2="+NotCoveredSDNeighborsNode2);
    	// Max (NotCoveredSDNeighborsNumber)
    	if (NotCoveredSDNeighborsNode1>NotCoveredSDNeighborsNode2) return true;
    	else if (NotCoveredSDNeighborsNode1<NotCoveredSDNeighborsNode2) return false;
    	
    	// Min IdealRay
    	
    	logger.debug("fectif:IdealRayNode1="+IdealRayNode1);
    	logger.debug("fectif:IdealRayNode2="+IdealRayNode2);
    	logger.debug("Real:Profile1.getIdealRay()="+Profile1.getIdealRay());
    	logger.debug("Real:Profile2.getIdealRay()="+Profile2.getIdealRay());
    	if (IdealRayNode1<IdealRayNode2) return true;
    	if (IdealRayNode1>IdealRayNode2) return false;
    	   	
    	int DirNeighborsOfNeighborsNumberNode1 = 0;
    	int DirNeighborsOfNeighborsNumberNode2 = 0;
    	
    	Vector dirTmp = new Vector();
    	
    	for(i=1; i<500; i++)
    	{
    		if (node1[i]!=null && !node1[i].equals(""))
    		{
    			int k=0;
    			while (k<list.length && !list[k][0].equals(getIPOf(node1[i]))) k++;
    			if (k<list.length)
    			{
    				for (int z=401; z<500; z++)
    				{
    					if (list[k][z]!=null && ! list[k][z].equals(""))
    					{
    						String ip = getIPOf(list[k][z]);
    						if (!dirTmp.contains(ip)) dirTmp.add(ip);
    					}
    				}
    			}
    		}
    	}
    	
    	DirNeighborsOfNeighborsNumberNode1 = dirTmp.size();
    	
    	dirTmp.removeAllElements();
    	
    	for(i=1; i<500; i++)
    	{
    		if (node2[i]!=null && !node2[i].equals(""))
    		{
    			int k=0;
    			while (k<list.length && !list[k][0].equals(getIPOf(node2[i]))) k++;
    			if (k<list.length)
    			{
    				for (int z=401; z<500; z++)
    				{
    					if (list[k][z]!=null && !list[k][z].equals(""))
    					{
    						String ip = getIPOf(list[k][z]);
    						if (!dirTmp.contains(ip)) dirTmp.add(ip);
    					}
    				}
    			}
    		}
    	}
    	
    	DirNeighborsOfNeighborsNumberNode2 = dirTmp.size();
    	dirTmp.removeAllElements();
    	logger.debug("DirNeighborsOfNeighborsNumberNode1="+DirNeighborsOfNeighborsNumberNode1);
    	logger.debug("DirNeighborsOfNeighborsNumberNode2="+DirNeighborsOfNeighborsNumberNode2);
    	// Min (DirNeighborsOfNeighborsNumber)
    	//if (DirNeighborsOfNeighborsNumberNode1<DirNeighborsOfNeighborsNumberNode2) return true;
    	//else if (DirNeighborsOfNeighborsNumberNode1>DirNeighborsOfNeighborsNumberNode2) return false;
    	
    	
    	int TotalSDNeighborsNode1 = SDNeighborsNode1+NotCoveredSDNeighborsNode1-itselfNode1; 
    	int TotalSDNeighborsNode2 = SDNeighborsNode2+NotCoveredSDNeighborsNode2-itselfNode2;
    	
    	logger.debug("TotalSDNeighborsNode1="+TotalSDNeighborsNode1);
    	logger.debug("TotalSDNeighborsNode2="+TotalSDNeighborsNode2);
    	//MAX (TotalNeighborsNumber - DirNeighborsNumber)
    	//if (TotalSDNeighborsProfile1>TotalSDNeighborsProfile2) return true;
    	//else if (TotalSDNeighborsProfile1<TotalSDNeighborsProfile2) return false;
    	
    	//calculate  DirNeighborsOfNeighborsNumber/TotalSDNeighborsProfile
    	
    	if (DirNeighborsOfNeighborsNumberNode1==0) 
    		{
    		if (DirNeighborsOfNeighborsNumberNode2!=0) return true;
    		if (TotalSDNeighborsNode1>TotalSDNeighborsNode2) return true;
        	else if (TotalSDNeighborsNode1<TotalSDNeighborsNode2) return false;
    		float benefit1 = Profile1.getBenefit();
        	float benefit2 = Profile2.getBenefit();
        	logger.debug("BenefitProfile1="+benefit1);
        	logger.debug("BenefitProfile2="+benefit2);
        	// compare the benefit
        	if (benefit1<benefit2) return false;
        	return true;
    		}
    	
    	if (DirNeighborsOfNeighborsNumberNode2==0) return false;
    	
    	float floatTotalSDNeighborsNode1 = TotalSDNeighborsNode1;
    	float floatDirNeighborsOfNeighborsNumberNode1 = DirNeighborsOfNeighborsNumberNode1;
    	float floatTotalSDNeighborsNode2 = TotalSDNeighborsNode2;
    	float floatDirNeighborsOfNeighborsNumberNode2 = DirNeighborsOfNeighborsNumberNode2;
    	
    	float SDPerDir1 = floatTotalSDNeighborsNode1/floatDirNeighborsOfNeighborsNumberNode1;
    	float SDPerDir2 = floatTotalSDNeighborsNode2/floatDirNeighborsOfNeighborsNumberNode2;
    	logger.debug("SDPerDir1="+SDPerDir1);
    	logger.debug("SDPerDir2="+SDPerDir2);
    	
    	if (SDPerDir1>SDPerDir2) return true;
    	else if (SDPerDir1<SDPerDir2) return false;
    		   	
    	    	
    	float benefit1 = Profile1.getBenefit();
    	float benefit2 = Profile2.getBenefit();
    	logger.debug("BenefitProfile1="+benefit1);
    	logger.debug("BenefitProfile2="+benefit2);
    	// compare the benefit
    	if (benefit1<benefit2) return false;
    	return true;
       }

    
    static boolean smallerThanIPAddress(String IPAddress1 , String IPAddress2)
    {
	int position1=0;
	int position2=0;
	String IP1 = IPAddress1+".";
	String IP2 = IPAddress2+".";
	logger.debug("Election.smallerIP:IP1="+IP1);
	logger.debug("Election.smallerIP:IP2="+IP2);
	for (int i=0;i<4;i++)
	    {
		String tmp = IP1.substring(position1,IP1.indexOf(".", position1));
		Integer tmpInt1 = new Integer(tmp);
		tmp = IP2.substring(position2,IP2.indexOf(".", position2));
		Integer tmpInt2 = new Integer(tmp);
		if (tmpInt1.intValue()<tmpInt2.intValue()) return true;
		else if (tmpInt1.intValue()>tmpInt2.intValue()) return false;
		position1 =  IP1.indexOf(".", position1)+1;
		position2 =  IP2.indexOf(".", position2)+1;
	    }
	return false; // equals
    }
}
