package ubisec.discovery;


import java.util.*;

import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.SimpleLayout;

import ubisec.discovery.message.*;
import ubisec.discovery.message.util.*;
import ubisec.profile.*;
import ubisec.util.*;
import ubisec.net.IPAddress;
import ubisec.configuration.Setting;

/**
*
* SD.java
*
* @author Rafik CHIBOUT 
* @author Francoise SAILHAN
* @version 0.1 oct. 2004
*
*  
*/

public class SD
{
	private static int LISTENING_PERIOD=0;
    private static int ADVERTISEMENT_PERIOD=1;
    private static int COMMUNICATION_PORT=2;
    private static int DIRECTORY_ELECTION_PROCESS_MSG_NUMBER=3;
    private static int DIRECTORY_ELECTION_PROCESS_RESPONSE_MSG_NUMBER=4;
    private static int DIRECTORY_ELECT_MSG_NUMBER=5;
    private static int GET_SERVICE_FROM_SD_MSG_NUMBER=6;
    private static int DIRECTORY_REGISTER_SERVICE_MSG_NUMBER=7;
    private static int DIRECTORY_UNREGISTER_SERVICE_MSG_NUMBER=8;
    private static int ELECTION_WAITING_RESPONSE_TIME=9;
    private static int WAITING_GET_SERVICE_RESPONSE_MIN=10;
    private static int WAITING_GET_SERVICE_RESPONSE_MAX=11;
    private static int PROFILE_PORT=12;
    private static int HOPS_NUMBER=13;
    private static int GET_SERVICE_REQUEST_HOPS=14;
    private static int CHECKING_PERFORMANCE_PERIOD=15;
    private static int MIN_PERFORMANCE_THRESHOLD=16;
    
    private static int LISTENING_PERIOD_VALUE=5;
    private static int ADVERTISEMENT_PERIOD_VALUE=6000; 
    private static int COMMUNICATION_PORT_VALUE=9999;
    private static int DIRECTORY_ELECTION_PROCESS_MSG_NUMBER_VALUE=0;
    private static int DIRECTORY_ELECTION_PROCESS_RESPONSE_MSG_NUMBER_VALUE=0;
    private static int DIRECTORY_ELECT_MSG_NUMBER_VALUE=0;
    private static int GET_SERVICE_FROM_SD_MSG_NUMBER_VALUE=0;
    private static int DIRECTORY_REGISTER_SERVICE_MSG_NUMBER_VALUE=0;
    private static int DIRECTORY_UNREGISTER_SERVICE_MSG_NUMBER_VALUE=0;
    private static int ELECTION_WAITING_RESPONSE_TIME_VALUE=2000;
    private static int WAITING_GET_SERVICE_RESPONSE_MIN_VALUE=50;
    private static int WAITING_GET_SERVICE_RESPONSE_MAX_VALUE=20000;
    private static int PROFILE_PORT_VALUE=5555;
    private static int INT_HOPS_NUMBER_VALUE=1;
    private static int INT_GET_SERVICE_REQUEST_HOPS_VALUE=1;
    private static int CHECKING_PERFORMANCE_PERIOD_VALUE=60000;
    private static float MIN_PERFORMANCE_THRESHOLD_VALUE=(new Float(0.5)).floatValue();
    
    Directory localDirectory               = null;
    private Hashtable directories          = null;
    private boolean hasDirectory           = false;
    public static MessagesManager manager  = null;
    public static NodeProfile localProfile = null;
    private boolean started                = false;
    private boolean stoped                 = false;
    private boolean findDirStarted         = false;
    private Setting setting                = null;
    private String deploymentManner        = "auto";      
    private Thread listening               = listeningProcess();
	private Thread performanceChecking     = PerformanceChecking();
	private Thread directoryChecking       = DirectroriesChecking();
	private boolean thereIsAnotherElection = false;
	
	
	private BitSet2 HOPS_NUMBER_VALUE=HOPS.N1;
	private BitSet2 GET_SERVICE_REQUEST_HOPS_VALUE=HOPS.N1;
	
	private static Logger logger = Logger.getLogger(SD.class); 
    private Level loggerLevel = Level.INFO;
	
    public SD()
    {
    	
    	SimpleLayout layout = new SimpleLayout();
    	ConsoleAppender appender = new ConsoleAppender(layout,ConsoleAppender.SYSTEM_OUT);
    	appender.setName("screen");
    	appender.activateOptions();
		logger.addAppender(appender);
    	logger.setLevel(loggerLevel);
    	
    	setting = new Setting();
    	
    	if (Setting.HopsNumber>1 && Setting.HopsNumber<128)
    	{
    		INT_HOPS_NUMBER_VALUE=Setting.HopsNumber;
    		BigBitSet tmp = new BigBitSet(INT_HOPS_NUMBER_VALUE);
    		HOPS_NUMBER_VALUE=tmp.getBitSet()[0];
    	}
    	
    	if (Setting.GetServiceRequestHops>0 && Setting.GetServiceRequestHops<128)
    	{
    		INT_GET_SERVICE_REQUEST_HOPS_VALUE=Setting.GetServiceRequestHops;
    		BigBitSet tmp = new BigBitSet(INT_GET_SERVICE_REQUEST_HOPS_VALUE);
    		GET_SERVICE_REQUEST_HOPS_VALUE=tmp.getBitSet()[0];
    	}
    	
    	if (Setting.SD_CHECKING_PERFORMANCE_PERIOD>0)
    		CHECKING_PERFORMANCE_PERIOD=Setting.SD_CHECKING_PERFORMANCE_PERIOD;
    	
    	if (Setting.MinPerformance>0)// && Setting.MaxDirectoryRequested<1)
    		MIN_PERFORMANCE_THRESHOLD_VALUE=Setting.MinPerformance;
    	
    	deploymentManner=Setting.DirectoryDeployment;
    	COMMUNICATION_PORT_VALUE=Setting.PortNumber;
    	PROFILE_PORT_VALUE=Setting.ProfilePort;
    	LISTENING_PERIOD_VALUE=Setting.SD_LISTENING_PERIOD_VALUE;
    	ADVERTISEMENT_PERIOD_VALUE=Setting.SD_ADVERTISEMENT_PERIOD_VALUE; 
    	ELECTION_WAITING_RESPONSE_TIME_VALUE=Setting.SD_ELECTION_WAITING_RESPONSE_TIME_VALUE;
    	WAITING_GET_SERVICE_RESPONSE_MIN_VALUE=Setting.SD_WAITING_GET_SERVICE_RESPONSE_MIN_VALUE;
    	WAITING_GET_SERVICE_RESPONSE_MAX_VALUE=Setting.SD_WAITING_GET_SERVICE_RESPONSE_MAX_VALUE;
    }

    public void setProperty(int propertyName, int propertyValue)
    {
    	switch (propertyName)
	    {
	    case 0:  LISTENING_PERIOD_VALUE=propertyValue;break;
	    case 1:  ADVERTISEMENT_PERIOD_VALUE=propertyValue;break;
	    case 2:  COMMUNICATION_PORT_VALUE=propertyValue;break;
	    case 3:  DIRECTORY_ELECTION_PROCESS_MSG_NUMBER_VALUE=propertyValue;break;
	    case 4:  DIRECTORY_ELECTION_PROCESS_RESPONSE_MSG_NUMBER_VALUE=propertyValue;break;
	    case 5:  DIRECTORY_ELECT_MSG_NUMBER_VALUE=propertyValue;break;
	    case 6:  GET_SERVICE_FROM_SD_MSG_NUMBER_VALUE=propertyValue;break;
	    case 7:  DIRECTORY_REGISTER_SERVICE_MSG_NUMBER_VALUE=propertyValue;break;
	    case 8:  DIRECTORY_REGISTER_SERVICE_MSG_NUMBER_VALUE=propertyValue;break;
	    case 9:  ELECTION_WAITING_RESPONSE_TIME_VALUE=propertyValue;break;
	    case 10: WAITING_GET_SERVICE_RESPONSE_MIN_VALUE=propertyValue;break;
	    case 11: WAITING_GET_SERVICE_RESPONSE_MAX_VALUE=propertyValue;break;
	    case 12: PROFILE_PORT_VALUE=propertyValue;break;
	    case 13: INT_HOPS_NUMBER_VALUE=propertyValue;
				 BigBitSet tmp = new BigBitSet(INT_HOPS_NUMBER_VALUE);
				 HOPS_NUMBER_VALUE=tmp.getBitSet()[0];
	             break;
	    case 14: INT_GET_SERVICE_REQUEST_HOPS_VALUE=propertyValue;
	    		 BigBitSet tmp2 = new BigBitSet(INT_GET_SERVICE_REQUEST_HOPS_VALUE);
	    		 GET_SERVICE_REQUEST_HOPS_VALUE=tmp2.getBitSet()[0];
	    		 break;
	    case 15: CHECKING_PERFORMANCE_PERIOD_VALUE=propertyValue;break;
	    }
    }

    public int getPropertyValue(int propertyName)
    {
	switch (propertyName)
	    {
	    case 0: return LISTENING_PERIOD_VALUE;
	    case 1: return ADVERTISEMENT_PERIOD_VALUE;
	    case 2: return COMMUNICATION_PORT_VALUE;
	    case 3: return DIRECTORY_ELECTION_PROCESS_MSG_NUMBER_VALUE;
	    case 4: return DIRECTORY_ELECTION_PROCESS_RESPONSE_MSG_NUMBER_VALUE;
	    case 5: return DIRECTORY_ELECT_MSG_NUMBER_VALUE;
	    case 6: return GET_SERVICE_FROM_SD_MSG_NUMBER_VALUE;
	    case 7: return DIRECTORY_REGISTER_SERVICE_MSG_NUMBER_VALUE;
	    case 8: return DIRECTORY_REGISTER_SERVICE_MSG_NUMBER_VALUE;
	    case 9: return ELECTION_WAITING_RESPONSE_TIME_VALUE;
	    case 10:return WAITING_GET_SERVICE_RESPONSE_MIN_VALUE;
	    case 11:return WAITING_GET_SERVICE_RESPONSE_MAX_VALUE;
	    case 12:return PROFILE_PORT_VALUE;
	    case 13:return INT_HOPS_NUMBER_VALUE;
	    case 14:return INT_GET_SERVICE_REQUEST_HOPS_VALUE;
	    case 15:return CHECKING_PERFORMANCE_PERIOD_VALUE;
	    }
	return -1;
    }

    public void setProperty(int propertyName, float propertyValue)
    {
    	switch (propertyName)
		{
    	case 16 : MIN_PERFORMANCE_THRESHOLD_VALUE=propertyValue;break;
		}
    }
    /**
     * 
     * @param propertyName
     * @return
     * 
     */
    public float getPropertyFloatValue(int propertyName)
    {
    	switch (propertyName)
		{
		case 16 : return MIN_PERFORMANCE_THRESHOLD_VALUE;
		}
    	return -1;
    }
    
    public void start()
    {
    	if (started) return; 
    	logger.info("Please wait for starting Ubisec Service Discovery...");
    	stoped = false;
    	directories = new Hashtable();
    	localProfile = new NodeProfile("0",PROFILE_PORT_VALUE);
    	logger.debug("localProfile="+localProfile.getXMLProfile());
    	
    	try
		{
    		Thread.sleep(5000);
		}
    	catch(Exception ex)
		{	
    		System.out.println("no");
		}
    	manager = new MessagesManager();
    	manager.start(COMMUNICATION_PORT_VALUE);
    	findDirStarted= true;
    	listening.start();
    	findDirectory();
    	findDirStarted = false;
    	started= true;
    	logger.info("Ubisec Service Discovery is started");
    	//if (!hasDirectory) 
	    	//{
    	directoryChecking.start();
           	//}
        performanceChecking.start();
        started= true;
    }
    /**
     * 
     *
     */
    public void findDirectory()
    {
    	manager.clean(new BitSet2[]{MessageType.DIRECTORY_ADVERTISEMENT, MessageType.DIRECTORY_ELECTION_PROCESS_REQUEST});
    	thereIsAnotherElection = false;
    	findDirStarted = true;
    	//manager.clean();
    	started=false;
    	if (directories == null)
    		logger.debug("directories is null");
    	if (deploymentManner.equals("true")) 
    		directoryElect();
	
    	else if (deploymentManner.equals("false"))
    	{
    		// wait until at least one directory is found
    		while (!hasDirectory && directories.size()==0)	
    		{
    			try
				{
    				Thread.sleep(200);
				}
    			catch (Exception ex)
				{
    				logger.error("at SD.sleep");
				}
    			//Message[] messages = manager.getMessagesType(MessageType.DIRECTORY_ADVERTISEMENT);
    			//if (messages!=null && messages.length!=0)
    			//directoryAdvertisement(messages);
    		}
    	}
    	else while (!hasDirectory && directories.size()==0)	
	    	{
    		//Message[] messages = manager.getMessagesType(MessageType.DIRECTORY_ADVERTISEMENT);
    		//if (messages!=null && messages.length!=0)
		    //{
    		//	directoryAdvertisement(messages);
		    //}
    		//else
		    //{
			try
			    {
				logger.debug("SD.sleep " +ADVERTISEMENT_PERIOD_VALUE);
				int counter=0;
				while (!hasDirectory && directories.size()==0 && counter <ADVERTISEMENT_PERIOD_VALUE)
					{
						Thread.sleep(50);
						counter = counter+50;
					}
				//Thread.sleep(ADVERTISEMENT_PERIOD_VALUE);
				logger.debug("SD.wakeup");
			    }
			catch (Exception ex)
			    {
				logger.error("Error at Election.sleep");
			    }
			//messages = manager.getMessagesType(MessageType.DIRECTORY_ADVERTISEMENT);
			//if (messages != null && messages.length!=0)
			//directoryAdvertisement(messages);
			logger.debug("hasDirectory="+hasDirectory+" directories.size()="+directories.size()+" thereIsAnotherElection="+thereIsAnotherElection);
			if (!hasDirectory && directories.size()==0 && !thereIsAnotherElection)
			{
				logger.debug("Election will begin");
				findDirStarted = false;
				Election election = new Election();
				NodeProfile[] nodeProfiles = election.election(MIN_PERFORMANCE_THRESHOLD_VALUE);
				findDirStarted = true;
				if (nodeProfiles!=null && nodeProfiles.length!=0)
				    {
					if (nodeProfiles[0].getIPAddress().equals(localProfile.getIPAddress()))
						directoryElect(nodeProfiles[0].getIdealRay());
					else addDirectory(nodeProfiles);
				    }
				else 
				{
					try
					{   
						long timeToWait = election.getPropertyValue(Election.HOPS_NUMBER)*ADVERTISEMENT_PERIOD_VALUE;
						logger.debug("SD Sleep "+timeToWait);
						int counter=0;
						while (!hasDirectory && directories.size()==0 && counter <timeToWait)
							{
								Thread.sleep(50);
								counter = counter+50;
							}
						//Thread.sleep(ADVERTISEMENT_PERIOD_VALUE);
						logger.debug("SD Sleep end");
					}
					catch (Exception ex)
					{
						logger.error("at SD.sleep ");
					}
				}
			}
			thereIsAnotherElection = false;
			try
			{   
				long timeToWait = Setting.ELECTION_HOPS_VALUE*ADVERTISEMENT_PERIOD_VALUE;
				logger.debug("SD Sleep "+timeToWait);
				int counter=0;
				while (!hasDirectory && directories.size()==0 && counter < timeToWait)
					{
						Thread.sleep(50);
						counter = counter+50;
					}
				//Thread.sleep(ADVERTISEMENT_PERIOD_VALUE);
				logger.debug("SD Sleep end");
			}
			catch (Exception ex)
			{
				logger.error("Error at SD.sleep ");
			}

			
			
			}
	    //}
		findDirStarted = false;
    	started= true;
    }

    public boolean isStarted()
    {
    	return started;
    }

    public void stop()
    {
    	stoped = true;
    	started = false;
    	manager.stop();
    	localProfile.stopCoverageDetection();
    	if (hasDirectory) localDirectory.undeploy();
    	hasDirectory = false;
    	logger.debug("SD is stoped");
    }

    public void directoryAdvertisement(Message[] messages)
    {
    	//System.out.println("directoryAdvertisement is invoked");
    	if (messages == null || messages.length==0) return;
    	NodeProfile[] directoryProfiles = new NodeProfile[messages.length];
    	for (int i=0; i<messages.length; i++)
	    directoryProfiles[i] = new NodeProfile(messages[i].getMessageContent());
    	addDirectory(directoryProfiles);
    }
    
    public void addDirectory(NodeProfile[] nodeProfiles)
    {
    	if (directories.size()==0) localProfile.setMessageType("1");
    	for (int i=0; i<nodeProfiles.length; i++)
	    {	   
		if (directories.containsKey(nodeProfiles[i].getIPAddress()))
		   	directories.remove(nodeProfiles[i].getIPAddress());
		DirectoryResume directory = new DirectoryResume(nodeProfiles[i]);
		directory.setTTL(ADVERTISEMENT_PERIOD_VALUE*5);
		directories.put(nodeProfiles[i].getIPAddress(),directory);
		logger.info("Directory found at "+nodeProfiles[i].getIPAddress());
		}
    }

    
    public void directoryElect()
    {
    	if (!hasDirectory) 
    		{
    			hasDirectory = true;
    			localDirectory = new Directory();
    			localDirectory.deploy();
    		}
    	
    }
    
    public void directoryElect(int hops)
    {
    	if (!hasDirectory) 
    		{
    			hasDirectory = true;
    			localDirectory = new Directory();
    			localDirectory.deploy(hops);
    		}
    	
    }

    public void directoryElect(Message[] messages)
    {
    	// if the election message is rigth call directoryElect()
    	if (messages!=null && messages.length>0) 
    		{
    		byte[] content = messages[0].getMessageContent();
    		BitSet2 hops = new BitSet2(content[0]);
    		
    		directoryElect(hops.getInt());
    		}
    }

    public void electionMessagesRequest(Message[] messages)
    {
    	if (messages == null || messages.length==0) return;
    	
    	localProfile.profileUpdate();
    	thereIsAnotherElection = true;
    	
    	Message messageElectionResponse = MessageFactory.createMessage();
    	messageElectionResponse.setMessageType(MessageType.DIRECTORY_ELECTION_PROCESS_RESPONSE); 
    	messageElectionResponse.setMessageSource(new IPAddress(localProfile.getIPAddress()));
    	messageElectionResponse.setMessagePort(COMMUNICATION_PORT_VALUE);
    	messageElectionResponse.setMessageHop(HOPS_NUMBER_VALUE);
    	messageElectionResponse.setMessageHopCounter(new BitSet2());
    	    	
    	long totalTime =0;	    	
    	for (int i=0;i<messages.length;i++)
	    	{
    		byte[] byteProfile = null;
        	//if (localProfile.getBenefit()>=MIN_PERFORMANCE_THRESHOLD_VALUE)
        	localProfile.profileUpdate();
    		localProfile.setHopsNumberFromInitiatir(messages[i].getMessageHopCounter().getInt()+1);
        	byteProfile = localProfile.getBytes();	
    		
        	logger.debug("Election Request from "+messages[i].getMessageSource().getHostIPAddress() +"is received");
    		messageElectionResponse.setMessageInitiator(messages[i].getMessageInitiator());
    		messageElectionResponse.setMessageNumber(messages[i].getMessageNumber());
    		//mi]essageElectionResponse.setMessageNumber(DIRECTORY_ELECTION_PROCESS_RESPONSE_MSG_NUMBER_VALUE);
    		//response only when my profile is better than the received profile 
    		//is commented
    		//NodeProfile Profile = new NodeProfile(messages[i].getBytes());
    		//if (Election.betterProfileComparedTo(localProfile, Profile))
		    	//{
    		// wait for response from others nodes
    		try
			{
    			long timeToWait = ELECTION_WAITING_RESPONSE_TIME_VALUE*(messages[i].getMessageHop().getInt()-(messages[i].getMessageHopCounter().getInt()+1));
    			logger.debug("ELECTION_WAITING_RESPONSE_TIME_VALUE="+ELECTION_WAITING_RESPONSE_TIME_VALUE);
    			logger.debug("messages[i].getMessageHop().getInt()="+messages[i].getMessageHop().getInt());
    			logger.debug("messages[i].getMessageHopCounter().getInt()"+messages[i].getMessageHopCounter().getInt());
    			
    			logger.debug("timeToWait="+timeToWait);
    			logger.debug("totalTime="+totalTime);
    			if (timeToWait > totalTime) timeToWait = timeToWait - totalTime;
    			else timeToWait = 0;
    			logger.debug("waiting for election response from the others nodes, time="+timeToWait);
    			Thread.sleep(timeToWait);
    			totalTime = totalTime+timeToWait;
			}
    		catch(Exception ex)
			{
    			logger.error("Error at waiting for election response from the others nodes");
			}
    		
    	  	
        	
    		byte[] toSend = new byte[0];
    		int totalLength=0;
        	if (byteProfile!=null) totalLength = 4+byteProfile.length;
    		
    		Message[] responseMessages = manager.getMessagesType(MessageType.DIRECTORY_ELECTION_PROCESS_RESPONSE, messages[i].getMessageNumber(), messages[i].getMessageInitiator());
    		if (responseMessages != null)
    		{
    			for (int r=0; r<responseMessages.length; r++)
    				if (responseMessages[r].getMessageContent()!=null ) totalLength = totalLength+responseMessages[r].getMessageContent().length;
    		}
    		
    		toSend = new byte[totalLength];
    		int position=0;
    		if (byteProfile!=null) 
    		{
    			logger.debug("my profile length ="+byteProfile.length);
    			BigBitSet profileLength = new BigBitSet(byteProfile.length);
    			BitSet2[] bitSet = profileLength.getBitSet();
    			toSend[0]=bitSet[0].getByte();
    			toSend[1]=bitSet[1].getByte();
    			toSend[2]=bitSet[2].getByte();
    			toSend[3]=bitSet[3].getByte();
    			position =4;
    			for (int z=0; z< byteProfile.length ; z++)
    			{
    				toSend[position]= byteProfile[z];
    			    position++;
    			}
    		}
    				
    		if (responseMessages != null)
    		{
    			for (int r=0; r<responseMessages.length; r++)
    				if (responseMessages[r].getMessageContent()!=null) 
    					{
    					/*
    					BigBitSet contentLength = new BigBitSet(responseMessages[r].getMessageContent().length);
    					BitSet2[] contentBitSet = contentLength.getBitSet();
    	    			toSend[position]=contentBitSet[0].getByte(); position++;
    	    			toSend[position]=contentBitSet[1].getByte(); position++;
    	    			toSend[position]=contentBitSet[2].getByte(); position++;
    	    			toSend[position]=contentBitSet[3].getByte();position++;
    	    			*/
    	    			for (int z=0; z< responseMessages[r].getMessageContent().length; z++)
    	    			{
    	    				toSend[position]= responseMessages[r].getMessageContent()[z];
    	    				position++;
    	    			}
    					}
    		}
    				
    		logger.debug("Total length of sent profile="+toSend.length);
    		messageElectionResponse.setMessageContent(toSend);
    		manager.sendInstantaneousMessageTo(messageElectionResponse, messages[i].getMessageSource().getHostIPAddress(), COMMUNICATION_PORT_VALUE);
    		DIRECTORY_ELECTION_PROCESS_RESPONSE_MSG_NUMBER_VALUE++;
    		logger.debug("Election candidature (type="+messageElectionResponse.getMessageType().getInt()+")is sent to "+messages[i].getMessageSource().getHostIPAddress() +" with final destination is "+messages[i].getMessageInitiator().getHostIPAddress());
		    	//}		
	    	}
    }
    
    public void register(String wsdl, String EP, String wsdlCustLocal, String wsdlCustRemote)
    {
    	//System.out.println("register is called");
    	if (hasDirectory) localDirectory.register(wsdl,EP, wsdlCustLocal, wsdlCustRemote);
    	else 
	    	{
    		logger.debug("hasn't local Directory");
    		Message messageRegisterService = MessageFactory.createMessage();
    		messageRegisterService.setMessageType(MessageType.REGISTER_SERVICE); 
    		messageRegisterService.setMessageSource(new IPAddress(localProfile.getIPAddress()));
    		messageRegisterService.setMessagePort(COMMUNICATION_PORT_VALUE);
    		messageRegisterService.setMessageHop(HOPS_NUMBER_VALUE);
    		messageRegisterService.setMessageHopCounter(new BitSet2());
    		//Message content
    		//length of EP(4bytes) + EP + WSDL document
    		byte[] byteEP   = EP.getBytes();
    		byte[] byteWSDL = wsdl.getBytes();
    		byte[] byteWSDLCustLocal = wsdlCustLocal.getBytes();
    		byte[] byteWSDLCustRemote = wsdlCustRemote.getBytes();
    		int EPLength = byteEP.length;
    		int WSDLLength = byteWSDL.length;
    		int WSDLCustLocalLength = byteWSDLCustLocal.length;
    		int WSDLCustRemoteLength = byteWSDLCustRemote.length;
    		
    		byte[] content = new byte[16+EPLength+WSDLLength+WSDLCustLocalLength+WSDLCustRemoteLength];
    		
    		BigBitSet length = new BigBitSet(EPLength);
    		content[0]=length.getBitSet()[0].getByte();
    		content[1]=length.getBitSet()[1].getByte();
    		content[2]=length.getBitSet()[2].getByte();
    		content[3]=length.getBitSet()[3].getByte();
    		int i=4;
    		int k=0;
    		for (k=0; k<EPLength;k++)
    			{
    			content[i]=byteEP[k];
    			i++;
    			}
    		    		
    		length = new BigBitSet(WSDLLength);
    		content[i]=length.getBitSet()[0].getByte();i++;
    		content[i]=length.getBitSet()[1].getByte();i++;
    		content[i]=length.getBitSet()[2].getByte();i++;
    		content[i]=length.getBitSet()[3].getByte();i++;
    		for (k=0; k<WSDLLength;k++) 
    			{
    			content[i]=byteWSDL[k];
    			i++;
    			}
    		    		
    		length = new BigBitSet(WSDLCustLocalLength);
    		content[i]=length.getBitSet()[0].getByte();i++;
    		content[i]=length.getBitSet()[1].getByte();i++;
    		content[i]=length.getBitSet()[2].getByte();i++;
    		content[i]=length.getBitSet()[3].getByte();i++;
    		for (k=0; k<WSDLCustLocalLength;k++)
    			{
    			content[i]=byteWSDLCustLocal[k];
    			i++;
    			}
    		
    		
    		length = new BigBitSet(WSDLCustRemoteLength);
    		content[i]=length.getBitSet()[0].getByte();i++;
    		content[i]=length.getBitSet()[1].getByte();i++;
    		content[i]=length.getBitSet()[2].getByte();i++;
    		content[i]=length.getBitSet()[3].getByte();i++;
    		for (k=0; k<WSDLCustRemoteLength;k++) 
    			{
    			content[i]=byteWSDLCustRemote[k];
    			i++;
    			}
    		
    		messageRegisterService.setMessageContent(content);
    		logger.debug("messageRegisterService.getMessageContent().length="+messageRegisterService.getMessageContent().length);
    		Enumeration ipAddresses = directories.keys();
    		while (ipAddresses.hasMoreElements())
    			manager.sendInstantaneousMessageTo(messageRegisterService,(String)ipAddresses.nextElement(), COMMUNICATION_PORT_VALUE); 
	    }
    }
    
    public void unregister(String wsdl, String EP)
    {
    	if (hasDirectory) localDirectory.unregister(wsdl,EP);
    	else 
	    {
    		Message messageUnregisterService = MessageFactory.createMessage();
    		messageUnregisterService.setMessageType(MessageType.UNREGISTER_SERVICE); 
    		messageUnregisterService.setMessageSource(new IPAddress(localProfile.getIPAddress()));
    		messageUnregisterService.setMessagePort(COMMUNICATION_PORT_VALUE);
    		messageUnregisterService.setMessageHop(HOPS_NUMBER_VALUE);
    		messageUnregisterService.setMessageHopCounter(new BitSet2());
    		//Message content
    		//length of EP(4bytes) + EP + WSDL document
    		byte[] byteEP   = EP.getBytes();
    		byte[] byteWSDL = wsdl.getBytes();

    		int EPLength = byteEP.length;
    		BigBitSet length = new BigBitSet(EPLength);
    		byte[] content = new byte[4+EPLength+byteWSDL.length];
    		content[0]=length.getBitSet()[0].getByte();
    		content[1]=length.getBitSet()[1].getByte();
    		content[2]=length.getBitSet()[2].getByte();
    		content[3]=length.getBitSet()[3].getByte();
    		for (int i=0; i<EPLength;i++) content[4+i]=byteEP[i];
    		for (int i=0; i<byteWSDL.length;i++) content[4+EPLength+i]=byteWSDL[i];
    		messageUnregisterService.setMessageContent(content);
    		Enumeration ipAddresses = directories.keys();
    		while (ipAddresses.hasMoreElements())
    			manager.sendInstantaneousMessageTo(messageUnregisterService,(String)ipAddresses.nextElement(), COMMUNICATION_PORT_VALUE); 
	    }
    }
    
    public GetServiceData[] getServices(String wsdl)
    {
    	manager.clean(new BitSet2[]{MessageType.GET_SERVICE_RESPONSE_TO_SD});
    	if (hasDirectory) return localDirectory.getServices(wsdl, GET_SERVICE_REQUEST_HOPS_VALUE,new BitSet2());
    	Message messageGetService = MessageFactory.createMessage();
    	messageGetService.setMessageNumber(GET_SERVICE_FROM_SD_MSG_NUMBER_VALUE);
    	GET_SERVICE_FROM_SD_MSG_NUMBER_VALUE++;
    	messageGetService.setMessageType(MessageType.GET_SERVICE_FROM_SD); 
    	messageGetService.setMessageSource(new IPAddress(localProfile.getIPAddress()));
    	messageGetService.setMessagePort(COMMUNICATION_PORT_VALUE);
    	messageGetService.setMessageHop(GET_SERVICE_REQUEST_HOPS_VALUE);
    	messageGetService.setMessageHopCounter(new BitSet2(0));
    	messageGetService.setMessageContent(wsdl.getBytes());
    	Enumeration ipAddresses = directories.keys();
    	//long t1 = System.currentTimeMillis();
    	while (ipAddresses.hasMoreElements())
    		manager.sendInstantaneousMessageTo(messageGetService, (String)ipAddresses.nextElement(), COMMUNICATION_PORT_VALUE);
    	Message[] messages=null;
    	int time = 0;
    	//long t2 = System.currentTimeMillis();
    	while ((messages == null || messages.length==0) && time<WAITING_GET_SERVICE_RESPONSE_MAX_VALUE)
	    	{
    		try
		    {	
    			Thread.sleep(WAITING_GET_SERVICE_RESPONSE_MIN_VALUE);
    			time = time + WAITING_GET_SERVICE_RESPONSE_MIN_VALUE;
		    }
		catch (Exception ex)
		    {
			   logger.error("at SD.sleep");
		    }
	    	messages = manager.getMessagesType(MessageType.GET_SERVICE_RESPONSE_TO_SD, messageGetService.getMessageNumber());
	    }
    	if (messages ==null || messages.length ==0) 
	    return (new GetServiceData[0]);
    	//********** some reponses are received get content and construct string[]
	
    	Vector servicesEPTmp = new Vector();
    	for (int i=0;i<messages.length;i++)
	    {
    		Vector vectTmp = getServicesFrom(messages[i].getMessageContent());
    		for (int z=0;z<vectTmp.size();z++) servicesEPTmp.add((GetServiceData)vectTmp.elementAt(z));
	    }
    	GetServiceData[] forReturn = new GetServiceData[servicesEPTmp.size()];
    	for (int i=0;i<forReturn.length;i++)
    		forReturn[i]=(GetServiceData)servicesEPTmp.elementAt(i);
    	return forReturn;
    }

    
    public GetServiceData[] getServices_ByInstanceURL(String wsdl, String instance_URL)
    {
    	String QoS="";
		if (wsdl.indexOf("<ServiceDiscovery>") !=-1 && wsdl.indexOf("</ServiceDiscovery>")!=-1)
			QoS = wsdl.substring(wsdl.indexOf("<ServiceDiscovery>"), wsdl.indexOf("</ServiceDiscovery>")+19);
		String wsdlDocument = wsdl.substring(QoS.length());
		logger.debug("getServices_ByInstanceURL: QoS="+QoS);
		QoSParameters QoSParam = new QoSParameters(QoS);
		logger.debug("QoSParam created with QoS");
		QoSParam.addQoSParameter("instance", 1, -1, instance_URL, -1);
		logger.debug("instance QoS is add to QoS");
		logger.debug("new QoSParam ="+QoSParam.getXMLDocument());
		wsdl = "<ServiceDiscovery>"+QoSParam.getXMLDocument()+"</ServiceDiscovery>"+wsdlDocument;
    	return getServices(wsdl);
    }
    

    public GetServiceData[] getAllServices(String GroupServiceWAU)
    {
    	// will be implemented to GroupService
    	    	return null;
    }

    
    
    public static Vector getServicesFrom(byte[] data)
    {
	if (data!=null) logger.debug("data.length="+data.length);
	Vector forReturn = new Vector();
	if (data == null || data.length == 0) return forReturn;
	int pos=0; 
	while (pos<data.length)
	    {
		byte[] EPLength = new byte[4];
		EPLength[0]=data[pos++];
		EPLength[1]=data[pos++];
		EPLength[2]=data[pos++];
		EPLength[3]=data[pos++];
		BigBitSet bitSetLength = new BigBitSet(EPLength);
		int length = bitSetLength.getInt();
		byte[] EPbytes = new byte[length];
		for (int i=0;i<length;i++) EPbytes[i]=data[pos++];
		String EP = new String(EPbytes);

		byte[] WSDLLength = new byte[4];
		WSDLLength[0]=data[pos++];
		WSDLLength[1]=data[pos++];
		WSDLLength[2]=data[pos++];
		WSDLLength[3]=data[pos++];
		BigBitSet bitSetWSDLLength = new BigBitSet(WSDLLength);
		int WSDLLength2 = bitSetWSDLLength.getInt();
		byte[] WSDLbytes = new byte[WSDLLength2];
		for (int i=0;i<WSDLLength2;i++) WSDLbytes[i]=data[pos++];
		String WSDL = new String(WSDLbytes);

		byte[] EPCustLocalLength = new byte[4];
		EPCustLocalLength[0]=data[pos++];
		EPCustLocalLength[1]=data[pos++];
		EPCustLocalLength[2]=data[pos++];
		EPCustLocalLength[3]=data[pos++];
		BigBitSet bitSetCustLocalLength = new BigBitSet(EPCustLocalLength);
		int CustLocalLength = bitSetCustLocalLength.getInt();
		byte[] EPCustLocalbytes = new byte[CustLocalLength];
		for (int i=0;i<CustLocalLength;i++) EPCustLocalbytes[i]=data[pos++];
		String EPCustLocal = new String(EPCustLocalbytes);
		
		byte[] WSDLCustLocalLength = new byte[4];
		WSDLCustLocalLength[0]=data[pos++];
		WSDLCustLocalLength[1]=data[pos++];
		WSDLCustLocalLength[2]=data[pos++];
		WSDLCustLocalLength[3]=data[pos++];
		BigBitSet bitSetWSDLCustLocalLength = new BigBitSet(WSDLCustLocalLength);
		int WSDLCustLocalLength2 = bitSetWSDLCustLocalLength.getInt();
		byte[] WSDLCustLocalbytes = new byte[WSDLCustLocalLength2];
		for (int i=0;i<WSDLCustLocalLength2;i++) WSDLCustLocalbytes[i]=data[pos++];
		String WSDLCustLocal = new String(WSDLCustLocalbytes);
		
		byte[] EPCustRemoteLength = new byte[4];
		EPCustRemoteLength[0]=data[pos++];
		EPCustRemoteLength[1]=data[pos++];
		EPCustRemoteLength[2]=data[pos++];
		EPCustRemoteLength[3]=data[pos++];
		BigBitSet bitSetCustRemoteLength = new BigBitSet(EPCustRemoteLength);
		int CustRemoteLength = bitSetCustRemoteLength.getInt();
		byte[] EPCustRemotebytes = new byte[CustRemoteLength];
		for (int i=0;i<CustRemoteLength;i++) EPCustRemotebytes[i]=data[pos++];
		String EPCustRemote = new String(EPCustRemotebytes);
		
		byte[] WSDLCustRemoteLength = new byte[4];
		WSDLCustRemoteLength[0]=data[pos++];
		WSDLCustRemoteLength[1]=data[pos++];
		WSDLCustRemoteLength[2]=data[pos++];
		WSDLCustRemoteLength[3]=data[pos++];
		BigBitSet bitSetWSDLCustRemoteLength = new BigBitSet(WSDLCustRemoteLength);
		int WSDLCustRemoteLength2 = bitSetWSDLCustRemoteLength.getInt();
		byte[] WSDLCustRemotebytes = new byte[WSDLCustRemoteLength2];
		for (int i=0;i<WSDLCustRemoteLength2;i++) WSDLCustRemotebytes[i]=data[pos++];
		String WSDLCustRemote = new String(WSDLCustRemotebytes);
		
		GetServiceData service = new GetServiceData();
		service.setServiceEP(EP);
		service.setWSDLDocument(WSDL);
		service.setCustLocalEP(EPCustLocal);
		service.setWSDLCustLocal(WSDLCustLocal);
		service.setCustRemoteEP(EPCustRemote);
		service.setWSDLCustRemote(WSDLCustRemote);
		forReturn.add(service); 
	    }
	return forReturn;
    } 
    
    public Thread listeningProcess() {
        try
	    {
		Thread listeningProcessThread = 
		    new Thread( new Runnable() 
			{
			    public void run() 
			    {
			    	while(!stoped)
			    		while ((started || findDirStarted) && !hasDirectory)
			    		{
			    			Message[] messages = manager.getMessagesType(MessageType.DIRECTORY_ADVERTISEMENT);
			    			directoryAdvertisement(messages);
			    			messages = manager.getMessagesType(MessageType.DIRECTORY_ELECTION_PROCESS_REQUEST);
			    			electionMessagesRequest(messages);
			    			messages = manager.getMessagesType(MessageType.DIRECTORY_ELECT);
			    			directoryElect(messages);
			    			manager.clean(new BitSet2[]{MessageType.DIRECTORY_TO_DIRECTORY_ADVERTISEMENT, MessageType.DIRECTORY_GET_ALL_RESUMES});
  		    
			    			try
							{
			    				Thread.sleep(LISTENING_PERIOD_VALUE);
							}
			    			catch (Exception ex)
							{
			    				System.out.println("Error at SD.listeningProcess: Thread.sleep");
							}
			    		}
			    	
			    }
			});
		return listeningProcessThread;
	    }
	catch (Exception ex)
	    {
		logger.error("at create SD.listeningProcess Thread");
	    }
	return null;
  }
    
    public Thread PerformanceChecking() {
    	try
		{
    	 Thread checkingProcessThread = new Thread(new Runnable(){
		    public void run() 
		    {
		    	while (!stoped)
		    		while (started && hasDirectory)
			    {
				try
				{
								
					//System.out.println("localnode benefit="+localProfile.getBenefit());
					//System.out.println("MIN_PERFORMANCE_THRESHOLD_VALUE="+MIN_PERFORMANCE_THRESHOLD_VALUE);
					Thread.sleep(CHECKING_PERFORMANCE_PERIOD);
					localProfile.profileUpdate();
					if (localProfile.getBenefit()<MIN_PERFORMANCE_THRESHOLD_VALUE)
					{
						logger.warn("poor performances of this node");
						localDirectory.undeploy();
						logger.info("local directory is undeployed");
						hasDirectory=false;
						findDirectory();
					}
				}
				catch(Exception ex)
				{
					logger.error("at SD.checkingProcessThread.run()");
				}
			    }
		    	
		    }			
    	 });			
    	 return checkingProcessThread; 
		}
    	catch (Exception ex)
		{
    		logger.error("at create SD.PerformanceChecking Thread");
		}
    	return null;
    }
    
    public Thread DirectroriesChecking() {
    	try
		{
    	 Thread directoriesCheckingProcessThread = new Thread(new Runnable(){
		    public void run() 
		    {
		    	while (!stoped)
		    		while (started && !findDirStarted && !hasDirectory)
			    {
				try
				{
					
					Thread.sleep(ADVERTISEMENT_PERIOD_VALUE);
					Iterator directoriesIterator = directories.values().iterator();
					while (directoriesIterator.hasNext())
					{
						DirectoryResume directory = (DirectoryResume)directoriesIterator.next();
						directory.setTTL(directory.getTTL()-ADVERTISEMENT_PERIOD_VALUE);
						if (directory.getTTL()<=0) 
							{
							logger.info("directory "+directory.getDirectoryProfile().getIPAddress()+" is removed");
								directoriesIterator.remove();
							}
					}
					
					if (directories.size()==0 && started) 
						{
							logger.info("SD.DirectoriesCheckingProcessThread no directory");
							localProfile.setMessageType("0");
							findDirectory();
						}
				}
				catch(Exception ex)
				{
					logger.error("at SD.DirectoriesCheckingProcessThread.run()");
					ex.printStackTrace();
				}
			    }			
		    	
		    }			
    	 });			
    	 return directoriesCheckingProcessThread; 
		}
    	catch (Exception ex)
		{
    	 logger.error("at create SD.DirectoriesChecking Thread");
		}
    	return null;
    }
}
