package ubisec.discovery.message;

import ubisec.util.*;
import ubisec.net.IPAddress;

/**
*
* 
* @author Rafik CHIBOUT 
* @author Francoise SAILHAN
* @version 0.1 sept. 2004
*
* @see ubisec.discovery.message.Message
* @see ubisec.discovery.message.MessageImpl
*  
* 
*/

public class MessageFactory
{
	
	
	/**
	*
	* Creates a new Message object with the default attributes and an empty content.
	* <p>
	* Content can be added to this message and the attributes can be modified before send it. 
	* @return a new ubisec.discovery.message.Message
	* @see ubisec.discovery.message.MessageFactory#createMessage(byte[] data)
	* 
	*/
    public static Message createMessage()
    {
	return new MessageImpl();
    }

    /**
	*
	* Internalizes the contents of the given array of bytes into a new Message object and returns the Message object.
	* 
	* @param data the array of bytes that contains the data (attributes and content) for a message. 
	* @return a new Message object containing the data from the given array of bytes.
	* @see ubisec.discovery.message.MessageFactory#createMessage()
	* 
	*/
    public static Message createMessage(byte[] data)
    {
	Message message = new MessageImpl();
	// init message
	if (data != null && data.length>=27)
	    {
		//Byte tmp = new Byte(data[0]);
		BitSet2[] tmpMessNum = new BitSet2[4];
		tmpMessNum[0] = new BitSet2(data[0]);
		tmpMessNum[1] = new BitSet2(data[1]);
		tmpMessNum[2] = new BitSet2(data[2]);
		tmpMessNum[3] = new BitSet2(data[3]);
		BigBitSet bigBitSet = new BigBitSet(tmpMessNum);
		message.setMessageNumber(bigBitSet.getInt());
		
		BitSet2 tmp2 = new BitSet2(data[4]);
		message.setMessageType(tmp2);
		BitSet2[] tmp3 = new BitSet2[4];
		tmp3[0] = new BitSet2(data[5]);
		tmp3[1] = new BitSet2(data[6]);
		tmp3[2] = new BitSet2(data[7]);
		tmp3[3] = new BitSet2(data[8]);
		message.setMessageSource(new IPAddress(tmp3));
		BitSet2[] tmp31 = new BitSet2[4];
		tmp31[0] = new BitSet2(data[9]);
		tmp31[1] = new BitSet2(data[10]);
		tmp31[2] = new BitSet2(data[11]);
		tmp31[3] = new BitSet2(data[12]);
		message.setMessageDestination(new IPAddress(tmp31));
		BitSet2[] tmp32 = new BitSet2[4];
		tmp32[0] = new BitSet2(data[13]);
		tmp32[1] = new BitSet2(data[14]);
		tmp32[2] = new BitSet2(data[15]);
		tmp32[3] = new BitSet2(data[16]);
		message.setMessageInitiator(new IPAddress(tmp32));
		
		
		//Byte tmp32 = new Byte(data[10]);
		//message.setMessagePort(tmp32.intValue());
		BitSet2[] tmpMessPort = new BitSet2[4];
		tmpMessPort[0] = new BitSet2(data[17]);
		tmpMessPort[1] = new BitSet2(data[18]);
		tmpMessPort[2] = new BitSet2(data[19]);
		tmpMessPort[3] = new BitSet2(data[20]);
		bigBitSet = new BigBitSet(tmpMessPort);
		message.setMessagePort(bigBitSet.getInt());

		tmp2 = new BitSet2(data[21]);
		message.setMessageHop(tmp2);
		tmp2 = new BitSet2(data[22]);
		message.setMessageHopCounter(tmp2);

		//tmp = new Byte(data[13]);
		//message.setMessageLength(tmp.shortValue());
		BitSet2[] tmpMessLen = new BitSet2[4];
		tmpMessLen[0] = new BitSet2(data[23]);
		tmpMessLen[1] = new BitSet2(data[24]);
		tmpMessLen[2] = new BitSet2(data[25]);
		tmpMessLen[3] = new BitSet2(data[26]);
		bigBitSet = new BigBitSet(tmpMessLen);
		message.setMessageLength(bigBitSet.getInt());
		
		BitSet2[] tmpMPRLength = new BitSet2[4];
		tmpMPRLength[0] = new BitSet2(data[27]);
		tmpMPRLength[1] = new BitSet2(data[28]);
		tmpMPRLength[2] = new BitSet2(data[29]);
		tmpMPRLength[3] = new BitSet2(data[30]);
		bigBitSet = new BigBitSet(tmpMPRLength);
		
		IPAddress[] MPRList = new IPAddress[bigBitSet.getInt()];
		for (int j=0; j<MPRList.length; j++)
		{
			BitSet2[] tmpAddress = new BitSet2[4];
			tmpAddress[0] = new BitSet2(data[31+(4*j)]);
			tmpAddress[1] = new BitSet2(data[31+(4*j)+1]);
			tmpAddress[2] = new BitSet2(data[31+(4*j)+2]);
			tmpAddress[3] = new BitSet2(data[31+(4*j)+3]);
			MPRList[j] = new IPAddress(tmpAddress);
		}
		message.setMPRList(MPRList);
		
		byte[] content = new byte[data.length-(31+(4*MPRList.length))];
		for (int i=0;i<content.length;i++)
		    content[i] = data[i+(31+(4*MPRList.length))];
		message.setMessageContent(content);
	    }
	else System.out.println("Message not compatible with our system");
	return message;
    }

    /**
	*
	* Creates a new MessageFactory object that is an instance of the default 
	* implementation.
	* 
	* @return a new ubisec.discovery.message.MessageFactory
	*  
	*/
    
	public static MessageFactory newInstance()
	{
	return new MessageFactory();
	}
}
