/*
*
* WSDLTools.java
*
* @author Rafik CHIBOUT & Francoise SAILHAN
* @version 0.1 sept. 2004
*
*  
*/

package ubisec.discovery.tool;

import java.io.*;
import java.util.*;

import javax.wsdl.*;
import javax.wsdl.factory.*;
import javax.wsdl.xml.*;

import org.xml.sax.*;

public class WSDLTools 
{

    public static Definition readWSDL(String wsdlDocument)
    {
	//System.out.println("wsdlDocument="+wsdlDocument);
	Definition def = null;
	try
	    {
		
		WSDLFactory factory=WSDLFactory.newInstance();
		WSDLReader reader=factory.newWSDLReader();
		reader.setFeature ("javax.wsdl.verbose", false);
		reader.setFeature ("javax.wsdl.importDocuments", false);
		CharArrayReader wsdlArray = new CharArrayReader(wsdlDocument.toCharArray());
		InputSource wsdlSource = new InputSource(wsdlArray);
		def = reader.readWSDL(null, wsdlSource);
			
		}
	catch (Exception ex)
	    {
		System.out.println("Error at readWSDL");
		//ex.printStackTrace();
	    }
	return def;
    }

    public static boolean equals(javax.wsdl.Message message1, javax.wsdl.Message message2)
    {
	// compare only the type of all parts and their ordre
	Map parts1 = message1.getParts();
	Map parts2 = message2.getParts();
	Collection partsCollection1 = parts1.values();
	Collection partsCollection2 = parts2.values();
	Iterator partsIterator1 = partsCollection1.iterator();
	Iterator partsIterator2 = partsCollection2.iterator();

	while(partsIterator1.hasNext())
	    {
		if (!partsIterator2.hasNext()) return false;
		Part part1 = (Part)partsIterator1.next();
		Part part2 = (Part)partsIterator2.next();
		if (!part1.getTypeName().equals(part2.getTypeName())) return false;
	    }
	if (partsIterator2.hasNext()) return false;
	return true;
    }

    public static boolean equals(Operation operation1, Operation operation2)
    {
	if (!operation1.getName().equals(operation2.getName())) return false;
	Input input1 = operation1.getInput();
	Input input2 = operation2.getInput();
	if (input1!=null) 
	    if (input2!=null)
		{
		    javax.wsdl.Message messageInput1= input1.getMessage();
		    javax.wsdl.Message messageInput2= input2.getMessage();
		    if (!equals(messageInput1, messageInput2)) return false;
		}
	    else return false;
	else if (input2!=null) return false;

	Output output1 = operation1.getOutput();
	Output output2 = operation2.getOutput();
	if (output1!=null) 
	    if (output2!=null)
		{
		    javax.wsdl.Message messageOutput1= output1.getMessage();
		    javax.wsdl.Message messageoutput2= output2.getMessage();
		    if (!equals(messageOutput1, messageoutput2)) return false;
		}
	    else return false;
	else if (output2!=null) return false;
	return true;
    }  

    public static boolean foundInto(Operation operation, List operations)
    {
	Iterator operationsIterator = operations.iterator();
	while (operationsIterator.hasNext())
	    {
		Operation operation2 = (Operation)operationsIterator.next();
		if (equals(operation, operation2)) return true;
	    }
	return false;
    }

    public static boolean compareOperations(List operations1, List operations2)
    {
	Iterator operationsIterator = operations1.iterator();
	while (operationsIterator.hasNext())
	    {
		Operation operation = (Operation)operationsIterator.next();
		if (!foundInto(operation, operations2)) return false; 
	    }
	return true;
    }

    public static boolean foundInto(PortType portType, Collection portTypesCollection)
    {
	Iterator portTypesIterator = portTypesCollection.iterator();
	while (portTypesIterator.hasNext())
	    {
		PortType portType2 = (PortType)portTypesIterator.next();
		List operations = portType.getOperations();
		List operations2 = portType2.getOperations();
		if (compareOperations(operations, operations2)) return true;
		  
	    }
	return false;
    }

    public static boolean compare(Definition definition1, Definition definition2)
    {
	if (!definition1.getQName().equals(definition2.getQName())) return false;
	Map portTypes1 = definition1.getPortTypes();
	Map portTypes2 = definition2.getPortTypes();
	Collection portTypesCollection1 = portTypes1.values();
	Collection portTypesCollection2 = portTypes2.values();
	Iterator portTypesIterator1 = portTypesCollection1.iterator();
	while (portTypesIterator1.hasNext())
	    {
		PortType portType1 = (PortType)portTypesIterator1.next();
		if (!foundInto(portType1, portTypesCollection2)) return false;
		    
	    }
	return true;
    }

    public static String[] getResume(String wsdlDocument)
    {
	// return a resume of this wsdlDocument
	// the resume contains somes string elements which are 
	// 1. serviceName
	// 2. serviceName+operationName for each operation
	// 3. serviceName+operationName+in+typeInParameter+out+typeOutParameter        //    for each operation
	// PB:we cannot distinguish between operations without any outputs ant 
	//    inputs parameters and operationName resume. 

	Vector resumes = new Vector();
	Definition def =WSDLTools.readWSDL(wsdlDocument);
	
	String serviceName = def.getQName().toString();
	resumes.add(serviceName);

	Map portTypes = def.getPortTypes();
	Collection portTypesCollection = portTypes.values();
	Iterator portTypesIterator = portTypesCollection.iterator();
	while (portTypesIterator.hasNext())
	    {
		PortType portType = (PortType)portTypesIterator.next();
		List operations = portType.getOperations();

		Iterator operationsIterator = operations.iterator();
		while (operationsIterator.hasNext())
		    {
			Operation operation = (Operation)operationsIterator.next();
			String operationName = operation.getName();
			resumes.add(serviceName+operationName);
			String typesList=""; // format instringoutint : input String and output Int
			Input input = operation.getInput();
			Output output = operation.getOutput();
			if (input!=null)
			    {
				javax.wsdl.Message messageInput= input.getMessage();
				Map parts = messageInput.getParts();
				Collection partsCollection = parts.values();
				Iterator partsIterator = partsCollection.iterator();
				if (partsIterator.hasNext()) typesList="in";
				while(partsIterator.hasNext())
				    {
					Part part = (Part)partsIterator.next();
					typesList = typesList+part.getTypeName();
				    }
			    }
			if (output!=null)
			    {
				javax.wsdl.Message messageOutput= output.getMessage();
				Map parts2 = messageOutput.getParts();
				Collection partsCollection2 = parts2.values();
				Iterator partsIterator2 = partsCollection2.iterator();
				if (partsIterator2.hasNext()) typesList=typesList+"out";
				while(partsIterator2.hasNext())
				    {
					Part part2 = (Part)partsIterator2.next();
					typesList = typesList+part2.getTypeName();
				    }
			    }
		resumes.add(serviceName+operationName+typesList);
 
		    }
		    
	    }
	String[] forReturn = new String[resumes.size()];
	for (int i=0; i<forReturn.length;i++) forReturn[i]=(String)resumes.elementAt(i);
	return forReturn; 
    }

} 
