package ubisec.net;

import java.net.*;
import java.io.*;
import ubisec.configuration.Setting;
import ubisec.util.system.IFUtils;

/**
*
* NetworkManager.java
*
* @author Rafik CHIBOUT
* @author Francoise SAILHAN
* @version 0.1 sept. 2004
*
*  
*/

public class NetworkManager 
{
    private DatagramSocket dataSocServer = null;
    private ServerSocket socServer       = null;
    private DatagramPacket dataPacket    = null;
    //private NetworkFilter networkFilter  = null;
     
    public NetworkManager()
    {
    	//networkFilter = new NetworkFilter();
    }
    
    public static String[] getLocalIPAddresses()
    {
    	IFUtils ifUtils = new IFUtils();
    	return ifUtils.getLocalHostAddresses();
    }

    public static String[] getBroadcastIPAddresses()
    {
    	return (new String[] {Setting.BroadcastAddress});
       	//return (new String[] {"255.255.255.255"});
    	//return (new String[] {"128.93.63.255"});
       	//return (new String[] {"192.168.255.255"});
    }

    public static boolean isGateway()
    {
	// true if ipforward actived and there are 2 LocalIPAddresses at least
	if (getLocalIPAddresses().length<2) return false;
	try
	    {
		File forwardFile = new File("/proc/sys/net/ipv4/ip_forward");
		if (forwardFile.exists())
		    {
			FileInputStream forwardFileIn = new FileInputStream(forwardFile);
            int forward = forwardFileIn.read();
			forwardFileIn.close();
 			String forwardStr = String.valueOf(forward);
			if (forwardStr !=null && forwardStr.equals("49")) return true;
			return false;  
		    }
		else return false;
	    }
	catch (Exception ex)
	    {
		System.out.println("Error at NetworkManager.isGateway()");
		return false;
	    }
    }

    public static void broadcast(byte[] message, int port)
    {
	DatagramSocket  dataSoc = null;
	try
	    {
		InetAddress address = InetAddress.getByName(getBroadcastIPAddresses()[0]);
		dataSoc = new DatagramSocket();
		dataSoc.setReuseAddress(true);
		if (message!=null)
		    dataSoc.send(new DatagramPacket(message, message.length, address, port)); 
		dataSoc.close();
	    }
	catch (Exception ex)
	    {
		System.out.println("Error at NetworkManager.Broadcast()");
		if (dataSoc!=null) dataSoc.close();
		ex.printStackTrace();
	    }
    }

    public static void sendTo(String message, String IPAddress, int port)
    {
	try
	    {
		InetAddress address = InetAddress.getByName(IPAddress);
		byte[] messageByte = message.getBytes();
		if (messageByte !=null)
		   NetworkManager.sendTo(messageByte, address, port);
	    }
	catch (Exception ex)
	    {
		System.out.println("Error at NetworkManager.sendTo()");
		ex.printStackTrace();
	    }
    }

    public static void sendTo(String message, InetAddress Address, int port)
    {
	try
	    {
		byte[] messageByte = message.getBytes();
		NetworkManager.sendTo(messageByte, Address, port);
	    }
	catch (Exception ex)
	    {
		System.out.println("Error at NetworkManager.sendTo()");
		ex.printStackTrace();
	    }
    }

    public static void sendTo(byte[] message, InetAddress Address, int port)
    {
	try
	    {
		Socket soc = new Socket(Address, port) ;
		soc.setReuseAddress(true);
		OutputStream outsoc = soc.getOutputStream();
		//long t1 = System.currentTimeMillis();
		if (message!=null)
		    outsoc.write(message);
		outsoc.flush();
		//long t2 = System.currentTimeMillis();
		//System.out.println("length of the sent message is "+message.length);
		//System.out.println("message sent in time="+(t2-t1));
		outsoc.close(); 
		soc.close();
	    }
	catch (Exception ex)
	    {
		System.out.println("Error at NetworkManager.sendTo()");
		ex.printStackTrace();
	    }
    }

   public static void sendUDPTo(byte[] message, InetAddress Address, int port)
    {
	try
	    {
		DatagramSocket dataSoc = new DatagramSocket();
		dataSoc.setReuseAddress(true);
		if (message!=null)
		    dataSoc.send(new DatagramPacket(message, message.length, Address, port)); 
		//System.out.println("length of the sent udp message is "+message.length);
	    }
	catch (Exception ex)
	    {
		System.out.println("Error at NetworkManager.sendTo()");
		ex.printStackTrace();
	    }
    }
   
    /* 
    public void startNeighborsDeamon(int port)
    {
	neighborsPort=port;
	Thread deamon = NeighborsDeamon();
	deamon.start();
    }
   
    public int getNeighbors(String message, int sendPort , int receivePort)
    {
	// start listener thread on port : port + 1
	cptNeighbors = 0;
	neighborsSendPort=sendPort;
	neighborsReceivePort=receivePort;

	Thread listenToResponse = listenToNeighborsResponse();
	listenToResponse.start();
	// sent broadcast message on port : port
	
	// wait
	try
	    {
		broadcast(message.getBytes(), neighborsSendPort);
		Thread.sleep(4000);
	    }
	catch (Exception ex)
	    {
		System.out.println("Error at NetworkManager.sleep()");
	    }
	return cptNeighbors;
    }
    
    public static int getNeighbors2(String message, int sendPort)
    {

	cptNeighbors = 0;
	neighborsSendPort=sendPort;
	// the listener reponse is already started by getNeighbor()
	
	// wait
	try
	    {
		broadcast(message.getBytes(), neighborsSendPort);
		Thread.sleep(4000);
	    }
	catch (Exception ex)
	    {
		System.out.println("Error at NetworkManager.sleep()");
	    }
	return cptNeighbors;
    }
    */
   
    public static byte[] getReceivedData(int port)
    {
	// socket server reception
	byte[] forReturn = new byte[1];
	try
	    {
		ServerSocket socServer = new ServerSocket(port);
		socServer.setReuseAddress(true);
		
		Socket soc = socServer.accept();
		//long t1 = System.currentTimeMillis();
		soc.setReuseAddress(true);
		InputStream socinput = soc.getInputStream();
		socinput.read(forReturn);
		int cp  = 1;
		int num = 0;
		//while (socinput.available()>0)
		while (socinput.available()>0 || num>0)
		    {
			//System.out.println("*******2The length is ="+socinput.available());
			//byte[] datatmp = new byte[socinput.available()];
			byte[] datatmp = new byte[1024];
			num = socinput.read(datatmp);
			//System.out.println("num="+num);
			cp = cp+num;
			if (num>0) forReturn = NetworkManager.concat(forReturn, datatmp, num);
		    }
		//long t2= System.currentTimeMillis();
		//System.out.println("message received in time="+(t2-t1));
		socinput.close();
		soc.close();
		socServer.close();
		return forReturn;
	    }
	catch(Exception ex)
	    {
		System.out.println("Error at NetworkManager.getReceivedData");
		ex.printStackTrace();
	    }
	return null;
    }

    /*
    public static byte[] getReceivedUDPData(int port) // it is not static because it need to keep all the received data for give it when this method is invoked
    {
	// thead needed
	DatagramSocket dataSocServer = null;
	try
	    {
		dataSocServer = new DatagramSocket(port);
		dataSocServer.setReuseAddress(true);

		DatagramPacket dataPacket = new DatagramPacket(new byte[dataSocServer.getReceiveBufferSize()], dataSocServer.getReceiveBufferSize());
		InetAddress address = null;
		while (address == null || address.getHostAddress().equals(NetworkManager.getLocalIPAddresses()[0])) 
		    {
			dataSocServer.receive(dataPacket);
			address = dataPacket.getAddress();
		    } 
		
		dataSocServer.close();
		return dataPacket.getData();
	    }
	catch (Exception ex)
	    {
		System.out.println("Error at NetworkManager.getReceivedUDPData()");
		if (dataSocServer != null) dataSocServer.close();
		ex.printStackTrace();
		return null;
	    }
    }
*/
    public DatagramPacket getReceivedUDPData2(int port) 
    {
	DatagramSocket dataSocServer = null;
	try
	    {
		boolean bool=false;
		dataSocServer = new DatagramSocket(port);
		dataSocServer.setReuseAddress(true);
		DatagramPacket dataPacket = new DatagramPacket(new byte[dataSocServer.getReceiveBufferSize()], dataSocServer.getReceiveBufferSize());
		while (!bool)
		{
		dataSocServer.receive(dataPacket);
		//if (!networkFilter.contains(dataPacket.getAddress().getHostAddress()))
			bool=true;
		//else System.out.println("NetworkManager:Address "+dataPacket.getAddress().getHostAddress()+" is filtred");
		}
		dataSocServer.close();
		return dataPacket;
	    }
		catch (Exception ex)
	    {
		System.out.println("Error at NetworkManager.getReceivedData2()");
		if (dataSocServer != null) dataSocServer.close();
		ex.printStackTrace();
		return null;
	    }
    }
  
    private static byte[] concat(byte[] dist , byte[] source, int num)
    {
	if (num > source.length) return dist;
	if (source !=null && dist!=null)
	    {
		byte[] result = new byte[dist.length+num];
		for (int i=0; i<dist.length; i++) result[i]= dist[i];
		for (int j=0; j<num; j++) result[j+dist.length]= source[j];
		return result;
	    }
	else return dist;
    }
    
    public boolean listenToPort(int port)
    {
	try
	    {
		socServer = new ServerSocket(port);
		socServer.setSoTimeout(4000);
		socServer.setReuseAddress(true);
		// UDP 
		dataSocServer = new DatagramSocket(port);
		dataSocServer.setReuseAddress(true);
		dataSocServer.setSoTimeout(4000);
		dataPacket = new DatagramPacket(new byte[dataSocServer.getReceiveBufferSize()], dataSocServer.getReceiveBufferSize());
		return true;
	    }
	catch(Exception ex)
	    {
		return false;
	    }
    }

    public byte[] getReceivedData()
    {
	byte[] forReturn = new byte[1];
	Socket soc = null;
	try
	    {
		//System.out.println("Waiting for data...");
		soc = socServer.accept();
	    }
		catch(Exception ex)
		{	
			return null;
		}
		try
		{
		//System.out.println("some data are received...");
		//long t1 = System.currentTimeMillis();
		InputStream socinput = soc.getInputStream();
		int num = socinput.read(forReturn);
		//System.out.println(forReturn.length);
		int cp  = 1;
		//int num = 0;
		
		//while(num!=-1)
		while (socinput.available()>0 || num>0)
		    {
			//System.out.println("*******2The length is ="+socinput.available());
			//byte[] datatmp = new byte[socinput.available()];
			byte[] datatmp = new byte[1024];
			num = socinput.read(datatmp);
			//System.out.println("num="+num);
			cp = cp+num;
			if (num>0) forReturn = NetworkManager.concat(forReturn, datatmp, num);
		    }
		//long t2= System.currentTimeMillis();
		//System.out.println("****message length="+forReturn.length);
		//System.out.println("****message received in time="+(t2-t1));
		//System.out.println("*******2end");
		socinput.close();
		soc.close();
		return forReturn;
	    }
	catch(Exception ex)
	    {
		System.out.println("Error at NetworkManager.getReceivedData()");
		ex.printStackTrace();
	    }
	return null;
    }

    public byte[] getReceivedUDPData() // it is not static because it need to keep all the received data for give it when this method is invoked
    {
	
		InetAddress address = null;
		boolean bool=false;
		while (address == null || address.getHostAddress().equals(NetworkManager.getLocalIPAddresses()[0]) || !bool) 
		    {
			bool=false;
			try
			{
			dataSocServer.receive(dataPacket);
			}
			catch (Exception ex)
		    {
				return null;
		    }
			address = dataPacket.getAddress();
			//if (!networkFilter.contains(address.getHostAddress())) 
			bool=true;
			//else System.out.println("NetworkManager:Address "+address.getHostAddress()+" is filtred");
			} 
		return dataPacket.getData();
	}
}
