package ubisec.profile;

import ubisec.net.*;
import ubisec.util.system.SystemInfo;
import ubisec.util.*;

/**
*
* NodeProfil.java
*
* @author Rafik CHIBOUT 
* @author Francoise SAILHAN
* @version 0.1 oct. 2004
*
*  
*/
public class NodeProfile
{
    IPAddress iPAddress = new IPAddress();
    IPAddress[] neighborsSD =  new IPAddress[0];
    IPAddress[] neighborsNotCoveredSD = new IPAddress[0];
    IPAddress[] neighborsDirectory = new IPAddress[0];
    int Neighbors;   float NeighborsNormalized;
    int SDNeighborsOfMyNeighbors;
    int NotCoveredSDNeighborsOfMyNeighbors;
    int DirNeighborsOfMyNeighbors;
    int Energy;      float EnergyNormalized; 
    int TotalMemory; float TotalMemoryNormalized;
    int FreeMemory;  float FreeMemoryNormalized;
    int CPUCapacity; float CPUCapacityNormalized;
    int CacheSize;   float CacheSizeNormalized;
    int Bandwith;    float BandwithNormalized;
    boolean Gateway;
    int idealRay = 1;
    int hopsNumberFromInitiator = 0;
    
    boolean automaticUpdate = false;
    int updateTime = 10000;
    String neighborMessage ="";
    int sendPort=0;
    static public NetworkCover networkCover = null;
    
    public NodeProfile(String messageType, int port)
    {
    	
    	networkCover = new NetworkCover(port, messageType);
    	// with neighbor message and port number parameters the local profil will be created
    	this.neighborMessage = neighborMessage;
    	this.sendPort = sendPort;
    	setIPAddress(NetworkManager.getLocalIPAddresses()[0]);
    	setNeighbors(networkCover.getNeighborsNumber());
    	setNeighborsSDAddresses(networkCover.getSDNeighborsAddresses());
	
    	setNeighborsNotCoveredSDAddresses(networkCover.getNotCoveredSDNeighborsAddresses());
    	setNeighborsDirectoriesAddresses(networkCover.getDirectoriesNeighborsAddresses());
    	setSDNeighborsOfMyNeighbors(networkCover.getSDNeighborsOfNeighborsNumber());
    	setNotCoveredSDNeighborsOfMyNeighbors(networkCover.getNotCoveredSDNeighborsOfNeighborsNumber());
    	setDirNeighborsOfMyNeighbors(networkCover.getDirNeighborsOfNeighborsNumber());
    	setEnergy(SystemInfo.getEnergy());
    	setTotalMemory(SystemInfo.getTotalMemory());
    	setFreeMemory(SystemInfo.getFreeMemory());
    	setCPUCapacity(SystemInfo.getCPUCapacity());
    	setCacheSize(SystemInfo.getCacheSize());
    	setGateway(NetworkManager.isGateway());
    	setIdealRay(networkCover.getHighNotCoveredSDLevel());
    	// 	start update profile thread
    	//Thread update = updateProcess();
    	//update.start();
    }

    public NodeProfile()
    {
	// without any parameters an empty profil will be created
    }

    public void stopCoverageDetection()
    {
    	networkCover.stop();
    }
    
    public void setMessageType(String messageType)
    {
    	networkCover.setMessageType(messageType);
    }

    public NodeProfile(byte[] profil)
    {
	try
	    {
		BitSet2[] bitsets = new BitSet2[4];
		bitsets[0]=new BitSet2(profil[0]);
		bitsets[1]=new BitSet2(profil[1]);
		bitsets[2]=new BitSet2(profil[2]);
		bitsets[3]=new BitSet2(profil[3]);
		IPAddress address = new IPAddress(bitsets);
		setIPAddress(address);
		// Directories IP Address
		bitsets = new BitSet2[4];
		bitsets[0]= new BitSet2(profil[4]);
		bitsets[1]= new BitSet2(profil[5]);
		bitsets[2]= new BitSet2(profil[6]);
		bitsets[3]= new BitSet2(profil[7]);
		BigBitSet bitSetLength = new BigBitSet(bitsets);
		int dirLength = bitSetLength.getInt();
		//System.out.println("dirLength="+dirLength);
		int position = 8;
		int i = 0;
		IPAddress[] dirIPAddress = new IPAddress[dirLength]; 
		for (i=0;i<dirLength;i++)
		    {
			bitsets = new BitSet2[4];
			bitsets[0]=new BitSet2(profil[position]);position++;
			bitsets[1]=new BitSet2(profil[position]);position++;
			bitsets[2]=new BitSet2(profil[position]);position++;
			bitsets[3]=new BitSet2(profil[position]);position++;
			dirIPAddress[i] = new IPAddress(bitsets);
		    }
		setNeighborsDirectoriesAddresses(dirIPAddress);
		// SD IP Address
		bitsets = new BitSet2[4];
		//System.out.println("position before sdLength="+position);
		bitsets[0]=new BitSet2(profil[position]);position++;
		bitsets[1]=new BitSet2(profil[position]);position++;
		bitsets[2]=new BitSet2(profil[position]);position++;
		bitsets[3]=new BitSet2(profil[position]);position++;

		bitSetLength = new BigBitSet(bitsets);
		int sdLength = bitSetLength.getInt();
		//System.out.println("sdLength="+sdLength);
	       	IPAddress[] sdIPAddress = new IPAddress[sdLength];
		for (i=0;i<sdLength;i++)
		    {
			bitsets = new BitSet2[4];
			bitsets[0]=new BitSet2(profil[position]);position++;
			bitsets[1]=new BitSet2(profil[position]);position++;
			bitsets[2]=new BitSet2(profil[position]);position++;
			bitsets[3]=new BitSet2(profil[position]);position++;
			sdIPAddress[i] = new IPAddress(bitsets);
		    }
		setNeighborsSDAddresses(sdIPAddress);
		
        //	Not covered SD IP Address
		bitsets = new BitSet2[4];
		//System.out.println("position before notCoveredSDLength="+position);
		bitsets[0]=new BitSet2(profil[position]);position++;
		bitsets[1]=new BitSet2(profil[position]);position++;
		bitsets[2]=new BitSet2(profil[position]);position++;
		bitsets[3]=new BitSet2(profil[position]);position++;

		bitSetLength = new BigBitSet(bitsets);
		int notCoveredSDLength = bitSetLength.getInt();
		//System.out.println("sdLength="+sdLength);
	       	IPAddress[] notCoveredSDIPAddress = new IPAddress[notCoveredSDLength];
		for (i=0;i<notCoveredSDLength;i++)
		    {
			bitsets = new BitSet2[4];
			bitsets[0]=new BitSet2(profil[position]);position++;
			bitsets[1]=new BitSet2(profil[position]);position++;
			bitsets[2]=new BitSet2(profil[position]);position++;
			bitsets[3]=new BitSet2(profil[position]);position++;
			notCoveredSDIPAddress[i] = new IPAddress(bitsets);
		    }
		setNeighborsNotCoveredSDAddresses(notCoveredSDIPAddress);
		
		
		String strProfil = new String(profil, 4+(4*(sdLength+notCoveredSDLength+dirLength+3)), profil.length-(4+(4*(sdLength+notCoveredSDLength+dirLength+3))));
		int beginIndex=1;
		int endIndex = strProfil.indexOf("#",beginIndex);
		String str = strProfil.substring(beginIndex, endIndex);
		Energy = (new Integer(str)).intValue();
		
		beginIndex=endIndex+1;
		endIndex = strProfil.indexOf("#",beginIndex);
		str = strProfil.substring(beginIndex, endIndex);
		Neighbors = (new Integer(str)).intValue();
		
		beginIndex=endIndex+1;
		endIndex = strProfil.indexOf("#",beginIndex);
		str = strProfil.substring(beginIndex, endIndex);
		SDNeighborsOfMyNeighbors = (new Integer(str)).intValue();
		
		beginIndex=endIndex+1;
		endIndex = strProfil.indexOf("#",beginIndex);
		str = strProfil.substring(beginIndex, endIndex);
		NotCoveredSDNeighborsOfMyNeighbors = (new Integer(str)).intValue();
		
		beginIndex=endIndex+1;
		endIndex = strProfil.indexOf("#",beginIndex);
		str = strProfil.substring(beginIndex, endIndex);
		DirNeighborsOfMyNeighbors = (new Integer(str)).intValue();
		
		beginIndex=endIndex+1;
		endIndex = strProfil.indexOf("#",beginIndex);
		str = strProfil.substring(beginIndex, endIndex);
		TotalMemory = (new Integer(str)).intValue();
		
		beginIndex=endIndex+1;
		endIndex = strProfil.indexOf("#",beginIndex);
		str = strProfil.substring(beginIndex, endIndex);
		FreeMemory = (new Integer(str)).intValue();
		
		beginIndex=endIndex+1;
		endIndex = strProfil.indexOf("#",beginIndex);
		str = strProfil.substring(beginIndex, endIndex);
		CPUCapacity = (new Integer(str)).intValue();
		
		beginIndex=endIndex+1;
		endIndex = strProfil.indexOf("#",beginIndex);
		str = strProfil.substring(beginIndex, endIndex);
		CacheSize = (new Integer(str)).intValue();
		
		beginIndex=endIndex+1;
		endIndex = strProfil.indexOf("#",beginIndex);
		str = strProfil.substring(beginIndex, endIndex);
		Bandwith = (new Integer(str)).intValue();
		
		beginIndex=endIndex+1;
		endIndex = strProfil.indexOf("#",beginIndex);
		str = strProfil.substring(beginIndex, endIndex);
		idealRay = (new Integer(str)).intValue();
		
		beginIndex=endIndex+1;
		endIndex = strProfil.indexOf("#",beginIndex);
		str = strProfil.substring(beginIndex, endIndex);
		hopsNumberFromInitiator = (new Integer(str)).intValue();
		
		beginIndex=endIndex+1;
		str = strProfil.substring(beginIndex);
		str = str.trim();
		if (str.equals("false")) setGateway(false);
		else setGateway(true);
		
		
	    }
	catch(Exception ex)
	    {
		System.out.println("NodeProfile: Erroneous Profile");
	    }
    }

    public void setAutomaticUpdate(boolean automaticUpdate)
    {
	this.automaticUpdate = automaticUpdate;
    }

    public boolean isAutomaticUpdate()
    {
	return automaticUpdate;
    }

    public void setIPAddress(IPAddress iPAddress)
    {
	this.iPAddress = iPAddress;
    }

    public void setIPAddress(String iPAddress)
    {
	this.iPAddress.setIPAddress(iPAddress);
    }

    public void setEnergy(int Energy)
    {
	this.Energy=Energy;
    }

    public void setNeighbors(int Neighbors)
    {
	this.Neighbors=Neighbors;
    }

    public void setNeighborsSDAddresses(IPAddress[] SDAddresses)
    {
	neighborsSD = SDAddresses;
    }

    public void setNeighborsNotCoveredSDAddresses(IPAddress[] NotCoveredSDAddresses)
    {
	neighborsNotCoveredSD = NotCoveredSDAddresses;
    }

    public void setNeighborsSDAddresses(String[] SDAddresses)
    {
    	if (SDAddresses == null) return;
	neighborsSD = new IPAddress[SDAddresses.length];
	for (int i=0;i<neighborsSD.length;i++)
	    neighborsSD[i] = new IPAddress(SDAddresses[i]);
    }
   
    public void setNeighborsNotCoveredSDAddresses(String[] NotCoveredSDAddresses)
    {
    	if (NotCoveredSDAddresses == null) System.out.println("Profile: NotCoveredSDAddresses is null");
	neighborsNotCoveredSD = new IPAddress[NotCoveredSDAddresses.length];
	for (int i=0;i<neighborsNotCoveredSD.length;i++)
	    neighborsNotCoveredSD[i] = new IPAddress(NotCoveredSDAddresses[i]);
    }
    
    public void setNeighborsDirectoriesAddresses(IPAddress[] DirectoriesAddresses)
    {
	neighborsDirectory = DirectoriesAddresses;
    }

    public void setNeighborsDirectoriesAddresses(String[] DirectoriesAddresses)
    {
	neighborsDirectory = new IPAddress[DirectoriesAddresses.length];
	for (int i=0;i<neighborsDirectory.length;i++)
	    neighborsDirectory[i] = new IPAddress(DirectoriesAddresses[i]);
    }

    public void setSDNeighborsOfMyNeighbors(int sdNumber)
    {
    	SDNeighborsOfMyNeighbors=sdNumber;
    }
    
    public void setNotCoveredSDNeighborsOfMyNeighbors(int notCoveredSDNumber)
    {
    	NotCoveredSDNeighborsOfMyNeighbors=notCoveredSDNumber;
    }
    
    public void setDirNeighborsOfMyNeighbors(int dirNumber)
    {
    	DirNeighborsOfMyNeighbors=dirNumber;
    }
    
    public void setTotalMemory(int TotalMemory)
    {
	//if (TotalMemory > 10000) this.TotalMemory = TotalMemory/1000;
	//	else
	this.TotalMemory=TotalMemory;
    }

    public void setFreeMemory(int FreeMemory)
    {
	//if (FreeMemory > 1000) this.FreeMemory = FreeMemory/1000;
	//else 
	this.FreeMemory=FreeMemory;
    }

    public void setCPUCapacity(int CPUCapacity)
    {
	this.CPUCapacity=CPUCapacity;
    }

    public void setCacheSize(int CacheSize)
    {
	this.CacheSize=CacheSize;
    }

    public void setBandwith(int Bandwith)
    {
	this.Bandwith=Bandwith;
    }

    public void setGateway(boolean Gateway)
    {
	this.Gateway=Gateway;
    }

    public void setIdealRay(int ray)
    {
    	if (ray >0) idealRay = ray;
    }
    
    public void setHopsNumberFromInitiatir(int hops)
    {
    	hopsNumberFromInitiator = hops;
    }
        
    public String getIPAddress()
    {
	return iPAddress.getHostIPAddress();
    }

    public int getEnergy()
    {
	return Energy;
    }

    public int getNeighbors()
    {
	return Neighbors;
    }

    public IPAddress[] getNeighborsSDAddresses()
    {
	return neighborsSD;
    }
    
    public IPAddress[] getNeighborsNotCoveredSDAddresses()
    {
	return neighborsNotCoveredSD;
    }
    
    public IPAddress[] getNeighborsDirectoriesAddresses()
    {
	return neighborsDirectory;
    }

    public int getSDNeighborsOfMyNeighbors()
    {
    	return SDNeighborsOfMyNeighbors;
    }
    
    public int getNotCoveredSDNeighborsOfMyNeighbors()
    {
    	return NotCoveredSDNeighborsOfMyNeighbors;
    }
    
    public int getDirNeighborsOfMyNeighbors()
    {
    	return DirNeighborsOfMyNeighbors;
    }
    
    public int getTotalMemory()
    {
	return TotalMemory;
    }

    public int getFreeMemory()
    {
	return FreeMemory;
    }

    public int getCPUCapacity()
    {
	return CPUCapacity;
    }

    public int getCacheSize()
    {
	return CacheSize;
    }

    public int getBandwith()
    {
    	return Bandwith;
    }

    public boolean isGateway()
    {
    	return Gateway;
    }

    public int getIdealRay()
    {
    	return idealRay;
    }
    
    public int getHopsNumberFromInitiator()
    {
    	return hopsNumberFromInitiator;
    }
    
    public void setUpdateTime(int updateTime)
    {
    	if (updateTime>=0 && updateTime<=999999) this.updateTime = updateTime;
    }

    public void profileUpdate()
    {
    setIPAddress(NetworkManager.getLocalIPAddresses()[0]);
    setNeighbors(networkCover.getNeighborsNumber());
	setNeighborsSDAddresses(networkCover.getSDNeighborsAddresses());
	setNeighborsNotCoveredSDAddresses(networkCover.getNotCoveredSDNeighborsAddresses());
	setNeighborsDirectoriesAddresses(networkCover.getDirectoriesNeighborsAddresses());
	setSDNeighborsOfMyNeighbors(networkCover.getSDNeighborsOfNeighborsNumber());
	setNotCoveredSDNeighborsOfMyNeighbors(networkCover.getNotCoveredSDNeighborsOfNeighborsNumber());
	setDirNeighborsOfMyNeighbors(networkCover.getDirNeighborsOfNeighborsNumber());
	setEnergy(SystemInfo.getEnergy());
	setFreeMemory(SystemInfo.getFreeMemory());
	setGateway(NetworkManager.isGateway());
	setIdealRay(networkCover.getHighNotCoveredSDLevel());
    }

    public byte[] getBytes()
    {
    //profileUpdate();
	String profile = "#"+Energy+"#"+Neighbors+"#"+SDNeighborsOfMyNeighbors+"#"+NotCoveredSDNeighborsOfMyNeighbors+"#"+DirNeighborsOfMyNeighbors+"#"+TotalMemory+"#"+FreeMemory+"#"+CPUCapacity+"#"+CacheSize+"#"+Bandwith+"#"+idealRay+"#"+hopsNumberFromInitiator+"#"+Gateway;
	byte[] strProfile = profile.getBytes();
	byte[] forReturn = new byte[4+strProfile.length+(4*(neighborsSD.length+neighborsNotCoveredSD.length+neighborsDirectory.length+3))];
	// Local IP Address
	BitSet2[] bitsets = iPAddress.getIPAddress();
	forReturn[0] = bitsets[0].getByte();
	forReturn[1] = bitsets[1].getByte();
	forReturn[2] = bitsets[2].getByte();
	forReturn[3] = bitsets[3].getByte();
	// Directories IP Address
	BigBitSet bigBitSet = null;
	if (neighborsDirectory == null) bigBitSet = new BigBitSet(0);
	else bigBitSet = new BigBitSet(neighborsDirectory.length);
	bitsets = bigBitSet.getBitSet();
	//System.out.println("neighborsDirectory number bigBitSet.getInt()="+bigBitSet.getInt());
	forReturn[4] = bitsets[0].getByte();
	forReturn[5] = bitsets[1].getByte();
	forReturn[6] = bitsets[2].getByte();
	forReturn[7] = bitsets[3].getByte();
	int position = 8;
	int i=0;
	if (neighborsDirectory != null)
	for (i=0; i<neighborsDirectory.length;i++)
	    {
		bitsets = neighborsDirectory[i].getIPAddress();
		forReturn[position] = bitsets[0].getByte();position++;
		forReturn[position] = bitsets[1].getByte();position++;
		forReturn[position] = bitsets[2].getByte();position++;
		forReturn[position] = bitsets[3].getByte();position++;
	    }

	// SD IP Address
	if (neighborsSD == null) bigBitSet = new BigBitSet(0);
	else bigBitSet = new BigBitSet(neighborsSD.length);
	//System.out.println("neighborsSD number bigBitSet.getInt()="+bigBitSet.getInt());
	bitsets = bigBitSet.getBitSet();
	forReturn[position] = bitsets[0].getByte();position++;
	forReturn[position] = bitsets[1].getByte();position++;
	forReturn[position] = bitsets[2].getByte();position++;
	forReturn[position] = bitsets[3].getByte();position++;
	if (neighborsSD != null)
	for (i=0; i<neighborsSD.length;i++)
	    {
		bitsets = neighborsSD[i].getIPAddress();
		forReturn[position] = bitsets[0].getByte();position++;
		forReturn[position] = bitsets[1].getByte();position++;
		forReturn[position] = bitsets[2].getByte();position++;
		forReturn[position] = bitsets[3].getByte();position++;
	    }
//	 not Covered SD IP Address
	if (neighborsNotCoveredSD == null) bigBitSet = new BigBitSet(0);
	else bigBitSet = new BigBitSet(neighborsNotCoveredSD.length);
	//System.out.println("neighborsNotCoveredSD number bigBitSet.getInt()="+bigBitSet.getInt());
	bitsets = bigBitSet.getBitSet();
	forReturn[position] = bitsets[0].getByte();position++;
	forReturn[position] = bitsets[1].getByte();position++;
	forReturn[position] = bitsets[2].getByte();position++;
	forReturn[position] = bitsets[3].getByte();position++;
	if (neighborsNotCoveredSD != null)
	for (i=0; i<neighborsNotCoveredSD.length;i++)
	    {
		bitsets = neighborsNotCoveredSD[i].getIPAddress();
		forReturn[position] = bitsets[0].getByte();position++;
		forReturn[position] = bitsets[1].getByte();position++;
		forReturn[position] = bitsets[2].getByte();position++;
		forReturn[position] = bitsets[3].getByte();position++;
	    }
	
	for (i=0; i<strProfile.length;i++)
	    forReturn[position+i] = strProfile[i];
	return forReturn;
    }

    public String getXMLProfile()
    {
	StringBuffer XMLProfil = new StringBuffer();
	// add XML Document version 1.0
	XMLProfil = XMLProfil.append("<?xml version=");
	XMLProfil = XMLProfil.append('"');
	XMLProfil = XMLProfil.append("1.0");
	XMLProfil = XMLProfil.append('"');
	XMLProfil = XMLProfil.append(" encoding=");
	XMLProfil = XMLProfil.append('"');
	XMLProfil = XMLProfil.append("UTF-8");
	XMLProfil = XMLProfil.append('"');
	XMLProfil = XMLProfil.append("?>");
	// Profil Part
	XMLProfil = XMLProfil.append("<NodeProfil>");
	XMLProfil = XMLProfil.append("<IPAddress>"+iPAddress.getHostIPAddress()+"</IPAddress>");
	XMLProfil = XMLProfil.append("<Energy>"+Energy+"</Energy>");
	XMLProfil = XMLProfil.append("<Neighbors>"+Neighbors+"</Neighbors>");
	if (neighborsDirectory ==null || neighborsDirectory.length==0) 
	    XMLProfil = XMLProfil.append("<DirNeighbors/>");
	else 
	    {
		XMLProfil = XMLProfil.append("<DirNeighbors>");
		for (int i=0; i<neighborsDirectory.length ; i++)
		    XMLProfil = XMLProfil.append("<IP>"+neighborsDirectory[i].getHostIPAddress()+"</IP>");
		XMLProfil = XMLProfil.append("</DirNeighbors>");	
	    }

	if (neighborsSD ==null || neighborsSD.length==0) 
	    XMLProfil = XMLProfil.append("<SDNeighbors/>");
	else 
	    {
		XMLProfil = XMLProfil.append("<SDNeighbors>");
		for (int i=0; i<neighborsSD.length ; i++)
		    XMLProfil = XMLProfil.append("<IP>"+neighborsSD[i].getHostIPAddress()+"</IP>");
		XMLProfil = XMLProfil.append("</SDNeighbors>");	
	    }
	if (neighborsNotCoveredSD ==null || neighborsNotCoveredSD.length==0) 
	    XMLProfil = XMLProfil.append("<NotCoveredSDNeighbors/>");
	else 
	    {
		XMLProfil = XMLProfil.append("<NotCoveredSDNeighbors>");
		for (int i=0; i<neighborsNotCoveredSD.length ; i++)
		    XMLProfil = XMLProfil.append("<IP>"+neighborsNotCoveredSD[i].getHostIPAddress()+"</IP>");
		XMLProfil = XMLProfil.append("</NotCoveredSDNeighbors>");	
	    }
	
	XMLProfil = XMLProfil.append("<DirNeighborsOfMyNeighbors>"+DirNeighborsOfMyNeighbors+"</DirNeighborsOfMyNeighbors>");
	XMLProfil = XMLProfil.append("<SDNeighborsOfMyNeighbors>"+SDNeighborsOfMyNeighbors+"</SDNeighborsOfMyNeighbors>");
	XMLProfil = XMLProfil.append("<NotCoveredSDNeighborsOfMyNeighbors>"+NotCoveredSDNeighborsOfMyNeighbors+"</NotCoveredSDNeighborsOfMyNeighbors>");
	XMLProfil = XMLProfil.append("<TotalMemory>"+TotalMemory+"</TotalMemory>");
	XMLProfil = XMLProfil.append("<FreeMemory>"+FreeMemory+"</FreeMemory>");
	XMLProfil = XMLProfil.append("<CPUCapacity>"+CPUCapacity+"</CPUCapacity>");
	XMLProfil = XMLProfil.append("<CacheSize>"+CacheSize+"</CacheSize>");
	XMLProfil = XMLProfil.append("<Bandwith>"+Bandwith+"</Bandwith>");
	XMLProfil = XMLProfil.append("<Gateway>"+Gateway+"</Gateway>");
	XMLProfil = XMLProfil.append("</NodeProfil>");
	return XMLProfil.toString();
    }
    

    public float getBenefit()
    {
	float benefit = ((Energy/Weight.StandardEnergy)*Weight.Energy)+ 
	    ((Neighbors/Weight.StandardNeighbors)*Weight.Neighbors) + 
	    ((TotalMemory/Weight.StandardTotalMemory)*Weight.TotalMemory)+ 
	    ((FreeMemory/Weight.StandardFreeMemory)*Weight.FreeMemory)+ 
	    ((CPUCapacity/Weight.StandardCPUCapacity)*Weight.CPUCapacity)+ 
	    ((CacheSize/Weight.StandardCacheSize)*Weight.CacheSize)+ 
	    ((Bandwith/Weight.StandardBandwith)*Weight.Bandwith); 
	//normalization();
	return benefit;
    }

    public Thread updateProcess() {
        try
	    {
		Thread updateProcessThread = 
		    new Thread( new Runnable() 
			{
			    public void run() 
			    {
				while (automaticUpdate)
				    {
					try
					    {
						Thread.sleep(updateTime);
						profileUpdate();
					    }
					catch (Exception ex)
					    {
						System.out.println("Error at NodeProfil.updateProcess.run()");
					    }
				    }
			    }
			});
		return updateProcessThread;
	    }
	catch (Exception ex)
	    {
		System.out.println("Error at create NodeProfil.updateProcess Thread");
	    }
	return null;
  }

    /*
    private void normalization()
    {
	System.out.println("normalization");
	String tmp = (new Integer(Energy)).toString();
	while (tmp.length()<16) tmp=" "+tmp;
	System.out.println("Energy     ="+tmp);

	tmp = (new Integer(Neighbors)).toString();
	while (tmp.length()<16) tmp=" "+tmp;
	System.out.println("Neighbors  ="+tmp);

	tmp = (new Integer(TotalMemory)).toString();
	while (tmp.length()<16) tmp=" "+tmp;
	System.out.println("TotalMemory="+tmp);

	tmp = (new Integer(FreeMemory)).toString();
	while (tmp.length()<16) tmp=" "+tmp;
	System.out.println("FreeMemory ="+tmp);

	tmp = (new Integer(CPUCapacity)).toString();
	while (tmp.length()<16) tmp=" "+tmp;
	System.out.println("CPUCapacity="+tmp);

	tmp = (new Integer(CacheSize)).toString();
	while (tmp.length()<16) tmp=" "+tmp;
	System.out.println("CacheSize  ="+tmp);

	tmp = (new Integer(Bandwith)).toString();
	while (tmp.length()<16) tmp=" "+tmp;
	System.out.println("Bandwith   ="+tmp);
	System.out.println("-------------------------------------------");
	
	
	int notZero =0;
	if (Energy>0) notZero++;
	if (Neighbors>0) notZero++;
	if (TotalMemory>0) notZero++;
	if (FreeMemory>0) notZero++;
	if (CPUCapacity>0) notZero++;
	if (CacheSize>0) notZero++;
	if (Bandwith>0) notZero++;
	
	if (notZero == 0) return;

	int[] values = new int[notZero];
	int i=0;
 	if (Energy>0) values[i++]=Energy;
	if (Neighbors>0) values[i++]=Neighbors;
	if (TotalMemory>0) values[i++]=TotalMemory;
	if (FreeMemory>0) values[i++]=FreeMemory;
	if (CPUCapacity>0) values[i++]=CPUCapacity;
	if (CacheSize>0) values[i++]=CacheSize;
	if (Bandwith>0) values[i++]=Bandwith;
	
	int total=0;
	for (i=0; i<values.length;i++) total= total+values[i];
		tmp = (new Integer(total)).toString();
	while (tmp.length()<16) tmp=" "+tmp;
	System.out.println("Total      ="+tmp);
System.out.println("");
System.out.println("");
System.out.println("*******************************");
	float md = total/notZero;

	System.out.println("");
System.out.println("Level 1 md="+md);
System.out.println("");
System.out.println("*******************************");




System.out.println("Power.power(Energy -md,2)      ="+Power.power(Energy -md,2));
System.out.println("Power.power(Neighbors -md,2)   ="+Power.power(Neighbors -md,2));
System.out.println("Power.power(TotalMemory -md,2) ="+Power.power(TotalMemory -md,2));
System.out.println("Power.power(FreeMemory -md,2)  ="+Power.power(FreeMemory -md,2));

System.out.println("Power.power(CPUCapacity -md, 2)="+Power.power(CPUCapacity -md, 2));
System.out.println("Power.power(CacheSize -md, 2)  ="+Power.power(CacheSize -md, 2));
System.out.println("Power.power(Bandwith -md, 2)   ="+Power.power(Bandwith -md, 2));

//float deviation2 = (Power.power(Energy -md,2)/7)+ (Power.power(Neighbors -md,2)/7)+ (Power.power(TotalMemory -md,2)/7)+ (Power.power(FreeMemory -md, 2)/7)+ (Power.power(CPUCapacity -md, 2)/7)+ (Power.power(CacheSize -md, 2)/7)+ (Power.power(Bandwith -md, 2)/7);
float deviation2=0;
for (i = 0; i< values.length;i++) deviation2=deviation2+(Power.power(values[i] -md,2)/notZero);

	System.out.println("Level 2 deviation2="+deviation2);
	Float deviationF = new Float(deviation2);
System.out.println("Level 3 deviationF.intValue()="+deviationF.intValue());
System.out.println("Level 3 deviationF.floatValue()="+deviationF.floatValue());
String deviationStr = deviationF.toString();
String deviationIntStr = deviationStr.substring(0, deviationStr.indexOf("."));
Integer deviationInt = new Integer(deviationIntStr);
System.out.println("Level 3 deviationInt.intValue()="+deviationInt.intValue());
	float deviation = Root.root(deviationInt.intValue(), Float.parseFloat(".0001"));
	//float deviation = Root.root2(deviation2, Float.parseFloat(".0001"));
	System.out.println("Level 4 deviation="+deviation);
	if (Energy == 0 ) EnergyNormalized =Float.parseFloat("0.0");
	else if ((Energy -md) > (2*deviation)) EnergyNormalized =Float.parseFloat("2.0");

	else if ((Energy -md) < (-2*deviation))  EnergyNormalized =Float.parseFloat(".0");
	else  EnergyNormalized = (Energy - md)/(2*deviation)+1;
	
	if (Neighbors == 0) NeighborsNormalized =Float.parseFloat(".0");
	else if ((Neighbors-md) > (2*deviation)) NeighborsNormalized =Float.parseFloat("2.0");
	else if ((Neighbors -md) < (-2*deviation)) NeighborsNormalized =Float.parseFloat(".0");
	else NeighborsNormalized = (Neighbors - md)/(2*deviation)+1;
	
	if (TotalMemory==0) TotalMemoryNormalized =Float.parseFloat(".0");
	else if ((TotalMemory-md) > (2*deviation)) TotalMemoryNormalized =Float.parseFloat("2.0");
	else if ((TotalMemory -md) < (-2*deviation)) TotalMemoryNormalized =Float.parseFloat(".0");
	else TotalMemoryNormalized = (TotalMemory - md)/(2*deviation)+1;

	if (FreeMemory==0)  FreeMemoryNormalized =Float.parseFloat(".0");
	else if ((FreeMemory-md) > (2*deviation)) FreeMemoryNormalized =Float.parseFloat("2.0");
	else if ((FreeMemory -md) < (-2*deviation)) FreeMemoryNormalized =Float.parseFloat(".0");
	else FreeMemoryNormalized = (FreeMemory - md)/(2*deviation)+1;

	if (CPUCapacity==0) CPUCapacityNormalized =Float.parseFloat(".0");
	else if ((CPUCapacity-md) > (2*deviation)) CPUCapacityNormalized =Float.parseFloat("2.0");
	else if ((CPUCapacity -md) < (-2*deviation)) CPUCapacityNormalized =Float.parseFloat(".0");
	else CPUCapacityNormalized = (CPUCapacity - md)/(2*deviation)+1;

	if (CacheSize ==0 ) CacheSizeNormalized =Float.parseFloat(".0");
	else if ((CacheSize-md) > (2*deviation)) CacheSizeNormalized =Float.parseFloat("2.0");
	else if ((CacheSize -md) < (-2*deviation)) CacheSizeNormalized =Float.parseFloat(".0");
	else CacheSizeNormalized = (CacheSize - md)/(2*deviation)+1;

	if (Bandwith==0) BandwithNormalized =Float.parseFloat(".0");
	else if ((Bandwith-md) > (2*deviation)) BandwithNormalized =Float.parseFloat("2.0");
	else if ((Bandwith -md) < (-2*deviation)) BandwithNormalized =Float.parseFloat(".0");
	else BandwithNormalized = (Bandwith - md)/(2*deviation)+1;





	System.out.println(" Normalized data :");
	System.out.println("    - Energy="+EnergyNormalized);
	System.out.println("    - Neighbor="+NeighborsNormalized);
	System.out.println("    - TotalMemory="+TotalMemoryNormalized);
	System.out.println("    - FreeMemory="+FreeMemoryNormalized);
	System.out.println("    - CPUCapacity="+CPUCapacityNormalized);
	System.out.println("    - CacheSize="+CacheSizeNormalized);
	System.out.println("    - Bandwith="+BandwithNormalized);

    }
    */
}

