package ubisec.util;

import java.util.BitSet;

/**
*
* BitSet2.java
*
* @author Rafik CHIBOUT 
* @author Francoise SAILHAN
* @version 0.1 oct. 2004
*
*  
*/
public class BitSet2 extends BitSet
{
    int size = 8;
    byte[] codeBytes = new byte[256];
    
    public BitSet2()
    {
	super(8);
	createTranslation();
    }

    public BitSet2(int nbits)
    {
	super(nbits);
	size = nbits;
	createTranslation();
    }

    public BitSet2(BitSet bitSet , int offset, int length)
    {
	super(length);
	size = length;
	for (int i=0;i<length;i++)
	    if (bitSet.get(i+offset)) this.set(i);
	createTranslation();
    }

    public BitSet2(byte codeByte)
    {
	super(8);
	createTranslation();
	int i=0;
	while (i<256 && codeBytes[i]!=codeByte) i++;
	if (i<256) 
	    {
		for (int j=0;j<i;j++) increment();
	    }
    }

    public BitSet2(byte[] codeBytes)
    {
	super(8*codeBytes.length);
	size = 8*codeBytes.length;
	createTranslation();
	for (int j=0;j<codeBytes.length;j++)
	    {
		int i=0;
		while (i<256 && this.codeBytes[i]!=codeBytes[j]) i++;
		if (i<256) 
		    {
			BitSet2 tmp = new BitSet2(8);
			for (int k=0;k<i;k++) tmp.increment();
			for (int l=0;l<8;l++)
			    if (tmp.get(l)) this.set(j*8+l);
		    }
	    }
    }


    public int size()
    {
	return size;
    }

    public BitSet2 get2(int fromIndex, int toIndex)
    {
	if ( (toIndex<fromIndex) || (fromIndex<0) || (toIndex>size) ) return this; 
	BitSet2 forReturn = new BitSet2(toIndex-fromIndex);
	for (int i=fromIndex; i<toIndex;i++)
	    if (get(i)) forReturn.set(i-fromIndex);
	return forReturn;
    }

    private void createTranslation()
    {
	for (int i=0;i<256;i++)
		codeBytes[i]=(new Integer(i)).byteValue();
    }

    public boolean equals(Object obj)
    {
	BitSet2 tmp = (BitSet2)obj;
	for (int i=0;i<8;i++)
	    if (tmp.get(i)!=get(i)) return false;
	return true;
    } 

    public BitSet2 increment()
    {
	int i= super.nextClearBit(0); 
	if (i!=-1) 
	    {
		super.flip(0,i);
		super.set(i);
	    }
	else super.clear();
	return this;
    }

    public BitSet2 decrement()
    {
	int i= super.nextSetBit(0); 
	if (i!=-1) 
	    {
		super.flip(0,i);
		super.clear(i);
	    }
	return this;
    }

    public void shiftRight()
    {
	int i=0;
	for (i=size()-1; i>0; i--)
	    {
		if (super.get(i-1)) super.set(i);
		else super.clear(i);
	    }
	super.clear(i);
    }

    public void shiftLeft()
    {
	int i=0;
	for (i=0; i<size()-1; i++)
	    {
		if (super.get(i+1)) super.set(i);
		else super.clear(i);
	    }
	super.clear(i);
    }

    public BitSet2 concatAtLast(BitSet2 bitset)
    {
	if ((size+bitset.size())>8) 
	    {
		System.out.println("The BitSet result size not supported use java.util.BitSet class");
		return this;
	    }

	BitSet2 newBitSet = new BitSet2(size()+bitset.size());
	for (int i=0; i<size() ; i++) if (get(i)) newBitSet.set(i);
	for (int j=0; j<bitset.size(); j++) if (bitset.get(j)) newBitSet.set(j+size());  
	return newBitSet;
    }

    public BitSet2 concatAtFirst(BitSet2 bitset)
    {
	if ((size+bitset.size())>8) 
	    {
		System.out.println("The BitSet result size not supported use java.util.BitSet class");
		return this;
	    }
	BitSet2 newBitSet = new BitSet2(size()+bitset.size());
	for (int j=0; j<bitset.size(); j++) if (bitset.get(j)) newBitSet.set(j); 
	for (int i=0; i<size() ; i++) if (super.get(i)) newBitSet.set(i+bitset.size());
	return newBitSet;
    }

    public String bitsSequence(int nbits)
    {
	String forReturn ="";
	if (nbits>size) nbits=size;
	for (int j=0;j<nbits; j++)
	    if (get(j)) forReturn=forReturn+"1";
	    else forReturn=forReturn+"0";
	return forReturn;
    }

    public int getInt()
    {
	int forReturn =0;
	int coef = 1;
	for (int j=0;j<size; j++)
	    {
		if (get(j)) forReturn=forReturn+coef;
		coef = coef*2;
	    }
	return forReturn;
    }

    public byte getByte()
    {
	if (getInt()<255) return codeBytes[getInt()];
	else return -1;
    }

    public byte[] getBytes()
    {
	if ((size%8) !=0) return new byte[0];
	byte[] forReturn = new byte[size/8];
	for (int i=0; i<forReturn.length;i++)
	    {
		BitSet2 tmp = new BitSet2(this,i*8,8);
		forReturn[i]=tmp.getByte();
	    }
	return forReturn;
    }
}
